! $Id: Direct2D.f90 2148 2016-05-29 23:41:36Z ian $
! Copyright 2016 M.E.G.M.S.  See LICENCE.txt for licence (Apache 2.0).
! See http://www.megms.com.au/winapi_bindings.htm for more information.



!*******************************************************************************

MODULE Direct2D
  
  USE :: COMBase
  USE :: DirectX, ONLY: DXGI_FORMAT, DXGI_FORMAT_UNKNOWN
  
  USE, INTRINSIC :: ISO_C_BINDING, ONLY:  &
      C_PTR,  &
      C_FUNPTR,  &
      C_LONG,  &
      C_SHORT,  &
      C_CHAR,  &
      C_SIGNED_CHAR,  &
      C_NULL_PTR,  &
      C_INTPTR_T,  &
      C_INT,  &
      C_INT64_T
  
  IMPLICIT NONE
  
  PRIVATE
  
  ! Things from COMBase that are ubiquitous in arguments.
  PUBLIC :: HRESULT
  PUBLIC :: FLOAT
  PUBLIC :: UINT32
  
  !-----------------------------------------------------------------------------
  ! Base types
  !
  ! We include the D2D version in the type here.
  
  TYPE, BIND(C), PUBLIC :: D2D1_COLOR_F
    REAL(FLOAT) :: r = 1.0_FLOAT
    REAL(FLOAT) :: g = 1.0_FLOAT
    REAL(FLOAT) :: b = 1.0_FLOAT
    REAL(FLOAT) :: a = 1.0_FLOAT
  END TYPE D2D1_COLOR_F
  
  TYPE, BIND(C), PUBLIC :: D2D1_POINT_2U
    INTEGER(UINT32) :: x = 0
    INTEGER(UINT32) :: y = 0
  END TYPE D2D1_POINT_2U
  
  TYPE, BIND(C), PUBLIC :: D2D1_POINT_2F
    REAL(FLOAT) :: x = 0.0_FLOAT
    REAL(FLOAT) :: y = 0.0_FLOAT
  END TYPE D2D1_POINT_2F
  
  TYPE, BIND(C), PUBLIC :: D2D1_RECT_F
    REAL(FLOAT) :: left = 0.0_FLOAT
    REAL(FLOAT) :: top = 0.0_FLOAT
    REAL(FLOAT) :: right = 0.0_FLOAT
    REAL(FLOAT) :: bottom = 0.0_FLOAT
  END TYPE D2D1_RECT_F
  INTERFACE D2D1_RECT_F_IfortBug
    MODULE PROCEDURE D2D1_RECT_F_constructor_point_size
    MODULE PROCEDURE D2D1_RECT_F_constructor_point_point
  END INTERFACE D2D1_RECT_F_IfortBug
  PUBLIC :: D2D1_RECT_F_IfortBug
  
  
  TYPE(D2D1_RECT_F), PARAMETER :: InfiniteRect  &
      = D2D1_RECT_F(  &
          -HUGE(InfiniteRect%left),  &
          -HUGE(InfiniteRect%top),  &
          HUGE(InfiniteRect%right),  &
          HUGE(InfiniteRect%bottom) )
  
  TYPE, BIND(C), PUBLIC :: D2D1_RECT_U
    INTEGER(UINT32) :: left = 0
    INTEGER(UINT32) :: top = 0
    INTEGER(UINT32) :: right = 0
    INTEGER(UINT32) :: bottom = 0
  END TYPE D2D1_RECT_U
  
  TYPE, BIND(C), PUBLIC :: D2D1_SIZE_F
    REAL(FLOAT) :: width = 0.0_FLOAT
    REAL(FLOAT) :: height = 0.0_FLOAT
  END TYPE D2D1_SIZE_F
  
  INTERFACE OPERATOR(+)
    MODULE PROCEDURE add_point_size
  END INTERFACE OPERATOR(+)
  PUBLIC :: OPERATOR(+)
  
  TYPE, BIND(C), PUBLIC :: D2D1_SIZE_U
    INTEGER(UINT32) :: width = 0
    INTEGER(UINT32) :: height = 0
  END TYPE D2D1_SIZE_U
  
  TYPE, BIND(C), PUBLIC :: D2D1_MATRIX_3x2_F
    REAL(FLOAT) :: data(2,3)
  END TYPE D2D1_MATRIX_3x2_F
  
  TYPE(D2D1_MATRIX_3x2_F), PARAMETER, PUBLIC :: IdentityMatrix  &
      = D2D1_MATRIX_3x2_F( RESHAPE(  &
        [ 1.0_FLOAT, 0.0_FLOAT,  &
          0.0_FLOAT, 1.0_FLOAT,  &
          0.0_FLOAT, 0.0_FLOAT ],  &
        [2, 3] ) )
  
  INTERFACE Translation
    MODULE PROCEDURE Translation_size
    MODULE PROCEDURE Translation_point
  END INTERFACE Translation
  PUBLIC :: Translation
  
  INTERFACE Scale
    MODULE PROCEDURE Scale_size
    MODULE PROCEDURE Scale_point
  END INTERFACE Scale
  PUBLIC :: Scale
  
  INTERFACE Rotation
    MODULE PROCEDURE Rotation_center
    MODULE PROCEDURE Rotation_no_center
  END INTERFACE Rotation
  PUBLIC :: Rotation
  
  INTERFACE Skew
    MODULE PROCEDURE Skew_center
    MODULE PROCEDURE Skew_no_center
  END INTERFACE Skew
  PUBLIC :: Skew
  
  INTERFACE
    SUBROUTINE MakeRotateMatrix(angle, center, matrix)
      IMPORT :: FLOAT
      IMPORT :: D2D1_POINT_2F
      IMPORT :: D2D1_MATRIX_3X2_F
      IMPLICIT NONE
      !-------------------------------------------------------------------------
      REAL(FLOAT), INTENT(IN) :: angle
      TYPE(D2D1_POINT_2F), INTENT(IN) :: center
      TYPE(D2D1_MATRIX_3X2_F), INTENT(OUT) :: matrix
      !DEC$ ATTRIBUTES REFERENCE :: matrix
      !DEC$ ATTRIBUTES STDCALL :: MakeRotateMatrix
      !DEC$ ATTRIBUTES ALIAS:'D2D1MakeRotateMatrix' :: MakeRotateMatrix
      !DEC$ ATTRIBUTES DECORATE :: MakeRotateMatrix
    END SUBROUTINE MakeRotateMatrix
    
    SUBROUTINE MakeSkewMatrix(angleX, angleY, center, matrix)
      IMPORT :: FLOAT
      IMPORT :: D2D1_POINT_2F
      IMPORT :: D2D1_MATRIX_3X2_F
      IMPLICIT NONE
      !-------------------------------------------------------------------------
      REAL(FLOAT), INTENT(IN) :: angleX
      REAL(FLOAT), INTENT(IN) :: angleY
      TYPE(D2D1_POINT_2F), INTENT(IN) :: center
      TYPE(D2D1_MATRIX_3X2_F), INTENT(OUT) :: matrix
      !DEC$ ATTRIBUTES REFERENCE :: matrix
      !DEC$ ATTRIBUTES STDCALL :: MakeSkewMatrix
      !DEC$ ATTRIBUTES ALIAS:'D2D1MakeSkewMatrix' :: MakeSkewMatrix
      !DEC$ ATTRIBUTES DECORATE :: MakeSkewMatrix
    END SUBROUTINE MakeSkewMatrix
  END INTERFACE
  PUBLIC :: MakeRotateMatrix
  PUBLIC :: MakeSkewMatrix
  
  PUBLIC :: IsInvertible
  PUBLIC :: Invert
  PUBLIC :: IsIdentity
  
  INTERFACE OPERATOR(*)
    MODULE PROCEDURE matrix_multiply
    MODULE PROCEDURE matrix_point_multiply
  END INTERFACE OPERATOR(*)
  PUBLIC :: OPERATOR(*)
  
  !-----------------------------------------------------------------------------
  
  INTEGER(HRESULT), PARAMETER :: fac_part  &
      = SHIFTL(INT(Z'899', HRESULT), 16)
  INTEGER(HRESULT), PARAMETER :: err_part  &
      = SHIFTL(1_HRESULT, 31)
  
  INTEGER(HRESULT), PARAMETER, PUBLIC :: D2DERR_RECREATE_TARGET  &
      = IANY([err_part, fac_part, INT(Z'0C', HRESULT)])
  
  !-----------------------------------------------------------------------------
  
  REAL(FLOAT), PARAMETER, PUBLIC :: D2D1_DEFAULT_FLATTENING_TOLERANCE  &
      = 0.25_FLOAT
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_ALPHA_MODE = DWORD
  INTEGER(D2D1_ALPHA_MODE), PARAMETER, PUBLIC  &
      :: D2D1_ALPHA_MODE_UNKNOWN = 0
  INTEGER(D2D1_ALPHA_MODE), PARAMETER, PUBLIC  &
      :: D2D1_ALPHA_MODE_PREMULTIPLIED = 1
  INTEGER(D2D1_ALPHA_MODE), PARAMETER, PUBLIC  &
      :: D2D1_ALPHA_MODE_STRAIGHT = 2
  INTEGER(D2D1_ALPHA_MODE), PARAMETER, PUBLIC  &
      :: D2D1_ALPHA_MODE_IGNORE = 3
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_GAMMA = DWORD
  INTEGER(D2D1_GAMMA), PARAMETER, PUBLIC :: D2D1_GAMMA_2_2 = 0
  INTEGER(D2D1_GAMMA), PARAMETER, PUBLIC :: D2D1_GAMMA_1_0 = 1
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_OPACITY_MASK_CONTENT = DWORD
  INTEGER(D2D1_OPACITY_MASK_CONTENT), PARAMETER, PUBLIC  &
      :: D2D1_OPACITY_MASK_CONTENT_GRAPHICS = 0
  INTEGER(D2D1_OPACITY_MASK_CONTENT), PARAMETER, PUBLIC  &
      :: D2D1_OPACITY_MASK_CONTENT_TEXT_NATURAL = 1
  INTEGER(D2D1_OPACITY_MASK_CONTENT), PARAMETER, PUBLIC  &
      :: D2D1_OPACITY_MASK_CONTENT_TEXT_GDI_COMPATIBLE = 2
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_EXTEND_MODE = DWORD
  INTEGER(D2D1_EXTEND_MODE), PARAMETER, PUBLIC :: D2D1_EXTEND_MODE_CLAMP = 0
  INTEGER(D2D1_EXTEND_MODE), PARAMETER, PUBLIC :: D2D1_EXTEND_MODE_WRAP = 1
  INTEGER(D2D1_EXTEND_MODE), PARAMETER, PUBLIC :: D2D1_EXTEND_MODE_MIRROR = 2
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_ANTIALIAS_MODE = DWORD
  INTEGER(D2D1_ANTIALIAS_MODE), PARAMETER, PUBLIC  &
      :: D2D1_ANTIALIAS_MODE_PER_PRIMITIVE = 0
  INTEGER(D2D1_ANTIALIAS_MODE), PARAMETER, PUBLIC  &
      :: D2D1_ANTIALIAS_MODE_ALIASED = 1
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_TEXT_ANTIALIAS_MODE = DWORD
  INTEGER(D2D1_TEXT_ANTIALIAS_MODE), PARAMETER, PUBLIC  &
      :: D2D1_TEXT_ANTIALIAS_MODE_DEFAULT = 0
  INTEGER(D2D1_TEXT_ANTIALIAS_MODE), PARAMETER, PUBLIC  &
      :: D2D1_TEXT_ANTIALIAS_MODE_CLEARTYPE = 1
  INTEGER(D2D1_TEXT_ANTIALIAS_MODE), PARAMETER, PUBLIC  &
      :: D2D1_TEXT_ANTIALIAS_MODE_GRAYSCALE = 2
  INTEGER(D2D1_TEXT_ANTIALIAS_MODE), PARAMETER, PUBLIC  &
      :: D2D1_TEXT_ANTIALIAS_MODE_ALIASED = 3
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_BITMAP_INTERPOLATION_MODE = DWORD
  INTEGER(D2D1_BITMAP_INTERPOLATION_MODE), PARAMETER, PUBLIC  &
      :: D2D1_BITMAP_INTERPOLATION_MODE_NEAREST_NEIGHBOUR = 0
  INTEGER(D2D1_BITMAP_INTERPOLATION_MODE), PARAMETER, PUBLIC  &
      :: D2D1_BITMAP_INTERPOLATION_MODE_LINEAR = 1
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_DRAW_TEXT_OPTIONS = DWORD
  INTEGER(D2D1_DRAW_TEXT_OPTIONS), PARAMETER, PUBLIC  &
      :: D2D1_DRAW_TEXT_OPTIONS_NO_SNAP = 1
  INTEGER(D2D1_DRAW_TEXT_OPTIONS), PARAMETER, PUBLIC  &
      :: D2D1_DRAW_TEXT_OPTIONS_CLIP = 2
  INTEGER(D2D1_DRAW_TEXT_OPTIONS), PARAMETER, PUBLIC  &
      :: D2D1_DRAW_TEXT_OPTIONS_NONE = 0
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_PIXEL_FORMAT
    INTEGER(DXGI_FORMAT) :: format = DXGI_FORMAT_UNKNOWN
    INTEGER(D2D1_ALPHA_MODE) :: alphaMode = D2D1_ALPHA_MODE_UNKNOWN
  END TYPE D2D1_PIXEL_FORMAT
  
  
  ! D2D to D2D1 typedef's elided.
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_TAG = C_INT64_T
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_BITMAP_PROPERTIES
    TYPE(D2D1_PIXEL_FORMAT) :: pixelFormat = D2D1_PIXEL_FORMAT()
    REAL(FLOAT) :: dpix = 96.0_FLOAT
    REAL(FLOAT) :: dpiY = 96.0_FLOAT
  END TYPE D2D1_BITMAP_PROPERTIES
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_GRADIENT_STOP
    REAL(FLOAT) :: position
    TYPE(D2D1_COLOR_F) :: color
  END TYPE D2D1_GRADIENT_STOP
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_BRUSH_PROPERTIES
    REAL(FLOAT) :: opacity = 1.0_FLOAT
    TYPE(D2D1_MATRIX_3x2_F) :: transform = IdentityMatrix
  END TYPE D2D1_BRUSH_PROPERTIES
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_BITMAP_BRUSH_PROPERTIES
    INTEGER(D2D1_EXTEND_MODE) :: extendModeX = D2D1_EXTEND_MODE_CLAMP
    INTEGER(D2D1_EXTEND_MODE) :: extendModeY = D2D1_EXTEND_MODE_CLAMP
    INTEGER(D2D1_BITMAP_INTERPOLATION_MODE) :: interpolationMode  &
        = D2D1_BITMAP_INTERPOLATION_MODE_LINEAR
  END TYPE D2D1_BITMAP_BRUSH_PROPERTIES
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_LINEAR_GRADIENT_BRUSH_PROPERTIES
    TYPE(D2D1_POINT_2F) :: startPoint
    TYPE(D2D1_POINT_2F) :: endPoint
  END TYPE D2D1_LINEAR_GRADIENT_BRUSH_PROPERTIES
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_RADIAL_GRADIENT_BRUSH_PROPERTIES
    TYPE(D2D1_POINT_2F) :: center
    TYPE(D2D1_POINT_2F) :: gradientOriginOffset = D2D1_POINT_2F()
    REAL(FLOAT) :: radiusX
    REAL(FLOAT) :: radiusY
  END TYPE D2D1_RADIAL_GRADIENT_BRUSH_PROPERTIES
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_ARC_SIZE = DWORD
  INTEGER(D2D1_ARC_SIZE), PARAMETER, PUBLIC :: D2D1_ARC_SIZE_SMALL = 0
  INTEGER(D2D1_ARC_SIZE), PARAMETER, PUBLIC :: D2D1_ARC_SIZE_LARGE = 1
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_CAP_STYLE = DWORD
  INTEGER(D2D1_CAP_STYLE), PARAMETER, PUBLIC :: D2D1_CAP_STYLE_FLAT = 0
  INTEGER(D2D1_CAP_STYLE), PARAMETER, PUBLIC :: D2D1_CAP_STYLE_SQUARE = 1
  INTEGER(D2D1_CAP_STYLE), PARAMETER, PUBLIC :: D2D1_CAP_STYLE_ROUND = 2
  INTEGER(D2D1_CAP_STYLE), PARAMETER, PUBLIC :: D2D1_CAP_STYLE_TRIANGLE = 3
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_DASH_STYLE = DWORD
  INTEGER(D2D1_DASH_STYLE), PARAMETER, PUBLIC :: D2D1_DASH_STYLE_SOLID = 0
  INTEGER(D2D1_DASH_STYLE), PARAMETER, PUBLIC :: D2D1_DASH_STYLE_DASH = 1
  INTEGER(D2D1_DASH_STYLE), PARAMETER, PUBLIC :: D2D1_DASH_STYLE_DOT = 2
  INTEGER(D2D1_DASH_STYLE), PARAMETER, PUBLIC :: D2D1_DASH_STYLE_DASH_DOT = 3
  INTEGER(D2D1_DASH_STYLE), PARAMETER, PUBLIC ::  &
      D2D1_DASH_STYLE_DASH_DOT_DOT = 4
  INTEGER(D2D1_DASH_STYLE), PARAMETER, PUBLIC :: D2D1_DASH_STYLE_CUSTOM = 5
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_LINE_JOIN = DWORD
  INTEGER(D2D1_LINE_JOIN), PARAMETER, PUBLIC :: D2D1_LINE_JOIN_MITER = 0
  INTEGER(D2D1_LINE_JOIN), PARAMETER, PUBLIC :: D2D1_LINE_JOIN_BEVEL = 1
  INTEGER(D2D1_LINE_JOIN), PARAMETER, PUBLIC :: D2D1_LINE_JOIN_ROUND = 2
  INTEGER(D2D1_LINE_JOIN), PARAMETER, PUBLIC  &
      :: D2D1_LINE_JOIN_MITER_OR_BEVEL = 3
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_COMBINE_MODE = DWORD
  INTEGER(D2D1_COMBINE_MODE), PARAMETER, PUBLIC :: D2D1_COMBINE_MODE_UNION = 0
  INTEGER(D2D1_COMBINE_MODE), PARAMETER, PUBLIC  &
      :: D2D1_COMBINE_MODE_INTERSECT = 1
  INTEGER(D2D1_COMBINE_MODE), PARAMETER, PUBLIC :: D2D1_COMBINE_MODE_XOR = 2
  INTEGER(D2D1_COMBINE_MODE), PARAMETER, PUBLIC  &
      :: D2D1_COMBINE_MODE_EXCLUDE = 3
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_GEOMETRY_RELATION = DWORD
  INTEGER(D2D1_GEOMETRY_RELATION), PARAMETER, PUBLIC  &
      :: D2D1_GEOMETRY_RELATION_UNKNOWN = 0
  INTEGER(D2D1_GEOMETRY_RELATION), PARAMETER, PUBLIC  &
      :: D2D1_GEOMETRY_RELATION_DISJOINT = 1
  INTEGER(D2D1_GEOMETRY_RELATION), PARAMETER, PUBLIC  &
      :: D2D1_GEOMETRY_RELATION_IS_CONTAINED = 2
  INTEGER(D2D1_GEOMETRY_RELATION), PARAMETER, PUBLIC  &
      :: D2D1_GEOMETRY_RELATION_CONTAINS = 3
  INTEGER(D2D1_GEOMETRY_RELATION), PARAMETER, PUBLIC  &
      :: D2D1_GEOMETRY_RELATION_OVERLAP = 4
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_GEOMETRY_SIMPLIFICATION_OPTION = DWORD
  INTEGER(D2D1_GEOMETRY_SIMPLIFICATION_OPTION), PARAMETER, PUBLIC  &
      :: D2D1_GEOMETRY_SIMPLIFICATION_OPTION_CUBICS_AND_LINES = 0
  INTEGER(D2D1_GEOMETRY_SIMPLIFICATION_OPTION), PARAMETER, PUBLIC  &
      :: D2D1_GEOMETRY_SIMPLIFICATION_OPTION_LINES = 1
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_FIGURE_BEGIN = DWORD
  INTEGER(D2D1_FIGURE_BEGIN), PARAMETER, PUBLIC  &
      :: D2D1_FIGURE_BEGIN_FILLED = 0
  INTEGER(D2D1_FIGURE_BEGIN), PARAMETER, PUBLIC  &
      :: D2D1_FIGURE_BEGIN_HOLLOW = 1
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_FIGURE_END = DWORD
  INTEGER(D2D1_FIGURE_END), PARAMETER, PUBLIC  &
      :: D2D1_FIGURE_END_OPEN = 0
  INTEGER(D2D1_FIGURE_END), PARAMETER, PUBLIC  &
      :: D2D1_FIGURE_END_CLOSED = 1
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_BEZIER_SEGMENT
    TYPE(D2D1_POINT_2F) :: point1
    TYPE(D2D1_POINT_2F) :: point2
    TYPE(D2D1_POINT_2F) :: point3
  END TYPE D2D1_BEZIER_SEGMENT
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_TRIANGLE
    TYPE(D2D1_POINT_2F) :: point1
    TYPE(D2D1_POINT_2F) :: point2
    TYPE(D2D1_POINT_2F) :: point3
  END TYPE D2D1_TRIANGLE
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_PATH_SEGMENT = DWORD
  INTEGER(D2D1_PATH_SEGMENT), PARAMETER, PUBLIC  &
      :: D2D1_PATH_SEGMENT_NONE = 0
  INTEGER(D2D1_PATH_SEGMENT), PARAMETER, PUBLIC  &
      :: D2D1_PATH_SEGMENT_UNSTROKED = 1
  INTEGER(D2D1_PATH_SEGMENT), PARAMETER, PUBLIC  &
      :: D2D1_PATH_SEGMENT_ROUND_LINE_JOIN = 2
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_SWEEP_DIRECTION = DWORD
  INTEGER(D2D1_SWEEP_DIRECTION), PARAMETER, PUBLIC  &
      :: D2D1_SWEEP_DIRECTION_COUNTER_CLOCKWISE = 0
  INTEGER(D2D1_SWEEP_DIRECTION), PARAMETER, PUBLIC  &
      :: D2D1_SWEEP_DIRECTION_CLOCKWISE = 1
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_FILL_MODE = DWORD
  INTEGER(D2D1_FILL_MODE), PARAMETER, PUBLIC :: D2D1_FILL_MODE_ALTERNATE = 0
  INTEGER(D2D1_FILL_MODE), PARAMETER, PUBLIC :: D2D1_FILL_MODE_WINDING = 1
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_ARC_SEGMENT
    TYPE(D2D1_POINT_2F) :: point
    TYPE(D2D1_SIZE_F) :: size
    REAL(FLOAT) :: rotationAngle
    INTEGER(D2D1_SWEEP_DIRECTION) :: sweepDirection
    INTEGER(D2D1_ARC_SIZE) :: arcSize
  END TYPE D2D1_ARC_SEGMENT
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_QUADRATIC_BEZIER_SEGMENT
    TYPE(D2D1_POINT_2F) :: point1
    TYPE(D2D1_POINT_2F) :: point2
  END TYPE D2D1_QUADRATIC_BEZIER_SEGMENT
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_ELLIPSE
    TYPE(D2D1_POINT_2F) :: point
    REAL(FLOAT) :: radiusX
    REAL(FLOAT) :: radiusY
  END TYPE D2D1_ELLIPSE
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_ROUNDED_RECT
    TYPE(D2D1_RECT_F) :: rect
    REAL(FLOAT) :: radiusX
    REAL(FLOAT) :: radiusY
  END TYPE D2D1_ROUNDED_RECT
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_STROKE_STYLE_PROPERTIES
    INTEGER(D2D1_CAP_STYLE) :: startCap = D2D1_CAP_STYLE_FLAT
    INTEGER(D2D1_CAP_STYLE) :: endCap = D2D1_CAP_STYLE_FLAT
    INTEGER(D2D1_CAP_STYLE) :: dashCap = D2D1_CAP_STYLE_FLAT
    INTEGER(D2D1_LINE_JOIN) :: lineJoin = D2D1_LINE_JOIN_MITER
    REAL(FLOAT) :: miterLimit = 10.0_FLOAT
    INTEGER(D2D1_DASH_STYLE) :: dashStyle = D2D1_DASH_STYLE_SOLID
    REAL(FLOAT) :: dashOffset = 0.0_FLOAT
  END TYPE D2D1_STROKE_STYLE_PROPERTIES
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_LAYER_OPTIONS = DWORD
  INTEGER(D2D1_LAYER_OPTIONS), PARAMETER, PUBLIC  &
      :: D2D1_LAYER_OPTIONS_NONE = 0
  INTEGER(D2D1_LAYER_OPTIONS), PARAMETER, PUBLIC  &
      :: D2D1_LAYER_OPTIONS_INITIALIZE_FOR_CLEARTYPE = 1
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_LAYER_PARAMETERS
    TYPE(D2D1_RECT_F) :: contentBounds = InfiniteRect
    TYPE(C_PTR) :: geometricMask = C_NULL_PTR
    INTEGER(D2D1_ANTIALIAS_MODE) :: maskAntialiasMode  &
        = D2D1_ANTIALIAS_MODE_PER_PRIMITIVE
    TYPE(D2D1_MATRIX_3X2_F) :: maskTransform = IdentityMatrix
    REAL(FLOAT) :: opacity = 1.0_FLOAT
    TYPE(C_PTR) :: opacityBrush = C_NULL_PTR
    INTEGER(D2D1_LAYER_OPTIONS) :: layerOptions = D2D1_LAYER_OPTIONS_NONE
  END TYPE D2D1_LAYER_PARAMETERS
  INTERFACE D2D1_LAYER_PARAMETERS
    MODULE PROCEDURE D2D1_LAYER_PARAMETERS_con
  END INTERFACE D2D1_LAYER_PARAMETERS
  
  ! Window state.
  INTEGER, PARAMETER, PUBLIC :: D2D1_WINDOW_STATE = DWORD
  INTEGER(D2D1_WINDOW_STATE), PARAMETER, PUBLIC  &
      :: D2D1_WINDOW_STATE_NONE = 0
  INTEGER(D2D1_WINDOW_STATE), PARAMETER, PUBLIC  &
      :: D2D1_WINDOW_STATE_OCCLUDED = 1
  
  
  ! Render target type
  INTEGER, PARAMETER, PUBLIC :: D2D1_RENDER_TARGET_TYPE = DWORD
  INTEGER(D2D1_RENDER_TARGET_TYPE), PARAMETER, PUBLIC  &
      :: D2D1_RENDER_TARGET_TYPE_DEFAULT = 0
  INTEGER(D2D1_RENDER_TARGET_TYPE), PARAMETER, PUBLIC  &
      :: D2D1_RENDER_TARGET_TYPE_SOFTWARE = 1
  INTEGER(D2D1_RENDER_TARGET_TYPE), PARAMETER, PUBLIC  &
      :: D2D1_RENDER_TARGET_TYPE_HARDWARE = 2
  
  
  ! Feature level
  INTEGER, PARAMETER, PUBLIC :: D2D1_FEATURE_LEVEL = DWORD
  INTEGER(D2D1_FEATURE_LEVEL), PARAMETER, PUBLIC  &
      :: D2D1_FEATURE_LEVEL_DEFAULT = 0
  INTEGER(D2D1_FEATURE_LEVEL), PARAMETER, PUBLIC  &
      :: D2D1_FEATURE_LEVEL_9 = INT(Z'9100', D2D1_FEATURE_LEVEL)
  INTEGER(D2D1_FEATURE_LEVEL), PARAMETER, PUBLIC  &
      :: D2D1_FEATURE_LEVEL_10 = INT(Z'a000', D2D1_FEATURE_LEVEL)
  
  
  ! Render target usage
  INTEGER, PARAMETER, PUBLIC :: D2D1_RENDER_TARGET_USAGE = DWORD
  INTEGER(D2D1_RENDER_TARGET_USAGE), PARAMETER, PUBLIC  &
      :: D2D1_RENDER_TARGET_USAGE_NONE = 0
  INTEGER(D2D1_RENDER_TARGET_USAGE), PARAMETER, PUBLIC  &
      :: D2D1_RENDER_TARGET_USAGE_FORCE_BITMAP_REMOTING = 1
  INTEGER(D2D1_RENDER_TARGET_USAGE), PARAMETER, PUBLIC  &
      :: D2D1_RENDER_TARGET_USAGE_GDI_COMPATIBLE = 2
  
  
  ! Present options
  INTEGER, PARAMETER, PUBLIC :: D2D1_PRESENT_OPTIONS = DWORD
  INTEGER(D2D1_PRESENT_OPTIONS), PARAMETER, PUBLIC  &
      :: D2D1_PRESENT_OPTIONS_NONE = 0
  INTEGER(D2D1_PRESENT_OPTIONS), PARAMETER, PUBLIC  &
      :: D2D1_PRESENT_OPTIONS_RETAIN_CONTENTS = 1
  INTEGER(D2D1_PRESENT_OPTIONS), PARAMETER, PUBLIC  &
      :: D2D1_PRESENT_OPTIONS_IMMEDIATELY = 2
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_RENDER_TARGET_PROPERTIES
    INTEGER(D2D1_RENDER_TARGET_TYPE) :: type  &
        = D2D1_RENDER_TARGET_TYPE_DEFAULT
    TYPE(D2D1_PIXEL_FORMAT) :: pixelFormat  &
        = D2D1_PIXEL_FORMAT()
    REAL(FLOAT) :: dpiX = 0.0_FLOAT
    REAL(FLOAT) :: dpiY = 0.0_FLOAT
    INTEGER(D2D1_RENDER_TARGET_USAGE) :: usage  &
        = D2D1_RENDER_TARGET_USAGE_NONE
    INTEGER(D2D1_FEATURE_LEVEL) :: minLevel  &
        = D2D1_FEATURE_LEVEL_DEFAULT
  END TYPE D2D1_RENDER_TARGET_PROPERTIES
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_HWND_RENDER_TARGET_PROPERTIES
    INTEGER(HANDLE) :: hwnd
    TYPE(D2D1_SIZE_U) :: pixelSize = D2D1_SIZE_U(0_UINT32, 0_UINT32)
    INTEGER(D2D1_PRESENT_OPTIONS) :: presentOptions = D2D1_PRESENT_OPTIONS_NONE
  END TYPE D2D1_HWND_RENDER_TARGET_PROPERTIES
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS = DWORD
  INTEGER(D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS), PARAMETER, PUBLIC  &
      :: D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS_NONE = 0
  INTEGER(D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS), PARAMETER, PUBLIC  &
      :: D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS_GDI_COMPATIBLE = 1
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_DRAWING_STATE_DESCRIPTION
    INTEGER(D2D1_ANTIALIAS_MODE) :: antialiasMode  &
        = D2D1_ANTIALIAS_MODE_PER_PRIMITIVE
    INTEGER(D2D1_TEXT_ANTIALIAS_MODE) :: textAntialiasMode  &
        = D2D1_TEXT_ANTIALIAS_MODE_DEFAULT
    INTEGER(D2D1_TAG) :: tag1 = 0_D2D1_TAG
    INTEGER(D2D1_TAG) :: tag2 = 0_D2D1_TAG
    TYPE(D2D1_MATRIX_3X2_F) :: transform = IdentityMatrix
  END TYPE D2D1_DRAWING_STATE_DESCRIPTION
  
  
  INTEGER, PARAMETER, PUBLIC :: D2D1_DC_INITIALIZE_MODE = DWORD
  INTEGER(D2D1_DC_INITIALIZE_MODE), PARAMETER, PUBLIC  &
      :: D2D1_DC_INITIALIZE_MODE_COPY = 0
  INTEGER(D2D1_DC_INITIALIZE_MODE), PARAMETER, PUBLIC  &
      :: D2D1_DC_INITIALIZE_MODE_CLEAR = 1
  
  
  ! Debug level.
  INTEGER, PARAMETER, PUBLIC :: D2D1_DEBUG_LEVEL = DWORD
  INTEGER(D2D1_DEBUG_LEVEL), PARAMETER, PUBLIC :: D2D1_DEBUG_LEVEL_NONE = 0
  INTEGER(D2D1_DEBUG_LEVEL), PARAMETER, PUBLIC :: D2D1_DEBUG_LEVEL_ERROR = 1
  INTEGER(D2D1_DEBUG_LEVEL), PARAMETER, PUBLIC :: D2D1_DEBUG_LEVEL_WARNING = 2
  INTEGER(D2D1_DEBUG_LEVEL), PARAMETER, PUBLIC  &
      :: D2D1_DEBUG_LEVEL_INFORMATION = 3
  
  
  ! Factory type.
  INTEGER, PARAMETER, PUBLIC :: D2D1_FACTORY_TYPE = DWORD
  INTEGER(D2D1_FACTORY_TYPE), PARAMETER, PUBLIC  &
      :: D2D1_FACTORY_TYPE_SINGLE_THREADED = 0
  INTEGER(D2D1_FACTORY_TYPE), PARAMETER, PUBLIC  &
      :: D2D1_FACTORY_TYPE_MULTI_THREADED = 1
  
  
  TYPE, BIND(C), PUBLIC :: D2D1_FACTORY_OPTIONS
    INTEGER(D2D1_DEBUG_LEVEL) :: debugLevel
  END TYPE
  
  !-----------------------------------------------------------------------------
  
  PUBLIC :: GetFactory
  PUBLIC :: GetSize
  PUBLIC :: SetTransform
  PUBLIC :: GetTransform
  PUBLIC :: SetBitmap
  PUBLIC :: GetBitmap
  PUBLIC :: GetPixelSize
  PUBLIC :: GetPixelFormat
  PUBLIC :: GetDpi
  PUBLIC :: SetDpi
  
  PUBLIC :: GetGradientStopCollection
  
  PUBLIC :: SetTextRenderingParams
  PUBLIC :: GetTextRenderingParams
  
  PUBLIC :: Open
  PUBLIC :: Close
  
  !-----------------------------------------------------------------------------
  ! "2cd90691-12e2-11dc-9fed-001143a055f9"  ID2D1Resource
  
  TYPE, BIND(C) :: ID2D1ResourceVTbl
    TYPE(IUnknownVTbl) :: Base
    TYPE(C_FUNPTR) :: GetFactory
  END TYPE ID2D1ResourceVTbl
  
  TYPE, EXTENDS(IUnknown), PUBLIC :: ID2D1Resource
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1resource_IID
  END TYPE ID2D1Resource
  
  !-----------------------------------------------------------------------------
  ! "65019f75-8da2-497c-b32c-dfa34e48ede6"  ID2D1Image
  !
  ! This interface was introduced with Windows 8.
  
  TYPE, BIND(C) :: ID2D1ImageVTbl
    TYPE(ID2D1ResourceVTbl) :: Base
  END TYPE ID2D1ImageVTbl
  
  TYPE, EXTENDS(ID2D1Resource) :: ID2D1Image
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1image_IID
  END TYPE ID2D1Image
  
  !-----------------------------------------------------------------------------
  ! "a2296057-ea42-4099-983b-539fb6505426"  ID2D1Bitmap
  
  TYPE, BIND(C) :: ID2D1BitmapVTbl
    TYPE(ID2D1ImageVTbl) :: Base
    TYPE(C_FUNPTR) :: GetSize
    TYPE(C_FUNPTR) :: GetPixelSize
    TYPE(C_FUNPTR) :: GetPixelFormat
    TYPE(C_FUNPTR) :: GetDpi
    TYPE(C_FUNPTR) :: CopyFromBitmap
    TYPE(C_FUNPTR) :: CopyFromRenderTarget
    TYPE(C_FUNPTR) :: CopyFromMemory
  END TYPE ID2D1BitmapVTbl
  
  TYPE, EXTENDS(ID2D1Image), PUBLIC :: ID2D1Bitmap
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1bitmap_IID
  END TYPE ID2D1Bitmap
  
  INTERFACE GetSize
    MODULE PROCEDURE GetSize_bitmap
  END INTERFACE GetSize
  INTERFACE GetPixelSize
    MODULE PROCEDURE GetPixelSize_bitmap
  END INTERFACE GetPixelSize
  INTERFACE GetPixelFormat
    MODULE PROCEDURE GetPixelFormat_bitmap
  END INTERFACE GetPixelFormat
  INTERFACE GetDpi
    MODULE PROCEDURE GetDpi_bitmap
  END INTERFACE GetDpi
  PUBLIC :: CopyFromBitmap
  PUBLIC :: CopyFromRenderTarget
  PUBLIC :: CopyFromMemory
  
  !-----------------------------------------------------------------------------
  ! "2cd906a7-12e2-11dc-9fed-001143a055f9" ID2D1GradientStopCollection
  
  TYPE, BIND(C) :: ID2D1GradientStopCollectionVTbl
    TYPE(ID2D1ResourceVTbl) :: Base
    TYPE(C_FUNPTR) :: GetGradientStopCount
    TYPE(C_FUNPTR) :: GetGradientStops
    TYPE(C_FUNPTR) :: GetColorInterpolationGamma
    TYPE(C_FUNPTR) :: GetExtendMode
  END TYPE ID2D1GradientStopCollectionVTbl
  
  TYPE, EXTENDS(ID2D1Resource), PUBLIC :: ID2D1GradientStopCollection
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1gradientstopcollection_IID
  END TYPE ID2D1GradientStopCollection
  
  PUBLIC :: GetGradientStopCount
  PUBLIC :: GetGradientStops
  PUBLIC :: GetColorInterpolationGamma
  PUBLIC :: GetExtendMode
  
  !-----------------------------------------------------------------------------
  ! "2cd906a8-12e2-11dc-9fed-001143a055f9"  ID2D1Brush
  
  TYPE, BIND(C) :: ID2D1BrushVTbl
    TYPE(ID2D1ResourceVTbl) :: Base
    TYPE(C_FUNPTR) :: SetOpacity
    TYPE(C_FUNPTR) :: SetTransform
    TYPE(C_FUNPTR) :: GetOpacity
    TYPE(C_FUNPTR) :: GetTransform
  END TYPE ID2D1BrushVTbl
  
  TYPE, EXTENDS(ID2D1Resource), PUBLIC :: ID2D1Brush
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1brush_IID
  END TYPE ID2D1Brush
  
  PUBLIC :: SetOpacity
  INTERFACE SetTransform
    MODULE PROCEDURE SetTransform_brush
  END INTERFACE SetTransform
  PUBLIC :: GetOpacity
  INTERFACE GetTransform
    MODULE PROCEDURE GetTransform_brush
  END INTERFACE GetTransform
  
  !-----------------------------------------------------------------------------
  ! "2cd906aa-12e2-11dc-9fed-001143a055f9"  ID2D1BitmapBrush
  
  TYPE, BIND(C) :: ID2D1BitmapBrushVTbl
    TYPE(ID2D1BrushVTbl) :: Base
    TYPE(C_FUNPTR) :: SetExtendModeX
    TYPE(C_FUNPTR) :: SetExtendModeY
    TYPE(C_FUNPTR) :: SetInterpolationMode
    TYPE(C_FUNPTR) :: SetBitmap
    TYPE(C_FUNPTR) :: GetExtendModeX
    TYPE(C_FUNPTR) :: GetExtendModeY
    TYPE(C_FUNPTR) :: GetInterpolationMode
    TYPE(C_FUNPTR) :: GetBitmap
  END TYPE ID2D1BitmapBrushVTbl
  
  TYPE, EXTENDS(ID2D1Brush), PUBLIC :: ID2D1BitmapBrush
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1bitmapbrush_IID
  END TYPE ID2D1BitmapBrush
  
  PUBLIC :: SetExtendModeX
  PUBLIC :: SetExtendModeY
  PUBLIC :: SetInterpolationMode
  INTERFACE SetBitmap
    MODULE PROCEDURE SetBitmap_bitmapbrush
  END INTERFACE SetBitmap
  PUBLIC :: GetExtendModeX
  PUBLIC :: GetExtendModeY
  PUBLIC :: GetInterpolationMode
  INTERFACE GetBitmap
    MODULE PROCEDURE GetBitmap_bitmapbrush
  END INTERFACE GetBitmap
  
  !-----------------------------------------------------------------------------
  ! "2cd906a9-12e2-11dc-9fed-001143a055f9"  ID2D1SolidColorBrush
  
  TYPE, BIND(C) :: ID2D1SolidColorBrushVTbl
    TYPE(ID2D1BrushVTbl) :: Base
    TYPE(C_FUNPTR) :: SetColor
    TYPE(C_FUNPTR) :: GetColor
  END TYPE ID2D1SolidColorBrushVTbl
  
  TYPE, EXTENDS(ID2D1Brush), PUBLIC :: ID2D1SolidColorBrush
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1solidcolorbrush_IID
  END TYPE ID2D1SolidColorBrush
  
  PUBLIC :: SetColor
  PUBLIC :: GetColor
  
  !-----------------------------------------------------------------------------
  ! "2cd906ab-12e2-11dc-9fed-001143a055f9"  ID2D1LinearGradientBrush
  
  TYPE, BIND(C) :: ID2D1LinearGradientBrushVTbl
    TYPE(ID2D1BrushVTbl) :: Base
    TYPE(C_FUNPTR) :: SetStartPoint
    TYPE(C_FUNPTR) :: SetEndPoint
    TYPE(C_FUNPTR) :: GetStartPoint
    TYPE(C_FUNPTR) :: GetEndPoint
    TYPE(C_FUNPTR) :: GetGradientStopCollection
  END TYPE ID2D1LinearGradientBrushVTbl
  
  TYPE, EXTENDS(ID2D1Brush), PUBLIC :: ID2D1LinearGradientBrush
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1lineargradientbrush_IID
  END TYPE ID2D1LinearGradientBrush
  
  PUBLIC :: SetStartPoint
  PUBLIC :: SetEndPoint
  PUBLIC :: GetStartPoint
  PUBLIC :: GetEndPoint
  INTERFACE GetGradientStopCollection
    MODULE PROCEDURE GetgradientStopCollection_linear
  END INTERFACE GetGradientStopCollection
  
  !-----------------------------------------------------------------------------
  ! "2cd906ac-12e2-11dc-9fed-001143a055f9"  ID2D1RadialGradientBrush
  
  TYPE, BIND(C) :: ID2D1RadialGradientBrushVTbl
    TYPE(ID2D1BrushVTbl) :: Base
    TYPE(C_FUNPTR) :: SetCenter
    TYPE(C_FUNPTR) :: SetGradientOriginOffset
    TYPE(C_FUNPTR) :: SetRadiusX
    TYPE(C_FUNPTR) :: SetRadiusY
    TYPE(C_FUNPTR) :: GetCenter
    TYPE(C_FUNPTR) :: GetGradientOriginOffset
    TYPE(C_FUNPTR) :: GetRadiusX
    TYPE(C_FUNPTR) :: GetRadiusY
    TYPE(C_FUNPTR) :: GetGradientStopCollection
  END TYPE ID2D1RadialGradientBrushVTbl
  
  TYPE, EXTENDS(ID2D1Brush), PUBLIC :: ID2D1RadialGradientBrush
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1radialgradientbrush_IID
  END TYPE ID2D1RadialGradientBrush
  
  PUBLIC :: SetCenter
  PUBLIC :: SetGradientOriginOffset
  PUBLIC :: SetRadiusX
  PUBLIC :: SetRadiusY
  PUBLIC :: GetCenter
  PUBLIC :: GetGradientOriginOffset
  PUBLIC :: GetRadiusX
  PUBLIC :: GetRadiusY
  INTERFACE GetGradientStopCollection
    MODULE PROCEDURE GetgradientStopCollection_radial
  END INTERFACE GetGradientStopCollection
  
  !-----------------------------------------------------------------------------
  ! "2cd9069d-12e2-11dc-9fed-001143a055f9"  ID2D1StrokeStyle
  
  TYPE, BIND(C) :: ID2D1StrokeStyleVTbl
    TYPE(ID2D1ResourceVTbl) :: Base
    TYPE(C_FUNPTR) :: GetStartCap
    TYPE(C_FUNPTR) :: GetEndCap
    TYPE(C_FUNPTR) :: GetDashCap
    TYPE(C_FUNPTR) :: GetMiterLimit
    TYPE(C_FUNPTR) :: GetLineJoin
    TYPE(C_FUNPTR) :: GetDashOffset
    TYPE(C_FUNPTR) :: GetDashStyle
    TYPE(C_FUNPTR) :: GetDashesCount
    TYPE(C_FUNPTR) :: GetDashes
  END TYPE ID2D1StrokeStyleVTbl
  
  TYPE, EXTENDS(ID2D1Resource), PUBLIC :: ID2D1StrokeStyle
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1strokestyle_IID
  END TYPE ID2D1StrokeStyle
  
  PUBLIC :: GetStartCap
  PUBLIC :: GetEndCap
  PUBLIC :: GetDashCap
  PUBLIC :: GetMiterLimit
  PUBLIC :: GetLineJoin
  PUBLIC :: GetDashOffset
  PUBLIC :: GetDashStyle
  PUBLIC :: GetDashesCount
  PUBLIC :: GetDashes
  
  !-----------------------------------------------------------------------------
  ! "2cd906a1-12e2-11dc-9fed-001143a055f9"  ID2D1Geometry
  
  TYPE, BIND(C) :: ID2D1GeometryVTbl
    TYPE(ID2D1ResourceVTbl) :: Base
    TYPE(C_FUNPTR) :: GetBounds
    TYPE(C_FUNPTR) :: GetWidenedBounds
    TYPE(C_FUNPTR) :: StrokeContainsPoint
    TYPE(C_FUNPTR) :: FillContainsPoint
    TYPE(C_FUNPTR) :: CompareWithGeometry
    TYPE(C_FUNPTR) :: Simplify
    TYPE(C_FUNPTR) :: Tessellate
    TYPE(C_FUNPTR) :: CombineWithGeometry
    TYPE(C_FUNPTR) :: Outline
    TYPE(C_FUNPTR) :: ComputeArea
    TYPE(C_FUNPTR) :: ComputeLength
    TYPE(C_FUNPTR) :: ComputePointAtLength
    TYPE(C_FUNPTR) :: Widen
  END TYPE ID2D1GeometryVTbl
  
  TYPE, EXTENDS(ID2D1Resource), PUBLIC :: ID2D1Geometry
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1geometry_IID
  END TYPE ID2D1Geometry
  
  PUBLIC :: GetBounds
  PUBLIC :: GetWidenedBounds
  PUBLIC :: StrokeContainsPoint
  PUBLIC :: FillContainsPoint
  PUBLIC :: CompareWithGeometry
  PUBLIC :: Simplify
  PUBLIC :: Tessellate
  PUBLIC :: CombineWithGeometry
  PUBLIC :: Outline
  PUBLIC :: ComputeArea
  PUBLIC :: ComputeLength
  PUBLIC :: ComputePointAtLength
  PUBLIC :: Widen
  
  !-----------------------------------------------------------------------------
  ! "2cd906a2-12e2-11dc-9fed-001143a055f9"  ID2D1RectangleGeometry
  
  TYPE, BIND(C) :: ID2D1RectangleGeometryVTbl
    TYPE(ID2D1GeometryVTbl) :: Base
    TYPE(C_FUNPTR) :: GetRect
  END TYPE ID2D1RectangleGeometryVTbl
  
  TYPE, EXTENDS(ID2D1Geometry), PUBLIC :: ID2D1RectangleGeometry
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1rectanglegeometry_IID
  END TYPE ID2D1RectangleGeometry
  
  PUBLIC :: GetRect
  
  !-----------------------------------------------------------------------------
  ! "2cd906a3-12e2-11dc-9fed-001143a055f9"  ID2D1RoundedRectangleGeometry
  
  TYPE, BIND(C) :: ID2D1RoundedRectangleGeometryVTbl
    TYPE(ID2D1GeometryVTbl) :: Base
    TYPE(C_FUNPTR) :: GetRoundedRect
  END TYPE ID2D1RoundedRectangleGeometryVTbl
  
  TYPE, EXTENDS(ID2D1Geometry), PUBLIC :: ID2D1RoundedRectangleGeometry
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1roundedrectanglegeometry_IID
  END TYPE ID2D1RoundedRectangleGeometry
  
  PUBLIC :: GetRoundedRect
  
  !-----------------------------------------------------------------------------
  ! "2cd906a4-12e2-11dc-9fed-001143a055f9"  ID2D1EllipseGeometry
  
  TYPE, BIND(C) :: ID2D1EllipseGeometryVTbl
    TYPE(ID2D1GeometryVTbl) :: Base
    TYPE(C_FUNPTR) :: GetEllipse
  END TYPE ID2D1EllipseGeometryVTbl
  
  TYPE, EXTENDS(ID2D1Geometry), PUBLIC :: ID2D1EllipseGeometry
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1ellipsegeometry_IID
  END TYPE ID2D1EllipseGeometry
  
  PUBLIC :: GetEllipse
  
  !-----------------------------------------------------------------------------
  ! "2cd906a6-12e2-11dc-9fed-001143a055f9"  ID2D1GeometryGroup
  
  TYPE, BIND(C) :: ID2D1GeometryGroupVTbl
    TYPE(ID2D1GeometryVTbl) :: Base
    TYPE(C_FUNPTR) :: GetFillMode
    TYPE(C_FUNPTR) :: GetSourceGeometryCount
    TYPE(C_FUNPTR) :: GetSourceGeometries
  END TYPE ID2D1GeometryGroupVTbl
  
  TYPE, EXTENDS(ID2D1Geometry), PUBLIC :: ID2D1GeometryGroup
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1geometrygroup_IID
  END TYPE ID2D1GeometryGroup
  
  PUBLIC :: GetFillMode
  PUBLIC :: GetSourceGeometryCount
  PUBLIC :: GetSourceGeometries
  
  !-----------------------------------------------------------------------------
  ! "2cd906bb-12e2-11dc-9fed-001143a055f9"  ID2D1TransformedGeometry
  
  TYPE, BIND(C) :: ID2D1TransformedGeometryVTbl
    TYPE(ID2D1GeometryVTbl) :: Base
    TYPE(C_FUNPTR) :: GetSourceGeometry
    TYPE(C_FUNPTR) :: GetTransform
  END TYPE ID2D1TransformedGeometryVTbl
  
  TYPE, EXTENDS(ID2D1Geometry), PUBLIC :: ID2D1TransformedGeometry
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1transformedgeometry_IID
  END TYPE ID2D1TransformedGeometry
  
  PUBLIC :: GetSourceGeometry
  INTERFACE GetTransform
    MODULE PROCEDURE GetTransform_tg
  END INTERFACE GetTransform
  
  !-----------------------------------------------------------------------------
  ! "2cd9069e-12e2-11dc-9fed-001143a055f9"  ID2D1SimplifiedGeometrySink
  
  TYPE, BIND(C) :: ID2D1SimplifiedGeometrySinkVTbl
    TYPE(IUnknownVTbl) :: Base
    TYPE(C_FUNPTR) :: SetFillMode
    TYPE(C_FUNPTR) :: SetSegmentFlags
    TYPE(C_FUNPTR) :: BeginFigure
    TYPE(C_FUNPTR) :: AddLines
    TYPE(C_FUNPTR) :: AddBeziers
    TYPE(C_FUNPTR) :: EndFigure
    TYPE(C_FUNPTR) :: Close
  END TYPE ID2D1SimplifiedGeometrySinkVTbl
  
  ! Extends IUnknown
  TYPE, EXTENDS(IUnknown), PUBLIC :: ID2D1SimplifiedGeometrySink
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1simpliedgeometrysink_IID
  END TYPE ID2D1SimplifiedGeometrySink
  
  PUBLIC :: SetFillMode
  PUBLIC :: SetSegmentFlags
  PUBLIC :: BeginFigure
  PUBLIC :: AddLines
  PUBLIC :: AddBeziers
  PUBLIC :: EndFigure
  INTERFACE Close
    MODULE PROCEDURE Close_sgs
  END INTERFACE Close
  
  !-----------------------------------------------------------------------------
  ! "2cd9069f-12e2-11dc-9fed-001143a055f9"  ID2D1GeometrySink
  
  TYPE, BIND(C) :: ID2D1GeometrySinkVTbl
    TYPE(ID2D1SimplifiedGeometrySinkVTbl) :: Base
    TYPE(C_FUNPTR) :: AddLine
    TYPE(C_FUNPTR) :: AddBezier
    TYPE(C_FUNPTR) :: AddQuadraticBezier
    TYPE(C_FUNPTR) :: AddQuadraticBeziers
    TYPE(C_FUNPTR) :: AddArc
  END TYPE ID2D1GeometrySinkVTbl
  
  TYPE, EXTENDS(ID2D1SimplifiedGeometrySink), PUBLIC :: ID2D1GeometrySink
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1geometrysink_IID
  END TYPE ID2D1GeometrySink
  
  PUBLIC :: AddLine
  PUBLIC :: AddBezier
  PUBLIC :: AddQuadraticBezier
  PUBLIC :: AddQuadraticBeziers
  PUBLIC :: AddArc
  
  !-----------------------------------------------------------------------------
  ! "2cd906c1-12e2-11dc-9fed-001143a055f9"  ID2D1TessellationSink
  
  TYPE, BIND(C) :: ID2D1TessellationSinkVTbl
    TYPE(IUnknownVTbl) :: Base
    TYPE(C_FUNPTR) :: AddTriangles
    TYPE(C_FUNPTR) :: Close
  END TYPE ID2D1TessellationSinkVTbl
  
  ! Extends IUnknown.
  TYPE, EXTENDS(IUnknown), PUBLIC :: ID2D1TessellationSink
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1tessellationsink_IID
  END TYPE ID2D1TessellationSink
  
  PUBLIC :: AddTriangles
  INTERFACE Close
    MODULE PROCEDURE Close_ts
  END INTERFACE Close
  
  !-----------------------------------------------------------------------------
  ! "2cd906a5-12e2-11dc-9fed-001143a055f9"  ID2D1PathGeometry
  
  TYPE, BIND(C) :: ID2D1PathGeometryVTbl
    TYPE(ID2D1GeometryVTbl) :: Base
    TYPE(C_FUNPTR) :: Open
    TYPE(C_FUNPTR) :: Stream
    TYPE(C_FUNPTR) :: GetSegmentCount
    TYPE(C_FUNPTR) :: GetFigureCount
  END TYPE ID2D1PathGeometryVTbl
  
  TYPE, EXTENDS(ID2D1Geometry), PUBLIC :: ID2D1PathGeometry
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1pathgeometry_IID
  END TYPE ID2D1PathGeometry
  
  INTERFACE Open
    MODULE PROCEDURE Open_pg
  END INTERFACE Open
  PUBLIC :: Stream
  PUBLIC :: GetSegmentCount
  PUBLIC :: GetFigureCount
  
  !-----------------------------------------------------------------------------
  ! "2cd906c2-12e2-11dc-9fed-001143a055f9"  ID2D1Mesh
  
  TYPE, BIND(C) :: ID2D1MeshVTbl
    TYPE(ID2D1ResourceVTbl) :: Base
    TYPE(C_FUNPTR) :: Open
  END TYPE ID2D1MeshVTbl
  
  TYPE, EXTENDS(ID2D1Resource), PUBLIC :: ID2D1Mesh
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1mesh_IID
  END TYPE ID2D1Mesh
  
  INTERFACE Open
    MODULE PROCEDURE Open_mesh
  END INTERFACE Open
  
  !-----------------------------------------------------------------------------
  ! "2cd9069b-12e2-11dc-9fed-001143a055f9"  ID2D1Layer
  
  TYPE, BIND(C) :: ID2D1LayerVTbl
    TYPE(ID2D1ResourceVTbl) :: Base
    TYPE(C_FUNPTR) :: GetSize
  END TYPE ID2D1LayerVTbl
  
  TYPE, EXTENDS(ID2D1Resource), PUBLIC :: ID2D1Layer
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1layer_IID
  END TYPE ID2D1Layer
  
  INTERFACE GetSize
    MODULE PROCEDURE GetSize_layer
  END INTERFACE GetSize
  
  !-----------------------------------------------------------------------------
  ! "28506e39-ebf6-46a1-bb47-fd85565ab957"  ID2D1DrawingStateBlock
  
  TYPE, BIND(C) :: ID2D1DrawingStateBlockVTbl
    TYPE(ID2D1ResourceVTbl) :: Base
    TYPE(C_FUNPTR) :: GetDescription
    TYPE(C_FUNPTR) :: SetDescription
    TYPE(C_FUNPTR) :: SetTextRenderingParams
    TYPE(C_FUNPTR) :: GetTextRenderingParams
  END TYPE ID2D1DrawingStateBlockVTbl
  
  TYPE, EXTENDS(ID2D1Resource), PUBLIC :: ID2D1DrawingStateBlock
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1drawingstateblock_IID
  END TYPE ID2D1DrawingStateBlock
  
  PUBLIC :: GetDescription
  PUBLIC :: SetDescription
  INTERFACE SetTextRenderingParams
    MODULE PROCEDURE SetTextRenderingParams_state
  END INTERFACE SetTextRenderingParams
  INTERFACE GetTextRenderingParams
    MODULE PROCEDURE GetTextRenderingParams_state
  END INTERFACE GetTextRenderingParams
  
  !-----------------------------------------------------------------------------
  ! "2cd90694-12e2-11dc-9fed-001143a055f9"  ID2D1RenderTarget
  
  TYPE, BIND(C) :: ID2D1RenderTargetVTbl
    TYPE(ID2D1ResourceVTbl) :: Base
    TYPE(C_FUNPTR) :: CreateBitmap
    TYPE(C_FUNPTR) :: CreateBitmapFromWicBitmap
    TYPE(C_FUNPTR) :: CreateSharedBitmap
    TYPE(C_FUNPTR) :: CreateBitmapBrush
    TYPE(C_FUNPTR) :: CreateSolidColorBrush
    TYPE(C_FUNPTR) :: CreateGradientStopCollection
    TYPE(C_FUNPTR) :: CreateLinearGradientBrush
    TYPE(C_FUNPTR) :: CreateRadialGradientBrush
    TYPE(C_FUNPTR) :: CreateCompatibleRenderTarget
    TYPE(C_FUNPTR) :: CreateLayer
    TYPE(C_FUNPTR) :: CreateMesh
    TYPE(C_FUNPTR) :: DrawLine
    TYPE(C_FUNPTR) :: DrawRectangle
    TYPE(C_FUNPTR) :: FillRectangle
    TYPE(C_FUNPTR) :: DrawRoundedRectangle
    TYPE(C_FUNPTR) :: FillRoundedRectangle
    TYPE(C_FUNPTR) :: DrawEllipse
    TYPE(C_FUNPTR) :: FillEllipse
    TYPE(C_FUNPTR) :: DrawGeometry
    TYPE(C_FUNPTR) :: FillGeometry
    TYPE(C_FUNPTR) :: FillMesh
    TYPE(C_FUNPTR) :: FillOpacityMask
    TYPE(C_FUNPTR) :: DrawBitmap
    TYPE(C_FUNPTR) :: DrawText
    TYPE(C_FUNPTR) :: DrawTextLayout
    TYPE(C_FUNPTR) :: DrawGlyphRun
    TYPE(C_FUNPTR) :: SetTransform
    TYPE(C_FUNPTR) :: GetTransform
    TYPE(C_FUNPTR) :: SetAntialiasMode
    TYPE(C_FUNPTR) :: GetAntialiasMode
    TYPE(C_FUNPTR) :: SetTextAntialiasMode
    TYPE(C_FUNPTR) :: GetTextAntialiasMode
    TYPE(C_FUNPTR) :: SetTextRenderingParams
    TYPE(C_FUNPTR) :: GetTextRenderingParams
    TYPE(C_FUNPTR) :: SetTags
    TYPE(C_FUNPTR) :: GetTags
    TYPE(C_FUNPTR) :: PushLayer
    TYPE(C_FUNPTR) :: PopLayer
    TYPE(C_FUNPTR) :: Flush
    TYPE(C_FUNPTR) :: SaveDrawingState
    TYPE(C_FUNPTR) :: RestoreDrawingState
    TYPE(C_FUNPTR) :: PushAxisAlignedClip
    TYPE(C_FUNPTR) :: PopAxisAlignedClip
    TYPE(C_FUNPTR) :: Clear
    TYPE(C_FUNPTR) :: BeginDraw
    TYPE(C_FUNPTR) :: EndDraw
    TYPE(C_FUNPTR) :: GetPixelFormat
    TYPE(C_FUNPTR) :: SetDpi
    TYPE(C_FUNPTR) :: GetDpi
    TYPE(C_FUNPTR) :: GetSize
    TYPE(C_FUNPTR) :: GetPixelSize
    TYPE(C_FUNPTR) :: GetMaximumBitmapSize
    TYPE(C_FUNPTR) :: IsSupported
  END TYPE ID2D1RenderTargetVTbl
  
  TYPE, EXTENDS(ID2D1Resource), PUBLIC :: ID2D1RenderTarget
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1rendertarget_IID
  END TYPE ID2D1RenderTarget
  
  PUBLIC :: CreateBitmap
  PUBLIC :: CreateBitmapFromWicBitmap
  PUBLIC :: CreateSharedBitmap
  PUBLIC :: CreateBitmapBrush
  PUBLIC :: CreateSolidColorBrush
  PUBLIC :: CreateGradientStopCollection
  PUBLIC :: CreateLinearGradientBrush
  PUBLIC :: CreateRadialGradientBrush
  PUBLIC :: CreateCompatibleRenderTarget
  PUBLIC :: CreateLayer
  PUBLIC :: CreateMesh
  PUBLIC :: DrawLine
  PUBLIC :: DrawRectangle
  PUBLIC :: FillRectangle
  PUBLIC :: DrawRoundedRectangle
  PUBLIC :: FillRoundedRectangle
  PUBLIC :: DrawEllipse
  PUBLIC :: FillEllipse
  PUBLIC :: DrawGeometry
  PUBLIC :: FillGeometry
  PUBLIC :: FillMesh
  PUBLIC :: FillOpacityMask
  PUBLIC :: DrawBitmap
  INTERFACE DrawText
    MODULE PROCEDURE DrawText_1
    MODULE PROCEDURE DrawText_2
  END INTERFACE
  PUBLIC :: DrawText
  PUBLIC :: DrawTextLayout
  PUBLIC :: DrawGlyphRun
  INTERFACE SetTransform
    MODULE PROCEDURE SetTransform_render
  END INTERFACE SetTransform
  INTERFACE GetTransform
    MODULE PROCEDURE GetTransform_render
  END INTERFACE GetTransform
  PUBLIC :: SetAntialiasMode
  PUBLIC :: GetAntialiasMode
  PUBLIC :: SetTextAntialiasMode
  PUBLIC :: GetTextAntialiasMode
  INTERFACE SetTextRenderingParams
    MODULE PROCEDURE SetTextRenderingParams_render
  END INTERFACE SetTextRenderingParams
  INTERFACE GetTextRenderingParams
    MODULE PROCEDURE GetTextRenderingParams_render
  END INTERFACE GetTextRenderingParams
  PUBLIC :: SetTags
  PUBLIC :: GetTags
  PUBLIC :: PushLayer
  PUBLIC :: PopLayer
  PUBLIC :: Flush
  PUBLIC :: SaveDrawingState
  PUBLIC :: RestoreDrawingState
  PUBLIC :: PushAxisAlignedClip
  PUBLIC :: PopAxisAlignedClip
  PUBLIC :: Clear
  PUBLIC :: BeginDraw
  PUBLIC :: EndDraw
  INTERFACE GetPixelFormat
    MODULE PROCEDURE GetPixelFormat_render
  END INTERFACE GetPixelFormat
  INTERFACE SetDpi
    MODULE PROCEDURE SetDpi_render
  END INTERFACE SetDpi
  INTERFACE GetDpi
    MODULE PROCEDURE GetDpi_render
  END INTERFACE GetDpi
  INTERFACE GetSize
    MODULE PROCEDURE GetSize_render
  END INTERFACE GetSize
  INTERFACE GetPixelSize
    MODULE PROCEDURE GetPixelSize_render
  END INTERFACE GetPixelSize
  PUBLIC :: GetMaximumBitmapSize
  PUBLIC :: IsSupported
  
  !-----------------------------------------------------------------------------
  ! "2cd90695-12e2-11dc-9fed-001143a055f9"  ID2D1BitmapRenderTarget
  
  TYPE, BIND(C) :: ID2D1BitmapRenderTargetVTbl
    TYPE(ID2D1RenderTargetVTbl) :: Base
    TYPE(C_FUNPTR) :: GetBitmap
  END TYPE ID2D1BitmapRenderTargetVTbl
  
  TYPE, EXTENDS(ID2D1RenderTarget), PUBLIC :: ID2D1BitmapRenderTarget
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1bitmaprendertarget_IID
  END TYPE ID2D1BitmapRenderTarget
  
  INTERFACE GetBitmap
    MODULE PROCEDURE GetBitmap_render
  END INTERFACE GetBitmap
  
  !-----------------------------------------------------------------------------
  ! "2cd90698-12e2-11dc-9fed-001143a055f9"  ID2D1HwndRenderTarget
  
  TYPE, BIND(C), PUBLIC :: ID2D1HwndRenderTargetVTbl
    TYPE(ID2D1RenderTargetVTbl) :: Base
    TYPE(C_FUNPTR) :: CheckWindowState
    TYPE(C_FUNPTR) :: Resize
    TYPE(C_FUNPTR) :: GetHwnd
  END TYPE ID2D1HwndRenderTargetVTbl
  
  TYPE, EXTENDS(ID2D1RenderTarget), PUBLIC :: ID2D1HwndRenderTarget
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1hwndrendertarget_IID
  END TYPE ID2D1HwndRenderTarget
  
  PUBLIC :: CheckWindowState
  PUBLIC :: Resize
  PUBLIC :: GetHwnd
  
  !-----------------------------------------------------------------------------
  ! "e0db51c3-6f77-4bae-b3d5-e47509b35838"  ID2D1GdiInteropRenderTarget
  
  TYPE, BIND(C) :: ID2D1GdiInteropRenderTargetVTbl
    TYPE(IUnknownVTbl) :: Base
    TYPE(C_FUNPTR) :: GetDC
    TYPE(C_FUNPTR) :: ReleaseDC
  END TYPE ID2D1GdiInteropRenderTargetVTbl
  
  ! Extends IUnknown
  TYPE, EXTENDS(IUnknown), PUBLIC :: ID2D1GdiInteropRenderTarget
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1gdiinteroprendertarget_IID
  END TYPE ID2D1GdiInteropRenderTarget
  
  PUBLIC :: GetDC
  PUBLIC :: ReleaseDC
  
  !-----------------------------------------------------------------------------
  ! "1c51bc64-de61-46fd-9899-63a5d8f03950"  ID2D1DCRenderTarget
  
  TYPE, BIND(C) :: ID2D1DCRenderTargetVTbl
    TYPE(ID2D1RenderTargetVTbl) :: Base
    TYPE(C_FUNPTR) :: BindDC
  END TYPE ID2D1DCRenderTargetVTbl
  
  TYPE, EXTENDS(ID2D1RenderTarget), PUBLIC :: ID2D1DCRenderTarget
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1dcrendertarget_IID
  END TYPE ID2D1DCRenderTarget
  
  PUBLIC :: BindDC
  
  !-----------------------------------------------------------------------------
  ! "06152247-6f50-465a-9245-118bfd3b6007"  ID2D1Factory
  
  TYPE, BIND(C) :: ID2D1FactoryVTbl
    TYPE(IUnknownVtbl) :: Base
    TYPE(C_FUNPTR) :: ReloadSystemMetrics
    TYPE(C_FUNPTR) :: GetDesktopDpi
    TYPE(C_FUNPTR) :: CreateRectangleGeometry
    TYPE(C_FUNPTR) :: CreateRoundedRectangleGeometry
    TYPE(C_FUNPTR) :: CreateEllipseGeometry
    TYPE(C_FUNPTR) :: CreateGeometryGroup
    TYPE(C_FUNPTR) :: CreateTransformedGeometry
    TYPE(C_FUNPTR) :: CreatePathGeometry
    TYPE(C_FUNPTR) :: CreateStrokeStyle
    TYPE(C_FUNPTR) :: CreateDrawingStateBlock
    TYPE(C_FUNPTR) :: CreateWicBitmapRenderTarget
    TYPE(C_FUNPTR) :: CreateHwndRenderTarget
    TYPE(C_FUNPTR) :: CreateDxgiSurfaceRenderTarget
    TYPE(C_FUNPTR) :: CreateDCRenderTarget
  END TYPE ID2D1FactoryVTbl
  
  ! Extends IUknown.
  TYPE, EXTENDS(IUnknown), PUBLIC :: ID2D1Factory
  CONTAINS
    PROCEDURE, NOPASS :: IID => d2d1factory_IID
  END TYPE ID2D1Factory
  
  PUBLIC :: D2D1CreateFactory
  
  PUBLIC :: ReloadSystemMetrics
  PUBLIC :: GetDesktopDpi
  PUBLIC :: CreateRectangleGeometry
  PUBLIC :: CreateRoundedRectangleGeometry
  PUBLIC :: CreateEllipseGeometry
  PUBLIC :: CreateGeometryGroup
  PUBLIC :: CreateTransformedGeometry
  PUBLIC :: CreatePathGeometry
  PUBLIC :: CreateStrokeStyle
  PUBLIC :: CreateDrawingStateBlock
  PUBLIC :: CreateWicBitmapRenderTarget
  PUBLIC :: CreateHwndRenderTarget
  PUBLIC :: CreateDxgiSurfaceRenderTarget
  PUBLIC :: CreateDCRenderTarget
  
CONTAINS
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1_POINT_2F and D2D1_SIZE_F.
  
  
  !*****************************************************************************
  
  ELEMENTAL FUNCTION add_point_size(point, size) RESULT(r)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_POINT_2F), INTENT(IN) :: point
    
    TYPE(D2D1_SIZE_F), INTENT(IN) :: size
    
    TYPE(D2D1_POINT_2F) :: r
    
    !***************************************************************************
    
    r%x = point%x + size%width
    r%y = point%y + size%height
    
  END FUNCTION add_point_size
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1_RECT_F
  
  
  !*****************************************************************************
  
  FUNCTION D2D1_RECT_F_constructor_point_size(top_left, size) RESULT(rect)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_POINT_2F), INTENT(IN) :: top_left
    TYPE(D2D1_SIZE_F), INTENT(IN) :: size
    
    TYPE(D2D1_RECT_F) :: rect
    
    !***************************************************************************
    
    IF (size%width >= 0.0_FLOAT) THEN
      rect%left = top_left%x
      rect%right = top_left%x + size%width
    ELSE
      rect%left = top_left%x + size%width
      rect%right = top_left%x
    END IF
    
    IF (size%height >= 0.0_FLOAT) THEN
      rect%top = top_left%y
      rect%bottom = top_left%y + size%height
    ELSE
      rect%top = top_left%y + size%height
      rect%bottom = top_left%y
    END IF
    
  END FUNCTION D2D1_RECT_F_constructor_point_size
  
  
  !*****************************************************************************
  
  FUNCTION D2D1_RECT_F_constructor_point_point(top_left, bottom_right)  &
       RESULT(rect)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_POINT_2F), INTENT(IN) :: top_left
    TYPE(D2D1_POINT_2F), INTENT(IN) :: bottom_right
    
    TYPE(D2D1_RECT_F) :: rect
    
    !***************************************************************************
    
    rect%left = MIN(top_left%x, bottom_right%x)
    rect%right = MAX(top_left%x, bottom_right%x)
    rect%top = MIN(top_left%y, bottom_right%y)
    
  END FUNCTION D2D1_RECT_F_constructor_point_point
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1_MATRIX_3X2_F
  
  
  !*****************************************************************************
  
  ELEMENTAL FUNCTION Translation_size(size) RESULT(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_SIZE_F), INTENT(IN) :: size
    
    TYPE(D2D1_MATRIX_3X2_F) :: matrix
    
    !***************************************************************************
    
    matrix%data = RESHAPE([  &
        1.0_FLOAT, 0.0_FLOAT,  &
        0.0_FLOAT, 1.0_FLOAT,  &
        size%width, size%height ], SHAPE(matrix%data))
    
  END FUNCTION Translation_size
  
  
  !*****************************************************************************
  
  ELEMENTAL FUNCTION Translation_point(x, y) RESULT(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    REAL(FLOAT), INTENT(IN) :: x
    REAL(FLOAT), INTENT(IN) :: y
    
    TYPE(D2D1_MATRIX_3X2_F) :: matrix
    
    !***************************************************************************
    
    matrix = Translation(D2D1_SIZE_F(x, y))
    
  END FUNCTION Translation_point
  
  
  !*****************************************************************************
  
  ELEMENTAL FUNCTION Scale_size(size, center) RESULT(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_SIZE_F), INTENT(IN) :: size
    TYPE(D2D1_POINT_2F), INTENT(IN) :: center
    
    TYPE(D2D1_MATRIX_3X2_F) :: matrix
    
    !***************************************************************************
    
    matrix%data = RESHAPE( [  &
          size%width,  &
          0.0_FLOAT,  &
          0.0_FLOAT,  &
          size%height,  &
          center%x - size%width * center%x,  &
          center%y - size%height * center%y ],  &
        SHAPE(matrix%data) )
    
  END FUNCTION Scale_size
  
  
  !*****************************************************************************
  
  ELEMENTAL FUNCTION Scale_point(x, y, center) RESULT(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    REAL(FLOAT), INTENT(IN) :: x
    REAL(FLOAT), INTENT(IN) :: y
    TYPE(D2D1_POINT_2F), INTENT(IN) :: center
    
    TYPE(D2D1_MATRIX_3X2_F) :: matrix
    
    !***************************************************************************
    
    matrix = Scale(D2D1_SIZE_F(x, y), center)
    
  END FUNCTION Scale_point
  
  
  !*****************************************************************************
  
  FUNCTION Rotation_center(angle, center) RESULT(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    REAL(FLOAT), INTENT(IN) :: angle
    TYPE(D2D1_POINT_2F), INTENT(IN) :: center
    
    TYPE(D2D1_MATRIX_3X2_F) :: matrix
    
    !***************************************************************************
    
    CALL MakeRotateMatrix(angle, center, matrix)
    
  END FUNCTION Rotation_center
  
  
  !*****************************************************************************
  
  FUNCTION Rotation_no_center(angle) RESULT(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    REAL(FLOAT), INTENT(IN) :: angle
    
    TYPE(D2D1_MATRIX_3X2_F) :: matrix
    
    !***************************************************************************
    
    CALL MakeRotateMatrix(angle, D2D1_POINT_2F(), matrix)
    
  END FUNCTION Rotation_no_center
  
  
  !*****************************************************************************
  
  FUNCTION Skew_center(angleX, angleY, center) RESULT(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    REAL(FLOAT), INTENT(IN) :: angleX
    REAL(FLOAT), INTENT(IN) :: angleY
    TYPE(D2D1_POINT_2F), INTENT(IN) :: center
    
    TYPE(D2D1_MATRIX_3X2_F) :: matrix
    
    !***************************************************************************
    
    CALL MakeSkewMatrix(angleX, angleY, center, matrix)
    
  END FUNCTION Skew_center
  
  
  !*****************************************************************************
  
  FUNCTION Skew_no_center(angleX, angleY) RESULT(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    REAL(FLOAT), INTENT(IN) :: angleX
    REAL(FLOAT), INTENT(IN) :: angleY
    
    TYPE(D2D1_MATRIX_3X2_F) :: matrix
    
    !***************************************************************************
    
    CALL MakeSkewMatrix(angleX, angleY, D2D1_POINT_2F(), matrix)
    
  END FUNCTION Skew_no_center
  
  
  !*****************************************************************************
  
  FUNCTION IsInvertible(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: matrix
    
    LOGICAL :: IsInvertible
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    INTERFACE
      ! Renamed to avoid having to wrap directive lines.
      FUNCTION D2D1IsMatrixIble(matrix)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT
        IMPORT :: D2D1_MATRIX_3X2_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: matrix
        !DEC$ ATTRIBUTES REFERENCE :: matrix
        INTEGER(C_INT) :: D2D1IsMatrixIble
        !DEC$ ATTRIBUTES STDCALL :: D2D1IsMatrixIble
        !DEC$ ATTRIBUTES ALIAS:'D2D1IsMatrixInvertible' :: D2D1IsMatrixIble
        !DEC$ ATTRIBUTES DECORATE :: D2D1IsMatrixIble
      END FUNCTION D2D1IsMatrixIble
    END INTERFACE
    
    !***************************************************************************
    
    IsInvertible = D2D1IsMatrixIble(matrix) /= 0
    
  END FUNCTION IsInvertible
  
  
  !*****************************************************************************
  
  SUBROUTINE Invert(matrix, was_inverted)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_MATRIX_3X2_F), INTENT(INOUT) :: matrix
    
    LOGICAL, INTENT(OUT) :: was_inverted
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    INTERFACE
      ! Renamed to avoid having to wrap directive lines.
      FUNCTION D2D1InvertMatrix(matrix)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT
        IMPORT :: D2D1_MATRIX_3X2_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(D2D1_MATRIX_3X2_F), INTENT(INOUT) :: matrix
        !DEC$ ATTRIBUTES REFERENCE :: matrix
        INTEGER(C_INT) :: D2D1InvertMatrix
        !DEC$ ATTRIBUTES STDCALL :: D2D1InvertMatrix
        !DEC$ ATTRIBUTES ALIAS:'D2D1InvertMatrix' :: D2D1InvertMatrix
        !DEC$ ATTRIBUTES DECORATE :: D2D1InvertMatrix
      END FUNCTION D2D1InvertMatrix
    END INTERFACE
    
    !***************************************************************************
    
    was_inverted = D2D1InvertMatrix(matrix) /= 0
    
  END SUBROUTINE Invert
  
  
  !*****************************************************************************
  
  ELEMENTAL FUNCTION Determinant(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: matrix
    
    REAL(FLOAT) :: Determinant
    
    !***************************************************************************
    
    Determinant = matrix%data(1,1) * matrix%data(2,2)  &
        - matrix%data(2, 1) * matrix%data(1, 2)
    
  END FUNCTION Determinant
  
  
  !*****************************************************************************
  
  ELEMENTAL FUNCTION IsIdentity(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: matrix
    
    LOGICAL :: IsIdentity
    
    !***************************************************************************
    
    IsIdentity = ALL(matrix%data == IdentityMatrix%data)
    
  END FUNCTION IsIdentity
  
  
  !*****************************************************************************
  
  ELEMENTAL SUBROUTINE SetProduct(matrix, a, b)
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    TYPE(D2D1_MATRIX_3X2_F), INTENT(OUT) :: matrix
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: a
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: b
    
    !***************************************************************************
    
    ! This has to take into account the row major storage of the API matrix.
    matrix%data(1,1) = a%data(1,1) * b%data(1,1) + a%data(2,1) * b%data(1,2)
    matrix%data(2,1) = a%data(1,1) * b%data(2,1) + a%data(2,1) * b%data(2,2)
    
    matrix%data(1,2) = a%data(1,2) * b%data(1,1) + a%data(2,2) * b%data(1,2)
    matrix%data(2,2) = a%data(1,2) * b%data(2,1) + a%data(2,2) * b%data(2,2)
    
    matrix%data(1,3) = a%data(1,3) * b%data(1,1) + a%data(2,3) * b%data(1,2)  &
        + b%data(1,3)
    matrix%data(2,3) = a%data(1,3) * b%data(2,1) + a%data(2,3) * b%data(2,2)  &
        + b%data(2,3)
    
  END SUBROUTINE SetProduct
  
  
  !*****************************************************************************
  
  ELEMENTAL FUNCTION TransformPoint(matrix, point) RESULT(p)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: matrix
    TYPE(D2D1_POINT_2F), INTENT(IN) :: point
    
    TYPE(D2D1_POINT_2F) :: p
    
    !***************************************************************************
    
    p = point * matrix
    
  END FUNCTION TransformPoint
  
  
  !*****************************************************************************
  
  ELEMENTAL FUNCTION matrix_multiply(a, b) RESULT(matrix)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: a
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: b
    
    TYPE(D2D1_MATRIX_3X2_F) :: matrix
    
    !***************************************************************************
    
    CALL SetProduct(matrix, a, b)
    
  END FUNCTION matrix_multiply
  
  
  !*****************************************************************************
  
  ELEMENTAL FUNCTION matrix_point_multiply(point, matrix) RESULT(r)
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(D2D1_POINT_2F), INTENT(IN) :: point
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: matrix
    
    TYPE(D2D1_POINT_2F) :: r
    
    !***************************************************************************
    
    r%x = point%x * matrix%data(1,1) + point%y * matrix%data(1,2)  &
        + matrix%data(1,3)
    r%y = point%y * matrix%data(2,1) + point%y * matrix%data(2,2)  &
        + matrix%data(2,3)
    
  END FUNCTION matrix_point_multiply
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1_LAYER_PARAMETERS
  
  
  !*****************************************************************************
  
  FUNCTION D2D1_LAYER_PARAMETERS_con( contentBounds, geometricMask,  &
      maskAntialiasMode, maskTransform, opacity, opacityBrush,  &
      layerOptions ) RESULT(r)
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    TYPE(D2D1_RECT_F), INTENT(IN), OPTIONAL :: contentBounds
    CLASS(ID2D1Geometry), INTENT(IN), OPTIONAL :: geometricMask
    INTEGER(D2D1_ANTIALIAS_MODE), INTENT(IN), OPTIONAL :: maskAntialiasMode
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL :: maskTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: opacity
    CLASS(ID2D1Brush), INTENT(IN), OPTIONAL :: opacityBrush
    INTEGER(D2D1_LAYER_OPTIONS), INTENT(IN), OPTIONAL :: layerOptions
    
    TYPE(D2D1_LAYER_PARAMETERS) :: r
    
    !***************************************************************************
    
    ! We rely on default initialization for the things that are not present.
    IF (PRESENT(contentBounds))     r%contentBounds = contentBounds
    IF (PRESENT(geometricMask))     r%geometricMask = geometricMask%object
    IF (PRESENT(maskAntialiasMode)) r%maskAntialiasMode = maskAntialiasMode
    IF (PRESENT(maskTransform))     r%maskTransform = maskTransform
    IF (PRESENT(opacity))           r%opacity = opacity
    IF (PRESENT(opacityBrush))      r%opacityBrush = opacityBrush%object
    IF (PRESENT(layerOptions))      r%layerOptions = layerOptions
    
  END FUNCTION D2D1_LAYER_PARAMETERS_con
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1Resource
  
  
  !*****************************************************************************
  
  FUNCTION d2d1resource_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1resource_IID
    
    !***************************************************************************
    
    d2d1resource_IID = GUID(  &
        INT(Z'2CD90691', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1resource_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE GetFactory(resource, factory)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Resource), INTENT(IN) :: resource
    
    CLASS(ID2D1Factory), INTENT(OUT) :: factory
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetFactory_intf(this, factory)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: factory
        !DEC$ ATTRIBUTES REFERENCE :: factory
        !DEC$ ATTRIBUTES STDCALL :: GetFactory_intf
      END SUBROUTINE GetFactory_intf
    END INTERFACE
    
    TYPE(ID2D1ResourceVTbl), POINTER :: ptbl
    PROCEDURE(GetFactory_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(resource), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetFactory, proc)
    CALL proc(resource%object, factory%object)
    
  END SUBROUTINE GetFactory
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1Image.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1image_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1image_IID
    
    !***************************************************************************
    
    d2d1image_IID = GUID(  &
        INT(Z'65019F75', C_INT32_T),  &
        INT(Z'8DA2', C_INT16_T),  &
        INT(Z'497C', C_INT16_T),  &
        [ INT(Z'B3', C_SIGNED_CHAR), INT(Z'2C', C_SIGNED_CHAR),  &
          INT(Z'DF', C_SIGNED_CHAR), INT(Z'A3', C_SIGNED_CHAR),  &
          INT(Z'4E', C_SIGNED_CHAR), INT(Z'48', C_SIGNED_CHAR),  &
          INT(Z'ED', C_SIGNED_CHAR), INT(Z'E6', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1image_IID
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1Bitmap
  
  
  !*****************************************************************************
  
  FUNCTION d2d1bitmap_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1bitmap_IID
    
    !***************************************************************************
    
    d2d1bitmap_IID = GUID(  &
        INT(Z'A2296057', C_INT32_T),  &
        INT(Z'EA42', C_INT16_T),  &
        INT(Z'4099', C_INT16_T),  &
        [ INT(Z'98', C_SIGNED_CHAR), INT(Z'3B', C_SIGNED_CHAR),  &
          INT(Z'53', C_SIGNED_CHAR), INT(Z'9F', C_SIGNED_CHAR),  &
          INT(Z'B6', C_SIGNED_CHAR), INT(Z'50', C_SIGNED_CHAR),  &
          INT(Z'54', C_SIGNED_CHAR), INT(Z'26', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1bitmap_IID
  
  
  !*****************************************************************************
  
  FUNCTION GetSize_bitmap(bitmap) RESULT(size)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Bitmap), INTENT(IN) :: bitmap
    
    TYPE(D2D1_SIZE_F) :: size
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetSize_intf(this, size)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_SIZE_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_SIZE_F), INTENT(OUT) :: size
        !DEC$ ATTRIBUTES REFERENCE :: size
        !DEC$ ATTRIBUTES STDCALL :: GetSize_intf
      END SUBROUTINE GetSize_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapVTbl), POINTER :: ptbl
    PROCEDURE(GetSize_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(bitmap), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetSize, proc)
    CALL proc(bitmap%object, size)
    
  END FUNCTION GetSize_bitmap
  
  
  !*****************************************************************************
  
  FUNCTION GetPixelSize_bitmap(bitmap) RESULT(size)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Bitmap), INTENT(IN) :: bitmap
    
    TYPE(D2D1_SIZE_U) :: size
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetPixelSize_intf(this, size)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_SIZE_U
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_SIZE_U), INTENT(OUT) :: size
        !DEC$ ATTRIBUTES REFERENCE :: size
        !DEC$ ATTRIBUTES STDCALL :: GetPixelSize_intf
      END SUBROUTINE GetPixelSize_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapVTbl), POINTER :: ptbl
    PROCEDURE(GetPixelSize_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(bitmap), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetPixelSize, proc)
    CALL proc(bitmap%object, size)
    
  END FUNCTION GetPixelSize_bitmap
  
  
  !*****************************************************************************
  
  FUNCTION GetPixelFormat_bitmap(bitmap) RESULT(fmt)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Bitmap), INTENT(IN) :: bitmap
    
    TYPE(D2D1_PIXEL_FORMAT) :: fmt
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetPixelFormat_intf(this, fmt)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_PIXEL_FORMAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_PIXEL_FORMAT), INTENT(OUT) :: fmt
        !DEC$ ATTRIBUTES REFERENCE :: fmt
        !DEC$ ATTRIBUTES STDCALL :: GetPixelFormat_intf
      END SUBROUTINE GetPixelFormat_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapVTbl), POINTER :: ptbl
    PROCEDURE(GetPixelFormat_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(bitmap), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetPixelFormat, proc)
    CALL proc(bitmap%object, fmt)
    
  END FUNCTION GetPixelFormat_bitmap
  
  
  !*****************************************************************************
  
  SUBROUTINE GetDpi_bitmap(bitmap, dpiX, dpiY)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Bitmap), INTENT(IN) :: bitmap
    
    REAL(FLOAT), INTENT(OUT) :: dpiX
    
    REAL(FLOAT), INTENT(OUT) :: dpiY
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetDpi_intf(this, dpiX, dpiY)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(OUT) :: dpiX
        !DEC$ ATTRIBUTES REFERENCE :: dpiX
        REAL(FLOAT), INTENT(OUT) :: dpiY
        !DEC$ ATTRIBUTES REFERENCE :: dpiY
        !DEC$ ATTRIBUTES STDCALL :: GetDpi_intf
      END SUBROUTINE GetDpi_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapVTbl), POINTER :: ptbl
    PROCEDURE(GetDpi_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(bitmap), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetDpi, proc)
    CALL proc(bitmap%object, dpiX, dpiY)
    
  END SUBROUTINE GetDpi_bitmap
  
  
  !*****************************************************************************
  
  SUBROUTINE CopyFromBitmap(dest_bitmap, src_bitmap, hr, destPoint, srcRect)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Bitmap), INTENT(IN) :: dest_bitmap
    CLASS(ID2D1Bitmap), INTENT(IN) :: src_bitmap
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_POINT_2U), INTENT(IN), OPTIONAL :: destPoint
    TYPE(D2D1_RECT_U), INTENT(IN), OPTIONAL :: srcRect
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CopyFromBitmap_intf(this, destPoint, bitmap, srcRect)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: destPoint
        TYPE(C_PTR), INTENT(IN) :: bitmap
        TYPE(C_PTR), INTENT(IN) :: srcRect
        INTEGER(HRESULT) :: CopyFromBitmap_intf
        !DEC$ ATTRIBUTES STDCALL :: CopyFromBitmap_intf
      END FUNCTION CopyFromBitmap_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_destPoint
    TYPE(C_PTR) :: local_srcRect
    
    TYPE(ID2D1BitmapVTbl), POINTER :: ptbl
    PROCEDURE(CopyFromBitmap_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(destPoint)) THEN
      local_destPoint = C_LOC(destPoint)
    ELSE
      local_destPoint = C_NULL_PTR
    END IF
    
    IF (PRESENT(srcRect)) THEN
      local_srcRect = C_LOC(srcRect)
    ELSE
      local_srcRect = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(dest_bitmap), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CopyFromBitmap, proc)
    hr = proc(  &
        dest_bitmap%object,  &
        local_destPoint,  &
        src_bitmap%object,  &
        local_srcRect )
    
  END SUBROUTINE CopyFromBitmap
  
  
  !*****************************************************************************
  
  SUBROUTINE CopyFromRenderTarget(bitmap, renderTarget, hr, destPoint, srcRect)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Bitmap), INTENT(IN) :: bitmap
    CLASS(ID2D1RenderTarget), INTENT(IN) :: renderTarget
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_POINT_2U), INTENT(IN), OPTIONAL, TARGET :: destPoint
    TYPE(D2D1_RECT_U), INTENT(IN), OPTIONAL, TARGET :: srcRect
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CopyFromRenderTarget_intf( this, destPoint, renderTarget,  &
          srcRect )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: destPoint
        TYPE(C_PTR), INTENT(IN) :: renderTarget
        TYPE(C_PTR), INTENT(IN) :: srcRect
        INTEGER(HRESULT) :: CopyFromRenderTarget_intf
        !DEC$ ATTRIBUTES STDCALL :: CopyFromRenderTarget_intf
      END FUNCTION CopyFromRenderTarget_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_destPoint
    TYPE(C_PTR) :: local_srcRect
    
    TYPE(ID2D1BitmapVTbl), POINTER :: ptbl
    PROCEDURE(CopyFromRenderTarget_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(destPoint)) THEN
      local_destPoint = C_LOC(destPoint)
    ELSE
      local_destPoint = C_NULL_PTR
    END IF
    
    IF (PRESENT(srcRect)) THEN
      local_srcRect = C_LOC(srcRect)
    ELSE
      local_srcRect = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(bitmap), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CopyFromRenderTarget, proc)
    hr = proc(  &
        bitmap%object,  &
        local_destPoint,  &
        renderTarget%object,  &
        local_srcRect )
    
  END SUBROUTINE CopyFromRenderTarget
  
  
  !*****************************************************************************
  
  SUBROUTINE CopyFromMemory(bitmap, srcData, pitch, hr, dstRect)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Bitmap), INTENT(IN) :: bitmap
    TYPE(C_PTR), INTENT(IN) :: srcData
    INTEGER(UINT32), INTENT(IN) :: pitch
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_RECT_U), INTENT(IN), OPTIONAL, TARGET :: dstRect
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CopyFromMemory_intf(this, dstRect, srcData, pitch)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: UINT32
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: dstRect
        TYPE(C_PTR), INTENT(IN) :: srcData
        INTEGER(UINT32), INTENT(IN) :: pitch
        INTEGER(HRESULT) :: CopyFromMemory_intf
        !DEC$ ATTRIBUTES STDCALL :: CopyFromMemory_intf
      END FUNCTION CopyFromMemory_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_dstRect
    
    TYPE(ID2D1BitmapVTbl), POINTER :: ptbl
    PROCEDURE(CopyFromMemory_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(dstRect)) THEN
      local_dstRect = C_LOC(dstRect)
    ELSE
      local_dstRect = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(bitmap), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CopyFromMemory, proc)
    hr = proc(bitmap%object, local_dstRect, srcData, pitch)
    
  END SUBROUTINE CopyFromMemory
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1GradientStopCollection.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1gradientstopcollection_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1gradientstopcollection_IID
    
    !***************************************************************************
    
    d2d1gradientstopcollection_IID = GUID(  &
        INT(Z'2CD906A7', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1gradientstopcollection_IID
  
  
  !*****************************************************************************
  
  FUNCTION GetGradientStopCount(gradient_stop_collection) RESULT(count)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GradientStopCollection), INTENT(IN) :: gradient_stop_collection
    
    INTEGER(UINT32) :: count
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetGradientStopCount_intf(this)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: UINT32
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(UINT32) :: GetGradientStopCount_intf
        !DEC$ ATTRIBUTES STDCALL :: GetGradientStopCount_intf
      END FUNCTION GetGradientStopCount_intf
    END INTERFACE
    
    TYPE(ID2D1GradientStopCollectionVTbl), POINTER :: ptbl
    PROCEDURE(GetGradientStopCount_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(gradient_stop_collection), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetGradientStopCount, proc)
    count = proc(gradient_stop_collection%object)
    
  END FUNCTION GetGradientStopCount
  
  
  !*****************************************************************************
  
  SUBROUTINE GetGradientStops(gradient_stop_collection, gradientStops)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1GradientStopCollection), INTENT(IN) :: gradient_stop_collection
    TYPE(D2D1_GRADIENT_STOP), INTENT(OUT), ALLOCATABLE :: gradientStops(:)
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetGradientStops_intf(this, gradientStops, gradientStopsCount)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_GRADIENT_STOP
        IMPORT :: UINT32
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_GRADIENT_STOP), INTENT(OUT) :: gradientStops(*)
        INTEGER(UINT32), INTENT(IN) :: gradientStopsCount
        !DEC$ ATTRIBUTES STDCALL :: GetGradientStops_intf
      END SUBROUTINE GetGradientStops_intf
    END INTERFACE
    
    INTEGER(UINT32) :: count
    
    TYPE(ID2D1GradientStopCollectionVTbl), POINTER :: ptbl
    PROCEDURE(GetGradientStops_intf), POINTER :: proc
    
    !***************************************************************************
    
    count = GetGradientStopCount(gradient_stop_collection)
    ALLOCATE(gradientStops(count))
    
    CALL C_F_POINTER(VTable(gradient_stop_collection), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetGradientStops, proc)
    CALL proc(gradient_stop_collection%object, gradientStops, count)
    
  END SUBROUTINE GetGradientStops
  
  
  !*****************************************************************************
  
  FUNCTION GetColorInterpolationGamma(gradient_stop_collection) RESULT(gamma)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GradientStopCollection), INTENT(IN) :: gradient_stop_collection
    
    INTEGER(D2D1_GAMMA) :: gamma
    
    !---------------------------------------------------------------------------
    
    ABSTRACT INTERFACE
      FUNCTION GetColorInterpolationGamma_intf(this)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_GAMMA
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_GAMMA) :: GetColorInterpolationGamma_intf
        !DEC$ ATTRIBUTES STDCALL :: GetColorInterpolationGamma_intf
      END FUNCTION GetColorInterpolationGamma_intf
    END INTERFACE
    
    TYPE(ID2D1GradientStopCollectionVTbl), POINTER :: ptbl
    PROCEDURE(GetColorInterpolationGamma_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(gradient_stop_collection), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetColorInterpolationGamma, proc)
    gamma = proc(gradient_stop_collection%object)
    
  END FUNCTION GetColorInterpolationGamma
  
  
  !*****************************************************************************
  
  FUNCTION GetExtendMode(gradient_stop_collection) RESULT(mode)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GradientStopCollection), INTENT(IN) :: gradient_stop_collection
    
    INTEGER(D2D1_EXTEND_MODE) :: mode
    
    !---------------------------------------------------------------------------
    
    ABSTRACT INTERFACE
      FUNCTION GetExtendMode_intf(this)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_EXTEND_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_EXTEND_MODE) :: GetExtendMode_intf
        !DEC$ ATTRIBUTES STDCALL :: GetExtendMode_intf
      END FUNCTION GetExtendMode_intf
    END INTERFACE
    
    TYPE(ID2D1GradientStopCollectionVTbl), POINTER :: ptbl
    PROCEDURE(GetExtendMode_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(gradient_stop_collection), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetExtendMode, proc)
    mode = proc(gradient_stop_collection%object)
    
  END FUNCTION GetExtendMode
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for ID2D1BitmapBrush.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1brush_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1brush_IID
    
    !***************************************************************************
    
    d2d1brush_IID = GUID(  &
        INT(Z'2CD906A8', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1brush_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE SetOpacity(brush, opacity)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    REAL(FLOAT), INTENT(IN) :: opacity
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetOpacity_intf(this, opacity)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(IN) :: opacity
        !DEC$ ATTRIBUTES STDCALL :: SetOpacity_intf
      END SUBROUTINE SetOpacity_intf
    END INTERFACE
    
    TYPE(ID2D1BrushVTbl), POINTER :: ptbl
    PROCEDURE(SetOpacity_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetOpacity, proc)
    CALL proc(brush%object, opacity)
    
  END SUBROUTINE SetOpacity
  
  
  !*****************************************************************************
  
  SUBROUTINE SetTransform_brush(brush, transform)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: transform
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetTransform_intf(this, transform)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_MATRIX_3X2_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: transform
        !DEC$ ATTRIBUTES REFERENCE :: transform
        !DEC$ ATTRIBUTES STDCALL :: SetTransform_intf
      END SUBROUTINE SetTransform_intf
    END INTERFACE
    
    TYPE(ID2D1BrushVTbl), POINTER :: ptbl
    PROCEDURE(SetTransform_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetOpacity, proc)
    CALL proc(brush%object, transform)
    
  END SUBROUTINE SetTransform_brush
  
  
  !*****************************************************************************
  
  FUNCTION GetOpacity(brush) RESULT(opacity)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    
    REAL(FLOAT) :: opacity
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetOpacity_intf(this) RESULT(opacity)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT) :: opacity
        !DEC$ ATTRIBUTES STDCALL :: GetOpacity_intf
      END FUNCTION GetOpacity_intf
    END INTERFACE
    
    TYPE(ID2D1BrushVTbl), POINTER :: ptbl
    PROCEDURE(GetOpacity_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetOpacity, proc)
    opacity = proc(brush%object)
    
  END FUNCTION GetOpacity
  
  
  !*****************************************************************************
  
  FUNCTION GetTransform_brush(brush) RESULT(transform)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    
    ! Function result.
    TYPE(D2D1_MATRIX_3X2_F) :: transform
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetTransform_intf(this, transform)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_MATRIX_3X2_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_MATRIX_3X2_F), INTENT(OUT) :: transform
        !DEC$ ATTRIBUTES REFERENCE :: transform
        !DEC$ ATTRIBUTES STDCALL :: GetTransform_intf
      END SUBROUTINE GetTransform_intf
    END INTERFACE
    
    TYPE(ID2D1BrushVTbl), POINTER :: ptbl
    PROCEDURE(GetTransform_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetOpacity, proc)
    CALL proc(brush%object, transform)
    
  END FUNCTION GetTransform_brush
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1BitmapBrush
  
  
  !*****************************************************************************
  
  FUNCTION d2d1bitmapbrush_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1bitmapbrush_IID
    
    !***************************************************************************
    
    d2d1bitmapbrush_IID = GUID(  &
        INT(Z'2CD906AA', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1bitmapbrush_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE SetExtendModeX(brush, extendModeX)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1BitmapBrush), INTENT(IN) :: brush
    INTEGER(D2D1_EXTEND_MODE), INTENT(IN) :: extendModeX
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetExtendModeX_intf(this, extendModeX)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_EXTEND_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_EXTEND_MODE), INTENT(IN) :: extendModeX
        !DEC$ ATTRIBUTES STDCALL :: SetExtendModeX_intf
      END SUBROUTINE SetExtendModeX_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetExtendModeX_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetExtendModeX, proc)
    CALL proc(brush%object, extendModeX)
    
  END SUBROUTINE SetExtendModeX
  
  
  !*****************************************************************************
  
  SUBROUTINE SetExtendModeY(brush, extendModeY)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1BitmapBrush), INTENT(IN) :: brush
    INTEGER(D2D1_EXTEND_MODE), INTENT(IN) :: extendModeY
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetExtendModeY_intf(this, extendModeY)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_EXTEND_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_EXTEND_MODE), INTENT(IN) :: extendModeY
        !DEC$ ATTRIBUTES STDCALL :: SetExtendModeY_intf
      END SUBROUTINE SetExtendModeY_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetExtendModeY_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetExtendModeY, proc)
    CALL proc(brush%object, extendModeY)
    
  END SUBROUTINE SetExtendModeY
  
  
  !*****************************************************************************
  
  SUBROUTINE SetInterpolationMode(brush, interpolationMode)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1BitmapBrush), INTENT(IN) :: brush
    INTEGER(D2D1_BITMAP_INTERPOLATION_MODE), INTENT(IN) :: interpolationMode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetInterpolationMode_intf(this, interpolationMode)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_BITMAP_INTERPOLATION_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_BITMAP_INTERPOLATION_MODE), INTENT(IN)  &
            :: interpolationMode
        !DEC$ ATTRIBUTES STDCALL :: SetInterpolationMode_intf
      END SUBROUTINE SetInterpolationMode_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetInterpolationMode_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetInterpolationMode, proc)
    CALL proc(brush%object, interpolationMode)
    
  END SUBROUTINE SetInterpolationMode
  
  
  !*****************************************************************************
  
  SUBROUTINE SetBitmap_bitmapbrush(brush, bitmap)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1BitmapBrush), INTENT(IN) :: brush
    CLASS(ID2D1Bitmap), INTENT(IN) :: bitmap
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetBitmap_intf(this, bitmap)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: bitmap
        !DEC$ ATTRIBUTES STDCALL :: SetBitmap_intf
      END SUBROUTINE SetBitmap_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetBitmap_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetBitmap, proc)
    CALL proc(brush%object, bitmap%object)
    
  END SUBROUTINE SetBitmap_bitmapbrush
  
  
  !*****************************************************************************
  
  FUNCTION GetExtendModeX(brush) RESULT(extendModeX)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1BitmapBrush), INTENT(IN) :: brush
    
    INTEGER(D2D1_EXTEND_MODE) :: extendModeX
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetExtendModeX_intf(this) RESULT(extendModeX)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_EXTEND_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_EXTEND_MODE) :: extendModeX
        !DEC$ ATTRIBUTES STDCALL :: GetExtendModeX_intf
      END FUNCTION GetExtendModeX_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetExtendModeX_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetExtendModeX, proc)
    extendModeX = proc(brush%object)
    
  END FUNCTION GetExtendModeX
  
  
  !*****************************************************************************
  
  FUNCTION GetExtendModeY(brush) RESULT(extendModeY)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1BitmapBrush), INTENT(IN) :: brush
    
    INTEGER(D2D1_EXTEND_MODE) :: extendModeY
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetExtendModeY_intf(this) RESULT(extendModeY)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_EXTEND_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_EXTEND_MODE) :: extendModeY
        !DEC$ ATTRIBUTES STDCALL :: GetExtendModeY_intf
      END FUNCTION GetExtendModeY_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetExtendModeY_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetExtendModeY, proc)
    extendModeY = proc(brush%object)
    
  END FUNCTION GetExtendModeY
  
  
  !*****************************************************************************
  
  FUNCTION GetInterpolationMode(brush) RESULT(mode)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1BitmapBrush), INTENT(IN) :: brush
    
    INTEGER(D2D1_BITMAP_INTERPOLATION_MODE) :: mode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetInterpolationMode_intf(this) RESULT(mode)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_BITMAP_INTERPOLATION_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_BITMAP_INTERPOLATION_MODE) :: mode
        !DEC$ ATTRIBUTES STDCALL :: GetInterpolationMode_intf
      END FUNCTION GetInterpolationMode_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetInterpolationMode_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetInterpolationMode, proc)
    mode = proc(brush%object)
    
  END FUNCTION GetInterpolationMode
  
  
  !*****************************************************************************
  
  SUBROUTINE GetBitmap_bitmapbrush(brush, bitmap)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1BitmapBrush), INTENT(IN) :: brush
    CLASS(ID2D1Bitmap), INTENT(OUT) :: bitmap
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetBitmap_intf(this, bitmap)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: bitmap
        !DEC$ ATTRIBUTES REFERENCE :: bitmap
        !DEC$ ATTRIBUTES STDCALL :: GetBitmap_intf
      END SUBROUTINE GetBitmap_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetBitmap_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetBitmap, proc)
    CALL proc(brush%object, bitmap%object)
    
  END SUBROUTINE GetBitmap_bitmapbrush
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1SolidColorBrush
  
  
  !*****************************************************************************
  
  FUNCTION d2d1solidcolorbrush_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1solidcolorbrush_IID
    
    !***************************************************************************
    
    d2d1solidcolorbrush_IID = GUID(  &
        INT(Z'2CD906A9', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1solidcolorbrush_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE SetColor(brush, color)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1SolidColorBrush), INTENT(IN) :: brush
    TYPE(D2D1_COLOR_F), INTENT(IN) :: color
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetColor_intf(this, color)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_COLOR_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_COLOR_F), INTENT(IN) :: color
        !DEC$ ATTRIBUTES REFERENCE :: color
        !DEC$ ATTRIBUTES STDCALL :: SetColor_intf
      END SUBROUTINE SetColor_intf
    END INTERFACE
    
    TYPE(ID2D1SolidColorBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetColor_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetColor, proc)
    CALL proc(brush%object, color)
    
  END SUBROUTINE SetColor
  
  
  !*****************************************************************************
  
  FUNCTION GetColor(brush) RESULT(color)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1SolidColorBrush), INTENT(IN) :: brush
    
    TYPE(D2D1_COLOR_F) :: color
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetColor_intf(this, color)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_COLOR_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_COLOR_F), INTENT(OUT) :: color
        !DEC$ ATTRIBUTES REFERENCE :: color
        !DEC$ ATTRIBUTES STDCALL :: GetColor_intf
      END SUBROUTINE GetColor_intf
    END INTERFACE
    
    TYPE(ID2D1SolidColorBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetColor_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetColor, proc)
    CALL proc(brush%object, color)
    
  END FUNCTION GetColor
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1LinearGradientBrush.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1lineargradientbrush_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1lineargradientbrush_IID
    
    !***************************************************************************
    
    d2d1lineargradientbrush_IID = GUID(  &
        INT(Z'2CD906AB', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1lineargradientbrush_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE SetStartPoint(brush, startPoint)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1LinearGradientBrush), INTENT(IN) :: brush
    
    TYPE(D2D1_POINT_2F), INTENT(IN) :: startPoint
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetStartPoint_intf(this, startPoint)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: startPoint
        !DEC$ ATTRIBUTES STDCALL :: SetStartPoint_intf
      END SUBROUTINE SetStartPoint_intf
    END INTERFACE
    
    TYPE(ID2D1LinearGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetStartPoint_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetStartPoint, proc)
    CALL proc(brush%object, startPoint)
    
  END SUBROUTINE SetStartPoint
  
  
  !*****************************************************************************
  
  SUBROUTINE SetEndPoint(brush, endPoint)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1LinearGradientBrush), INTENT(IN) :: brush
    
    TYPE(D2D1_POINT_2F), INTENT(IN) :: endPoint
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetEndPoint_intf(this, endPoint)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: endPoint
        !DEC$ ATTRIBUTES STDCALL :: SetEndPoint_intf
      END SUBROUTINE SetEndPoint_intf
    END INTERFACE
    
    TYPE(ID2D1LinearGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetEndPoint_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetEndPoint, proc)
    CALL proc(brush%object, endPoint)
    
  END SUBROUTINE SetEndPoint
  
  
  !*****************************************************************************
  
  FUNCTION GetStartPoint(brush) RESULT(startPoint)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1LinearGradientBrush), INTENT(IN) :: brush
    
    TYPE(D2D1_POINT_2F) :: startPoint
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetStartPoint_intf(this, startPoint)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(OUT) :: startPoint
        !DEC$ ATTRIBUTES REFERENCE :: startPoint
        !DEC$ ATTRIBUTES STDCALL :: GetStartPoint_intf
      END SUBROUTINE GetStartPoint_intf
    END INTERFACE
    
    TYPE(ID2D1LinearGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetStartPoint_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetStartPoint, proc)
    CALL proc(brush%object, startPoint)
    
  END FUNCTION GetStartPoint
  
  
  !*****************************************************************************
  
  FUNCTION GetEndPoint(brush) RESULT(endPoint)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1LinearGradientBrush), INTENT(IN) :: brush
    
    TYPE(D2D1_POINT_2F) :: endPoint
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetEndPoint_intf(this, endPoint)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(OUT) :: endPoint
        !DEC$ ATTRIBUTES REFERENCE :: endPoint
        !DEC$ ATTRIBUTES STDCALL :: GetEndPoint_intf
      END SUBROUTINE GetEndPoint_intf
    END INTERFACE
    
    TYPE(ID2D1LinearGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetEndPoint_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetEndPoint, proc)
    CALL proc(brush%object, endPoint)
    
  END FUNCTION GetEndPoint
  
  
  !*****************************************************************************
  
  SUBROUTINE GetgradientStopCollection_linear(brush, gradientStopCollection)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1LinearGradientBrush), INTENT(IN) :: brush
    
    CLASS(ID2D1GradientStopCollection), INTENT(OUT) :: gradientStopCollection
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetGradientStopCollection_intf(this, gradientStopCollection)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: gradientStopCollection
        !DEC$ ATTRIBUTES REFERENCE :: gradientStopCollection
        !DEC$ ATTRIBUTES STDCALL :: GetGradientStopCollection_intf
      END SUBROUTINE GetGradientStopCollection_intf
    END INTERFACE
    
    TYPE(ID2D1LinearGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetGradientStopCollection_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetGradientStopCollection, proc)
    CALL proc(brush%object, gradientStopCollection%object)
    
  END SUBROUTINE GetGradientStopCollection_linear
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1RadialGradientBrush.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1radialgradientbrush_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1radialgradientbrush_IID
    
    !***************************************************************************
    
    d2d1radialgradientbrush_IID = GUID(  &
        INT(Z'2CD906AC', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1radialgradientbrush_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE SetCenter(brush, center)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RadialGradientBrush), INTENT(IN) :: brush
    TYPE(D2D1_POINT_2F), INTENT(IN) :: center
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetCenter_intf(this, center)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: center
        !DEC$ ATTRIBUTES STDCALL :: SetCenter_intf
      END SUBROUTINE SetCenter_intf
    END INTERFACE
    
    TYPE(ID2D1RadialGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetCenter_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetCenter, proc)
    CALL proc(brush%object, center)
    
  END SUBROUTINE SetCenter
  
  
  !*****************************************************************************
  
  SUBROUTINE SetGradientOriginOffset(brush, gradientOriginOffset)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RadialGradientBrush), INTENT(IN) :: brush
    TYPE(D2D1_POINT_2F), INTENT(IN) :: gradientOriginOffset
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetGradientOriginOffset_intf(this, gradientOriginOffset)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: gradientOriginOffset
        !DEC$ ATTRIBUTES STDCALL :: SetGradientOriginOffset_intf
      END SUBROUTINE SetGradientOriginOffset_intf
    END INTERFACE
    
    TYPE(ID2D1RadialGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetGradientOriginOffset_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetGradientOriginOffset, proc)
    CALL proc(brush%object, gradientOriginOffset)
    
  END SUBROUTINE SetGradientOriginOffset
  
  
  !*****************************************************************************
  
  SUBROUTINE SetRadiusX(brush, radiusX)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RadialGradientBrush), INTENT(IN) :: brush
    REAL(FLOAT), INTENT(IN) :: radiusX
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetRadiusX_intf(this, radiusX)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(IN) :: radiusX
        !DEC$ ATTRIBUTES STDCALL :: SetRadiusX_intf
      END SUBROUTINE SetRadiusX_intf
    END INTERFACE
    
    TYPE(ID2D1RadialGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetRadiusX_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetRadiusX, proc)
    CALL proc(brush%object, radiusX)
    
  END SUBROUTINE SetRadiusX
  
  
  !*****************************************************************************
  
  SUBROUTINE SetRadiusY(brush, radiusY)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RadialGradientBrush), INTENT(IN) :: brush
    REAL(FLOAT), INTENT(IN) :: radiusY
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetRadiusY_intf(this, radiusY)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(IN) :: radiusY
        !DEC$ ATTRIBUTES STDCALL :: SetRadiusY_intf
      END SUBROUTINE SetRadiusY_intf
    END INTERFACE
    
    TYPE(ID2D1RadialGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(SetRadiusY_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetRadiusY, proc)
    CALL proc(brush%object, radiusY)
    
  END SUBROUTINE SetRadiusY
  
  
  !*****************************************************************************
  
  FUNCTION GetCenter(brush) RESULT(center)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RadialGradientBrush), INTENT(IN) :: brush
    
    TYPE(D2D1_POINT_2F) :: center
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetCenter_intf(this, center)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(OUT) :: center
        !DEC$ ATTRIBUTES REFERENCE :: center
        !DEC$ ATTRIBUTES STDCALL :: GetCenter_intf
      END SUBROUTINE GetCenter_intf
    END INTERFACE
    
    TYPE(ID2D1RadialGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetCenter_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetCenter, proc)
    CALL proc(brush%object, center)
    
  END FUNCTION GetCenter
  
  
  !*****************************************************************************
  
  FUNCTION GetGradientOriginOffset(brush) RESULT(gradientOriginOffset)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RadialGradientBrush), INTENT(IN) :: brush
    
    TYPE(D2D1_POINT_2F) :: gradientOriginOffset
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetGradientOriginOffset_intf(this, gradientOriginOffset)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(OUT) :: gradientOriginOffset
        !DEC$ ATTRIBUTES REFERENCE :: gradientOriginOffset
        !DEC$ ATTRIBUTES STDCALL :: GetGradientOriginOffset_intf
      END SUBROUTINE GetGradientOriginOffset_intf
    END INTERFACE
    
    TYPE(ID2D1RadialGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetGradientOriginOffset_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetGradientOriginOffset, proc)
    CALL proc(brush%object, gradientOriginOffset)
    
  END FUNCTION GetGradientOriginOffset
  
  
  !*****************************************************************************
  
  FUNCTION GetRadiusX(brush) RESULT(radiusX)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RadialGradientBrush), INTENT(IN) :: brush
    
    REAL(FLOAT) :: radiusX
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetRadiusX_intf(this) RESULT(radiusX)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT) :: radiusX
        !DEC$ ATTRIBUTES STDCALL :: GetRadiusX_intf
      END FUNCTION GetRadiusX_intf
    END INTERFACE
    
    TYPE(ID2D1RadialGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetRadiusX_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetRadiusX, proc)
    radiusX = proc(brush%object)
    
  END FUNCTION GetRadiusX
  
  
  !*****************************************************************************
  
  FUNCTION GetRadiusY(brush) RESULT(radiusY)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RadialGradientBrush), INTENT(IN) :: brush
    
    REAL(FLOAT) :: radiusY
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetRadiusY_intf(this) RESULT(radiusY)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT) :: radiusY
        !DEC$ ATTRIBUTES STDCALL :: GetRadiusY_intf
      END FUNCTION GetRadiusY_intf
    END INTERFACE
    
    TYPE(ID2D1RadialGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetRadiusY_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetRadiusY, proc)
    radiusY = proc(brush%object)
    
  END FUNCTION GetRadiusY
  
  
  !*****************************************************************************
  
  SUBROUTINE GetGradientStopCollection_radial(brush, gradientStopCollection)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RadialGradientBrush), INTENT(IN) :: brush
    CLASS(ID2D1GradientStopCollection), INTENT(OUT) :: gradientStopCollection
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetGradientStopCollection_intf(this, gradientStopCollection)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: gradientStopCollection
        !DEC$ ATTRIBUTES REFERENCE :: gradientStopCollection
        !DEC$ ATTRIBUTES STDCALL :: GetGradientStopCollection_intf
      END SUBROUTINE GetGradientStopCollection_intf
    END INTERFACE
    
    TYPE(ID2D1RadialGradientBrushVTbl), POINTER :: ptbl
    PROCEDURE(GetGradientStopCollection_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(brush), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetGradientStopCollection, proc)
    CALL proc(brush%object, gradientStopCollection%object)
    
  END SUBROUTINE GetGradientStopCollection_radial
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1StrokeStyle.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1strokestyle_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1strokestyle_IID
    
    !***************************************************************************
    
    d2d1strokestyle_IID = GUID(  &
        INT(Z'2CD906AD', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1strokestyle_IID
  
  
  !*****************************************************************************
  
  FUNCTION GetStartCap(stroke_style) RESULT(start_cap)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1StrokeStyle), INTENT(IN) :: stroke_style
    
    INTEGER(D2D1_CAP_STYLE) :: start_cap
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetStartCap_intf(this) RESULT(start_cap)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_CAP_STYLE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_CAP_STYLE) :: start_cap
        !DEC$ ATTRIBUTES STDCALL :: GetStartCap_intf
      END FUNCTION GetStartCap_intf
    END INTERFACE
    
    TYPE(ID2D1StrokeStyleVTbl), POINTER :: ptbl
    PROCEDURE(GetStartCap_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(stroke_style), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetStartCap, proc)
    start_cap = proc(stroke_style%object)
    
  END FUNCTION GetStartCap
  
  
  !*****************************************************************************
  
  FUNCTION GetEndCap(stroke_style) RESULT(end_cap)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1StrokeStyle), INTENT(IN) :: stroke_style
    
    INTEGER(D2D1_CAP_STYLE) :: end_cap
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetEndCap_intf(this) RESULT(end_cap)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_CAP_STYLE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_CAP_STYLE) :: end_cap
        !DEC$ ATTRIBUTES STDCALL :: GetEndCap_intf
      END FUNCTION GetEndCap_intf
    END INTERFACE
    
    TYPE(ID2D1StrokeStyleVTbl), POINTER :: ptbl
    PROCEDURE(GetEndCap_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(stroke_style), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetEndCap, proc)
    end_cap = proc(stroke_style%object)
    
  END FUNCTION GetEndCap
  
  
  !*****************************************************************************
  
  FUNCTION GetDashCap(stroke_style) RESULT(end_cap)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1StrokeStyle), INTENT(IN) :: stroke_style
    
    INTEGER(D2D1_CAP_STYLE) :: end_cap
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetDashCap_intf(this) RESULT(end_cap)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_CAP_STYLE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_CAP_STYLE) :: end_cap
        !DEC$ ATTRIBUTES STDCALL :: GetDashCap_intf
      END FUNCTION GetDashCap_intf
    END INTERFACE
    
    TYPE(ID2D1StrokeStyleVTbl), POINTER :: ptbl
    PROCEDURE(GetDashCap_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(stroke_style), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetDashCap, proc)
    end_cap = proc(stroke_style%object)
    
  END FUNCTION GetDashCap
  
  
  !*****************************************************************************
  
  FUNCTION GetMiterLimit(stroke_style) RESULT(miter_limit)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1StrokeStyle), INTENT(IN) :: stroke_style
    
    REAL(FLOAT) :: miter_limit
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetMiterLimit_intf(this) RESULT(miter_limit)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT) :: miter_limit
        !DEC$ ATTRIBUTES STDCALL :: GetMiterLimit_intf
      END FUNCTION GetMiterLimit_intf
    END INTERFACE
    
    TYPE(ID2D1StrokeStyleVTbl), POINTER :: ptbl
    PROCEDURE(GetMiterLimit_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(stroke_style), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetMiterLimit, proc)
    miter_limit = proc(stroke_style%object)
    
  END FUNCTION GetMiterLimit
  
  
  !*****************************************************************************
  
  FUNCTION GetLineJoin(stroke_style) RESULT(line_join)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1StrokeStyle), INTENT(IN) :: stroke_style
    
    INTEGER(D2D1_LINE_JOIN) :: line_join
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetLineJoin_intf(this) RESULT(line_join)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_LINE_JOIN
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_LINE_JOIN) :: line_join
        !DEC$ ATTRIBUTES STDCALL :: GetLineJoin_intf
      END FUNCTION GetLineJoin_intf
    END INTERFACE
    
    TYPE(ID2D1StrokeStyleVTbl), POINTER :: ptbl
    PROCEDURE(GetLineJoin_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(stroke_style), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetLineJoin, proc)
    line_join = proc(stroke_style%object)
    
  END FUNCTION GetLineJoin
  
  
  !*****************************************************************************
  
  FUNCTION GetDashOffset(stroke_style) RESULT(dash_offset)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1StrokeStyle), INTENT(IN) :: stroke_style
    
    REAL(FLOAT) :: dash_offset
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetDashOffset_intf(this) RESULT(dash_offset)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT) :: dash_offset
        !DEC$ ATTRIBUTES STDCALL :: GetDashOffset_intf
      END FUNCTION GetDashOffset_intf
    END INTERFACE
    
    TYPE(ID2D1StrokeStyleVTbl), POINTER :: ptbl
    PROCEDURE(GetDashOffset_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(stroke_style), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetDashOffset, proc)
    dash_offset = proc(stroke_style%object)
    
  END FUNCTION GetDashOffset
  
  
  !*****************************************************************************
  
  FUNCTION GetDashStyle(stroke_style) RESULT(dash_style)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1StrokeStyle), INTENT(IN) :: stroke_style
    
    INTEGER(D2D1_DASH_STYLE) :: dash_style
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetDashStyle_intf(this) RESULT(dash_style)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_DASH_STYLE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_DASH_STYLE) :: dash_style
        !DEC$ ATTRIBUTES STDCALL :: GetDashStyle_intf
      END FUNCTION GetDashStyle_intf
    END INTERFACE
    
    TYPE(ID2D1StrokeStyleVTbl), POINTER :: ptbl
    PROCEDURE(GetDashStyle_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(stroke_style), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetDashStyle, proc)
    dash_style = proc(stroke_style%object)
    
  END FUNCTION GetDashStyle
  
  
  !*****************************************************************************
  
  FUNCTION GetDashesCount(stroke_style) RESULT(dashes_count)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1StrokeStyle), INTENT(IN) :: stroke_style
    
    INTEGER(UINT32) :: dashes_count
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetDashesCount_intf(this) RESULT(dashes_count)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: UINT32
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(UINT32) :: dashes_count
        !DEC$ ATTRIBUTES STDCALL :: GetDashesCount_intf
      END FUNCTION GetDashesCount_intf
    END INTERFACE
    
    TYPE(ID2D1StrokeStyleVTbl), POINTER :: ptbl
    PROCEDURE(GetDashesCount_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(stroke_style), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetDashesCount, proc)
    dashes_count = proc(stroke_style%object)
    
  END FUNCTION GetDashesCount
  
  
  !*****************************************************************************
  
  SUBROUTINE GetDashes(stroke_style, dashes)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_INT
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1StrokeStyle), INTENT(IN) :: stroke_style
    
    REAL(FLOAT), INTENT(OUT), ALLOCATABLE :: dashes(:)
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetDashes_intf(this, dashes, dashes_count)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(OUT) :: dashes(*)
        !DEC$ ATTRIBUTES REFERENCE :: dashes
        INTEGER(C_INT), INTENT(IN) :: dashes_count
        !DEC$ ATTRIBUTES STDCALL :: GetDashesCount_intf
      END SUBROUTINE GetDashes_intf
    END INTERFACE
    
    INTEGER(C_INT) :: count
    
    TYPE(ID2D1StrokeStyleVTbl), POINTER :: ptbl
    PROCEDURE(GetDashes_intf), POINTER :: proc
    
    !***************************************************************************
    
    count = GetDashesCount(stroke_style)
    ALLOCATE(dashes(count))
    
    CALL C_F_POINTER(VTable(stroke_style), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetDashes, proc)
    CALL proc(stroke_style%object, dashes, count)
    
  END SUBROUTINE GetDashes
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1Geometry
  
  
  !*****************************************************************************
  
  FUNCTION d2d1geometry_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1geometry_IID
    
    !***************************************************************************
    
    d2d1geometry_IID = GUID(  &
        INT(Z'2CD906A1', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1geometry_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE GetBounds(geometry, bounds, hr, worldTransform)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    TYPE(D2D1_RECT_F), INTENT(OUT) :: bounds
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET :: worldTransform
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetBounds_intf(this, worldTransform, bounds) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RECT_F
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        TYPE(D2D1_RECT_F), INTENT(OUT) :: bounds
        !DEC$ ATTRIBUTES REFERENCE :: bounds
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: GetBounds_intf
      END FUNCTION GetBounds_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_transform
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(GetBounds_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetBounds, proc)
    hr = proc(geometry%object, local_transform, bounds)
    
  END SUBROUTINE GetBounds
  
  
  !*****************************************************************************
  
  SUBROUTINE GetWidenedBounds( geometry, strokeWidth, bounds, hr,  &
      strokeStyle, worldTransform, flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    REAL(FLOAT), INTENT(IN) :: strokeWidth
    TYPE(D2D1_RECT_F), INTENT(OUT) :: bounds
    INTEGER(HRESULT), INTENT(OUT) :: hr
    CLASS(ID2D1StrokeStyle), INTENT(IN), OPTIONAL, TARGET :: strokeStyle
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET :: worldTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetWidenedBounds_intf( this, strokeWidth, strokeStyle,  &
          worldTransform, flatteningTolerance, bounds ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPORT :: D2D1_RECT_F
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(IN) :: strokeWidth
        TYPE(C_PTR), INTENT(IN) :: strokeStyle
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        TYPE(D2D1_RECT_F), INTENT(OUT) :: bounds
        !DEC$ ATTRIBUTES REFERENCE :: bounds
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: GetWidenedBounds_intf
      END FUNCTION GetWidenedBounds_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_strokeStyle
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(GetWidenedBounds_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(strokeStyle)) THEN
      local_strokeStyle = strokeStyle%object
    ELSE
      local_strokeStyle = C_NULL_PTR
    END IF
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetWidenedBounds, proc)
    hr = proc( geometry%object, strokeWidth, local_strokeStyle,  &
        local_transform, local_tolerance, bounds )
    
  END SUBROUTINE GetWidenedBounds
  
  
  !*****************************************************************************
  
  SUBROUTINE StrokeContainsPoint( geometry, point, strokeWidth,  &
      contains, hr, strokeStyle, worldTransform, flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    TYPE(D2D1_POINT_2F), INTENT(IN) :: point
    REAL(FLOAT), INTENT(IN) :: strokeWidth
    LOGICAL, INTENT(OUT) :: contains
    INTEGER(HRESULT), INTENT(OUT) :: hr
    CLASS(ID2D1StrokeStyle), INTENT(IN), OPTIONAL, TARGET :: strokeStyle
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET :: worldTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION StrokeContainsPoint_intf( this, point, strokeWidth, strokeStyle,  &
          worldTransform, flatteningTolerance, contains ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_POINT_2F
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: point
        REAL(FLOAT), INTENT(IN) :: strokeWidth
        TYPE(C_PTR), INTENT(IN) :: strokeStyle
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        INTEGER(C_INT), INTENT(OUT) :: contains
        !DEC$ ATTRIBUTES REFERENCE :: contains
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: StrokeContainsPoint_intf
      END FUNCTION StrokeContainsPoint_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_strokeStyle
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    INTEGER(C_INT) :: int_contains
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(StrokeContainsPoint_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(strokeStyle)) THEN
      local_strokeStyle = strokeStyle%object
    ELSE
      local_strokeStyle = C_NULL_PTR
    END IF
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%StrokeContainsPoint, proc)
    hr = proc( geometry%object, point, strokeWidth, local_strokeStyle,  &
        local_transform, local_tolerance, int_contains )
    
    contains = int_contains /= 0
    
  END SUBROUTINE StrokeContainsPoint
  
  
  !*****************************************************************************
  
  SUBROUTINE FillContainsPoint( geometry, point, contains, hr,  &
      worldTransform, flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    TYPE(D2D1_POINT_2F), INTENT(IN) :: point
    LOGICAL, INTENT(OUT) :: contains
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET :: worldTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION FillContainsPoint_intf( this, point, worldTransform,  &
          flatteningTolerance, contains ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_POINT_2F
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: point
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        INTEGER(C_INT), INTENT(OUT) :: contains
        !DEC$ ATTRIBUTES REFERENCE :: contains
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: FillContainsPoint_intf
      END FUNCTION FillContainsPoint_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    INTEGER(C_INT) :: int_contains
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(FillContainsPoint_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%FillContainsPoint, proc)
    hr = proc( geometry%object, point, local_transform,  &
        local_tolerance, int_contains )
    
    contains = int_contains /= 0
    
  END SUBROUTINE FillContainsPoint
  
  
  !*****************************************************************************
  
  SUBROUTINE CompareWithGeometry( geometry, inputGeometry, relation, hr,  &
      inputGeometryTransform, flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    CLASS(ID2D1Geometry), INTENT(IN) :: inputGeometry
    INTEGER(D2D1_GEOMETRY_RELATION), INTENT(OUT) :: relation
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET  &
        :: inputGeometryTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CompareWithGeometry_intf( this, inputGeometry,  &
          inputGeometryTransform, flatteningTolerance, relation ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_GEOMETRY_RELATION
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: inputGeometry
        TYPE(C_PTR), INTENT(IN) :: inputGeometryTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        INTEGER(D2D1_GEOMETRY_RELATION), INTENT(OUT) :: relation
        !DEC$ ATTRIBUTES REFERENCE :: relation
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CompareWithGeometry_intf
      END FUNCTION CompareWithGeometry_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(CompareWithGeometry_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(inputGeometryTransform)) THEN
      local_transform = C_LOC(inputGeometryTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CompareWithGeometry, proc)
    hr = proc( geometry%object, inputGeometry%object, local_transform,  &
        local_tolerance, relation )
    
  END SUBROUTINE CompareWithGeometry
  
  
  !*****************************************************************************
  
  SUBROUTINE Simplify( geometry, simplificationOption, geometrySink, hr,  &
      worldTransform, flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    INTEGER(D2D1_GEOMETRY_SIMPLIFICATION_OPTION), INTENT(IN)  &
        :: simplificationOption
    CLASS(ID2D1Geometry), INTENT(IN) :: geometrySink
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET  &
        :: worldTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION Simplify_intf( this, simplificationOption,  &
          worldTransform, flatteningTolerance, geometrySink ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_GEOMETRY_SIMPLIFICATION_OPTION
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_GEOMETRY_SIMPLIFICATION_OPTION), INTENT(IN)  &
            :: simplificationOption
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        TYPE(C_PTR), INTENT(IN) :: geometrySink
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: Simplify_intf
      END FUNCTION Simplify_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(Simplify_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Simplify, proc)
    hr = proc( geometry%object, simplificationOption, local_transform,  &
        local_tolerance, geometrySink%object )
    
  END SUBROUTINE Simplify
  
  
  !*****************************************************************************
  
  SUBROUTINE Tessellate( geometry, tessellationSink, hr, worldTransform,  &
      flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    CLASS(ID2D1TessellationSink), INTENT(IN) :: tessellationSink
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET  &
        :: worldTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION Tessellate_intf( this, worldTransform, flatteningTolerance,  &
          tessellationSink ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        TYPE(C_PTR), INTENT(IN) :: tessellationSink
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: Tessellate_intf
      END FUNCTION Tessellate_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(Tessellate_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Tessellate, proc)
    hr = proc( geometry%object, local_transform, local_tolerance,  &
        tessellationSink%object )
    
  END SUBROUTINE Tessellate
  
  
  !*****************************************************************************
  
  SUBROUTINE CombineWithGeometry( geometry, inputGeometry, combineMode,  &
      geometrySink, hr, inputGeometryTransform, flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    CLASS(ID2D1Geometry), INTENT(IN) :: inputGeometry
    INTEGER(D2D1_COMBINE_MODE), INTENT(IN) :: combineMode
    CLASS(ID2D1SimplifiedGeometrySink), INTENT(IN) :: geometrySink
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET  &
        :: inputGeometryTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CombineWithGeometry_intf( this, inputGeometry, combineMode,  &
          inputGeometryTransform, flatteningTolerance, geometrySink )  &
          RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_COMBINE_MODE
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: inputGeometry
        INTEGER(D2D1_COMBINE_MODE), INTENT(IN) :: combineMode
        TYPE(C_PTR), INTENT(IN) :: inputGeometryTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        TYPE(C_PTR), INTENT(IN) :: geometrySink
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CombineWithGeometry_intf
      END FUNCTION CombineWithGeometry_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(CombineWithGeometry_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(inputGeometryTransform)) THEN
      local_transform = C_LOC(inputGeometryTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CombineWithGeometry, proc)
    hr = proc( geometry%object, inputGeometry%object, combineMode,  &
        local_transform, local_tolerance, geometrySink%object )
    
  END SUBROUTINE CombineWithGeometry
  
  
  !*****************************************************************************
  
  SUBROUTINE Outline( geometry, geometrySink, hr, worldTransform,  &
      flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    CLASS(ID2D1SimplifiedGeometrySink), INTENT(IN) :: geometrySink
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET  &
        :: worldTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION Outline_intf( this, worldTransform, flatteningTolerance,  &
          geometrySink ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        TYPE(C_PTR), INTENT(IN) :: geometrySink
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: Outline_intf
      END FUNCTION Outline_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(Outline_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Outline, proc)
    hr = proc( geometry%object, local_transform, local_tolerance,  &
        geometrySink%object )
    
  END SUBROUTINE Outline
  
  
  !*****************************************************************************
  
  SUBROUTINE ComputeArea( geometry, area, hr, worldTransform,  &
      flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    REAL(FLOAT), INTENT(OUT) :: area
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET  &
        :: worldTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION ComputeArea_intf( this, worldTransform, flatteningTolerance,  &
          area ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        REAL(FLOAT), INTENT(OUT) :: area
        !DEC$ ATTRIBUTES REFERENCE :: area
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: ComputeArea_intf
      END FUNCTION ComputeArea_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(ComputeArea_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%ComputeArea, proc)
    hr = proc(geometry%object, local_transform, local_tolerance, area)
    
  END SUBROUTINE ComputeArea
  
  
  !*****************************************************************************
  
  SUBROUTINE ComputeLength( geometry, length, hr, worldTransform,  &
      flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    REAL(FLOAT), INTENT(OUT) :: length
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET  &
        :: worldTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION ComputeLength_intf( this, worldTransform,  &
          flatteningTolerance, length ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        REAL(FLOAT), INTENT(OUT) :: length
        !DEC$ ATTRIBUTES REFERENCE :: length
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: ComputeLength_intf
      END FUNCTION ComputeLength_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(ComputeLength_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%ComputeArea, proc)
    hr = proc(geometry%object, local_transform, local_tolerance, length)
    
  END SUBROUTINE ComputeLength
  
  
  !*****************************************************************************
  
  SUBROUTINE ComputePointAtLength( geometry, length, hr,  &
      worldTransform, flatteningTolerance, point, unitTangentVector )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    REAL(FLOAT), INTENT(IN) :: length
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET  &
        :: worldTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    TYPE(D2D1_POINT_2F), INTENT(OUT), OPTIONAL, TARGET :: point
    TYPE(D2D1_POINT_2F), INTENT(OUT), OPTIONAL, TARGET :: unitTangentVector
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION ComputePointAtLength_intf( this, length, worldTransform,  &
          flatteningTolerance, point, unitTangentVector ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(IN) :: length
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        TYPE(C_PTR), INTENT(IN) :: point
        TYPE(C_PTR), INTENT(IN) :: unitTangentVector
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: ComputePointAtLength_intf
      END FUNCTION ComputePointAtLength_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    TYPE(C_PTR) :: local_point
    TYPE(C_PTR) :: local_tangent
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(ComputePointAtLength_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    IF (PRESENT(point)) THEN
      local_point = C_LOC(point)
    ELSE
      local_point = C_NULL_PTR
    END IF
    
    IF (PRESENT(unitTangentVector)) THEN
      local_tangent = C_LOC(unitTangentVector)
    ELSE
      local_tangent = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%ComputePointAtLength, proc)
    hr = proc( geometry%object, length, local_transform,  &
        local_tolerance, local_point, local_tangent )
    
  END SUBROUTINE ComputePointAtLength
  
  
  !*****************************************************************************
  
  SUBROUTINE Widen( geometry, strokeWidth, geometrySink, hr,  &
      strokeStyle, worldTransform, flatteningTolerance )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    REAL(FLOAT), INTENT(IN) :: strokeWidth
    TYPE(C_PTR), INTENT(IN) :: geometrySink
    INTEGER(HRESULT), INTENT(OUT) :: hr
    CLASS(ID2D1StrokeStyle), INTENT(IN), OPTIONAL :: strokeStyle
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN), OPTIONAL, TARGET  &
        :: worldTransform
    REAL(FLOAT), INTENT(IN), OPTIONAL :: flatteningTolerance
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION Widen_intf( this, strokeWidth, strokeStyle, worldTransform,  &
          flatteningTolerance, geometrySink ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(IN) :: strokeWidth
        TYPE(C_PTR), INTENT(IN) :: strokeStyle
        TYPE(C_PTR), INTENT(IN) :: worldTransform
        REAL(FLOAT), INTENT(IN) :: flatteningTolerance
        TYPE(C_PTR), INTENT(IN) :: geometrySink
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: Widen_intf
      END FUNCTION Widen_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_style
    TYPE(C_PTR) :: local_transform
    REAL(FLOAT) :: local_tolerance
    
    TYPE(ID2D1GeometryVTbl), POINTER :: ptbl
    PROCEDURE(Widen_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(strokeStyle)) THEN
      local_style = strokeStyle%object
    ELSE
      local_style = C_NULL_PTR
    END IF
    
    IF (PRESENT(worldTransform)) THEN
      local_transform = C_LOC(worldTransform)
    ELSE
      local_transform = C_NULL_PTR
    END IF
    
    IF (PRESENT(flatteningTolerance)) THEN
      local_tolerance = flatteningTolerance
    ELSE
      local_tolerance = D2D1_DEFAULT_FLATTENING_TOLERANCE
    END IF
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Widen, proc)
    hr = proc( geometry%object, strokeWidth, local_style, local_transform,  &
        local_tolerance, geometrySink )
    
  END SUBROUTINE Widen
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1RectangleGeometry
  
  
  !*****************************************************************************
  
  FUNCTION d2d1rectanglegeometry_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1rectanglegeometry_IID
    
    !***************************************************************************
    
    d2d1rectanglegeometry_IID = GUID(  &
        INT(Z'2CD906A2', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1rectanglegeometry_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE GetRect(geometry, rect)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1RectangleGeometry), INTENT(IN) :: geometry
    TYPE(D2D1_RECT_F), INTENT(OUT) :: rect
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetRect_intf(this, rect)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RECT_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_RECT_F), INTENT(OUT) :: rect
        !DEC$ ATTRIBUTES REFERENCE :: rect
        !DEC$ ATTRIBUTES STDCALL :: GetRect_intf
      END SUBROUTINE GetRect_intf
    END INTERFACE
    
    TYPE(ID2D1RectangleGeometryVTbl), POINTER :: ptbl
    PROCEDURE(GetRect_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetRect, proc)
    CALL proc(geometry%object, rect)
    
  END SUBROUTINE GetRect
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1RoundedRectangleGeometry
  
  
  !*****************************************************************************
  
  FUNCTION d2d1roundedrectanglegeometry_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1roundedrectanglegeometry_IID
    
    !***************************************************************************
    
    d2d1roundedrectanglegeometry_IID = GUID(  &
        INT(Z'2CD906A3', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1roundedrectanglegeometry_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE GetRoundedRect(geometry, roundedRect)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1RoundedRectangleGeometry), INTENT(IN) :: geometry
    TYPE(D2D1_ROUNDED_RECT), INTENT(OUT) :: roundedRect
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetRoundedRect_intf(this, roundedRect)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ROUNDED_RECT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_ROUNDED_RECT), INTENT(OUT) :: roundedRect
        !DEC$ ATTRIBUTES REFERENCE :: roundedRect
        !DEC$ ATTRIBUTES STDCALL :: GetRoundedRect_intf
      END SUBROUTINE GetRoundedRect_intf
    END INTERFACE
    
    TYPE(ID2D1RoundedRectangleGeometryVTbl), POINTER :: ptbl
    PROCEDURE(GetRoundedRect_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetRoundedRect, proc)
    CALL proc(geometry%object, roundedRect)
    
  END SUBROUTINE GetRoundedRect
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1EllipseGeometry
  
  
  !*****************************************************************************
  
  FUNCTION d2d1ellipsegeometry_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1ellipsegeometry_IID
    
    !***************************************************************************
    
    d2d1ellipsegeometry_IID = GUID(  &
        INT(Z'2CD906A4', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1ellipsegeometry_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE GetEllipse(geometry, ellipse)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1EllipseGeometry), INTENT(IN) :: geometry
    TYPE(D2D1_ELLIPSE), INTENT(OUT) :: ellipse
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetEllipse_intf(this, ellipse)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ELLIPSE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_ELLIPSE), INTENT(OUT) :: ellipse
        !DEC$ ATTRIBUTES REFERENCE :: ellipse
        !DEC$ ATTRIBUTES STDCALL :: GetEllipse_intf
      END SUBROUTINE GetEllipse_intf
    END INTERFACE
    
    TYPE(ID2D1EllipseGeometryVTbl), POINTER :: ptbl
    PROCEDURE(GetEllipse_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetEllipse, proc)
    CALL proc(geometry%object, ellipse)
    
  END SUBROUTINE GetEllipse
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1GeometryGroup
  
  
  !*****************************************************************************
  
  FUNCTION d2d1geometrygroup_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1geometrygroup_IID
    
    !***************************************************************************
    
    d2d1geometrygroup_IID = GUID(  &
        INT(Z'2CD906A6', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1geometrygroup_IID
  
  
  !*****************************************************************************
  
  FUNCTION GetFillMode(geometry) RESULT(fill_mode)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GeometryGroup), INTENT(IN) :: geometry
    
    INTEGER(D2D1_FILL_MODE) :: fill_mode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetFillMode_intf(this) RESULT(fill_mode)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_FILL_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_FILL_MODE) :: fill_mode
        !DEC$ ATTRIBUTES STDCALL :: GetFillMode_intf
      END FUNCTION GetFillMode_intf
    END INTERFACE
    
    TYPE(ID2D1GeometryGroupVTbl), POINTER :: ptbl
    PROCEDURE(GetFillMode_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetFillMode, proc)
    fill_mode = proc(geometry%object)
    
  END FUNCTION GetFillMode
  
  
  !*****************************************************************************
  
  FUNCTION GetSourceGeometryCount(geometry) RESULT(count)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GeometryGroup), INTENT(IN) :: geometry
    
    INTEGER(UINT32) :: count
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetSourceGeometryCount_intf(this) RESULT(count)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: UINT32
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(UINT32) :: count
        !DEC$ ATTRIBUTES STDCALL :: GetSourceGeometryCount_intf
      END FUNCTION GetSourceGeometryCount_intf
    END INTERFACE
    
    TYPE(ID2D1GeometryGroupVTbl), POINTER :: ptbl
    PROCEDURE(GetSourceGeometryCount_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetSourceGeometryCount, proc)
    count = proc(geometry%object)
    
  END FUNCTION GetSourceGeometryCount
  
  
  !*****************************************************************************
  
  SUBROUTINE GetSourceGeometries(geometry, geometries)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_INT
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GeometryGroup), INTENT(IN) :: geometry
    CLASS(ID2D1Geometry), INTENT(OUT), ALLOCATABLE :: geometries(:)
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetSourceGeometries_intf(this, geometries, geometriesCount)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: UINT32
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: geometries(*)
        INTEGER(C_INT), INTENT(IN) :: geometriesCount
        !DEC$ ATTRIBUTES STDCALL :: GetSourceGeometries_intf
      END SUBROUTINE GetSourceGeometries_intf
    END INTERFACE
    
    INTEGER(C_INT) :: count
    
    TYPE(ID2D1GeometryGroupVTbl), POINTER :: ptbl
    PROCEDURE(GetSourceGeometries_intf), POINTER :: proc
    
    !***************************************************************************
    
    count = GetSourceGeometryCount(geometry)
    ALLOCATE(geometries(count))
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetSourceGeometries, proc)
    CALL proc(geometry%object, geometries%object, count)
    
  END SUBROUTINE GetSourceGeometries
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1TransformedGeometry
  
  
  !*****************************************************************************
  
  FUNCTION d2d1transformedgeometry_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1transformedgeometry_IID
    
    !***************************************************************************
    
    d2d1transformedgeometry_IID = GUID(  &
        INT(Z'2CD906BB', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1transformedgeometry_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE GetSourceGeometry(geometry, sourceGeometry)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1TransformedGeometry), INTENT(IN) :: geometry
    TYPE(ID2D1Geometry), INTENT(OUT) :: sourceGeometry
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetSourceGeometry_intf(this, sourceGeometry)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: UINT32
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: sourceGeometry
        !DEC$ ATTRIBUTES REFERENCE :: sourceGeometry
        !DEC$ ATTRIBUTES STDCALL :: GetSourceGeometry_intf
      END SUBROUTINE GetSourceGeometry_intf
    END INTERFACE
    
    TYPE(ID2D1TransformedGeometryVTbl), POINTER :: ptbl
    PROCEDURE(GetSourceGeometry_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetSourceGeometry, proc)
    CALL proc(geometry%object, sourceGeometry%object)
    
  END SUBROUTINE GetSourceGeometry
  
  
  !*****************************************************************************
  
  FUNCTION GetTransform_tg(geometry) RESULT(transform)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1TransformedGeometry), INTENT(IN) :: geometry
    
    ! Function result.
    TYPE(D2D1_MATRIX_3X2_F) :: transform
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetTransform_intf(this, transform)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_MATRIX_3X2_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_MATRIX_3X2_F), INTENT(OUT) :: transform
        !DEC$ ATTRIBUTES REFERENCE :: transform
        !DEC$ ATTRIBUTES STDCALL :: GetTransform_intf
      END SUBROUTINE GetTransform_intf
    END INTERFACE
    
    TYPE(ID2D1TransformedGeometryVTbl), POINTER :: ptbl
    PROCEDURE(GetTransform_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetTransform, proc)
    CALL proc(geometry%object, transform)
    
  END FUNCTION GetTransform_tg
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1SimplifiedGeometrySink
  
  
  !*****************************************************************************
  
  FUNCTION d2d1simpliedgeometrysink_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1simpliedgeometrysink_IID
    
    !***************************************************************************
    
    d2d1simpliedgeometrysink_IID = GUID(  &
        INT(Z'2CD9069E', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1simpliedgeometrysink_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE SetFillMode(sink, fillMode)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1SimplifiedGeometrySink), INTENT(IN) :: sink
    INTEGER(D2D1_FILL_MODE), INTENT(IN) :: fillMode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetFillMode_intf(this, fillMode)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_FILL_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_FILL_MODE), INTENT(IN) :: fillMode
        !DEC$ ATTRIBUTES STDCALL :: SetFillMode_intf
      END SUBROUTINE SetFillMode_intf
    END INTERFACE
    
    TYPE(ID2D1SimplifiedGeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(SetFillMode_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetFillMode, proc)
    CALL proc(sink%object, fillMode)
    
  END SUBROUTINE SetFillMode
  
  
  !*****************************************************************************
  
  SUBROUTINE SetSegmentFlags(sink, vertexFlags)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1SimplifiedGeometrySink), INTENT(IN) :: sink
    INTEGER(D2D1_PATH_SEGMENT), INTENT(IN) :: vertexFlags
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetSegmentFlags_intf(this, vertexFlags)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_PATH_SEGMENT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_PATH_SEGMENT), INTENT(IN) :: vertexFlags
        !DEC$ ATTRIBUTES STDCALL :: SetSegmentFlags_intf
      END SUBROUTINE SetSegmentFlags_intf
    END INTERFACE
    
    TYPE(ID2D1SimplifiedGeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(SetSegmentFlags_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetSegmentFlags, proc)
    CALL proc(sink%object, vertexFlags)
    
  END SUBROUTINE SetSegmentFlags
  
  
  !*****************************************************************************
  
  SUBROUTINE BeginFigure(sink, startPoint, figureBegin)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1SimplifiedGeometrySink), INTENT(IN) :: sink
    TYPE(D2D1_POINT_2F), INTENT(IN) :: startPoint
    INTEGER(D2D1_FIGURE_BEGIN), INTENT(IN) :: figureBegin
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE BeginFigure_intf(this, startPoint, figureBegin)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPORT :: D2D1_FIGURE_BEGIN
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: startPoint
        INTEGER(D2D1_FIGURE_BEGIN), INTENT(IN) :: figureBegin
        !DEC$ ATTRIBUTES STDCALL :: BeginFigure_intf
      END SUBROUTINE BeginFigure_intf
    END INTERFACE
    
    TYPE(ID2D1SimplifiedGeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(BeginFigure_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%BeginFigure, proc)
    CALL proc(sink%object, startPoint, figureBegin)
    
  END SUBROUTINE BeginFigure
  
  
  !*****************************************************************************
  
  SUBROUTINE AddLines(sink, points)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1SimplifiedGeometrySink), INTENT(IN) :: sink
    TYPE(D2D1_POINT_2F), INTENT(IN) :: points(:)
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE AddLines_intf(this, points, pointsCount)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: points(*)
        INTEGER(C_INT), INTENT(IN) :: pointsCount
        !DEC$ ATTRIBUTES STDCALL :: AddLines_intf
      END SUBROUTINE AddLines_intf
    END INTERFACE
    
    TYPE(ID2D1SimplifiedGeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(AddLines_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%AddLines, proc)
    CALL proc(sink%object, points, SIZE(points, KIND=C_INT))
    
  END SUBROUTINE AddLines
  
  
  !*****************************************************************************
  
  SUBROUTINE AddBeziers(sink, beziers)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1SimplifiedGeometrySink), INTENT(IN) :: sink
    TYPE(D2D1_BEZIER_SEGMENT), INTENT(IN) :: beziers(:)
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE AddBeziers_intf(this, beziers, beziersCount)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_BEZIER_SEGMENT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_BEZIER_SEGMENT), INTENT(IN) :: beziers(*)
        INTEGER(C_INT), INTENT(IN) :: beziersCount
        !DEC$ ATTRIBUTES STDCALL :: AddBeziers_intf
      END SUBROUTINE AddBeziers_intf
    END INTERFACE
    
    TYPE(ID2D1SimplifiedGeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(AddBeziers_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%AddBeziers, proc)
    CALL proc(sink%object, beziers, SIZE(beziers, KIND=C_INT))
    
  END SUBROUTINE AddBeziers
  
  
  !*****************************************************************************
  
  SUBROUTINE EndFigure(sink, figureEnd)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1SimplifiedGeometrySink), INTENT(IN) :: sink
    INTEGER(D2D1_FIGURE_END), INTENT(IN) :: figureEnd
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE EndFigure_intf(this, figureEnd)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_FIGURE_END
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_FIGURE_END), INTENT(IN) :: figureEnd
        !DEC$ ATTRIBUTES STDCALL :: EndFigure_intf
      END SUBROUTINE EndFigure_intf
    END INTERFACE
    
    TYPE(ID2D1SimplifiedGeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(EndFigure_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%EndFigure, proc)
    CALL proc(sink%object, figureEnd)
    
  END SUBROUTINE EndFigure
  
  
  !*****************************************************************************
  
  SUBROUTINE Close_sgs(sink, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1SimplifiedGeometrySink), INTENT(IN) :: sink
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION Close_intf(this) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: Close_intf
      END FUNCTION Close_intf
    END INTERFACE
    
    TYPE(ID2D1SimplifiedGeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(Close_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Close, proc)
    hr = proc(sink%object)
    
  END SUBROUTINE Close_sgs
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1GeometrySink
  
  
  !*****************************************************************************
  
  FUNCTION d2d1geometrysink_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1geometrysink_IID
    
    !***************************************************************************
    
    d2d1geometrysink_IID = GUID(  &
        INT(Z'2CD9069F', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1geometrysink_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE AddLine(sink, point)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GeometrySink), INTENT(IN) :: sink
    TYPE(D2D1_POINT_2F), INTENT(IN) :: point
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE AddLine_intf(this, point)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: point
        !DEC$ ATTRIBUTES STDCALL :: AddLine_intf
      END SUBROUTINE AddLine_intf
    END INTERFACE
    
    TYPE(ID2D1GeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(AddLine_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%AddLine, proc)
    CALL proc(sink%object, point)
    
  END SUBROUTINE AddLine
  
  
  !*****************************************************************************
  
  SUBROUTINE AddBezier(sink, bezier)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GeometrySink), INTENT(IN) :: sink
    TYPE(D2D1_BEZIER_SEGMENT), INTENT(IN) :: bezier
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE AddBezier_intf(this, bezier)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_BEZIER_SEGMENT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_BEZIER_SEGMENT), INTENT(IN) :: bezier
        !DEC$ ATTRIBUTES REFERENCE :: bezier
        !DEC$ ATTRIBUTES STDCALL :: AddBezier_intf
      END SUBROUTINE AddBezier_intf
    END INTERFACE
    
    TYPE(ID2D1GeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(AddBezier_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%AddBezier, proc)
    CALL proc(sink%object, bezier)
    
  END SUBROUTINE AddBezier
  
  
  !*****************************************************************************
  
  SUBROUTINE AddQuadraticBezier(sink, bezier)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GeometrySink), INTENT(IN) :: sink
    TYPE(D2D1_QUADRATIC_BEZIER_SEGMENT), INTENT(IN) :: bezier
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE AddQuadraticBezier_intf(this, bezier)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_QUADRATIC_BEZIER_SEGMENT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_QUADRATIC_BEZIER_SEGMENT), INTENT(IN) :: bezier
        !DEC$ ATTRIBUTES REFERENCE :: bezier
        !DEC$ ATTRIBUTES STDCALL :: AddQuadraticBezier_intf
      END SUBROUTINE AddQuadraticBezier_intf
    END INTERFACE
    
    TYPE(ID2D1GeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(AddQuadraticBezier_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%AddQuadraticBezier, proc)
    CALL proc(sink%object, bezier)
    
  END SUBROUTINE AddQuadraticBezier
  
  
  !*****************************************************************************
  
  SUBROUTINE AddQuadraticBeziers(sink, beziers)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GeometrySink), INTENT(IN) :: sink
    TYPE(D2D1_QUADRATIC_BEZIER_SEGMENT), INTENT(IN) :: beziers(:)
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE AddQuadraticBeziers_intf(this, beziers, beziersCount)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_QUADRATIC_BEZIER_SEGMENT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_QUADRATIC_BEZIER_SEGMENT), INTENT(IN) :: beziers(*)
        INTEGER(C_INT), INTENT(IN) :: beziersCount
        !DEC$ ATTRIBUTES STDCALL :: AddQuadraticBeziers_intf
      END SUBROUTINE AddQuadraticBeziers_intf
    END INTERFACE
    
    TYPE(ID2D1GeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(AddQuadraticBeziers_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%AddQuadraticBezier, proc)
    CALL proc(sink%object, beziers, SIZE(beziers, KIND=C_INT))
    
  END SUBROUTINE AddQuadraticBeziers
  
  
  !*****************************************************************************
  
  SUBROUTINE AddArc(sink, arc)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1GeometrySink), INTENT(IN) :: sink
    TYPE(D2D1_ARC_SEGMENT), INTENT(IN) :: arc
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE AddArc_intf(this, arc)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ARC_SEGMENT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_ARC_SEGMENT), INTENT(IN) :: arc
        !DEC$ ATTRIBUTES REFERENCE :: arc
        !DEC$ ATTRIBUTES STDCALL :: AddArc_intf
      END SUBROUTINE AddArc_intf
    END INTERFACE
    
    TYPE(ID2D1GeometrySinkVTbl), POINTER :: ptbl
    PROCEDURE(AddArc_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%AddArc, proc)
    CALL proc(sink%object, arc)
    
  END SUBROUTINE AddArc
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1TessellationSink
  
  
  !*****************************************************************************
  
  FUNCTION d2d1tessellationsink_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1tessellationsink_IID
    
    !***************************************************************************
    
    d2d1tessellationsink_IID = GUID(  &
        INT(Z'2CD906C1', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1tessellationsink_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE AddTriangles(sink, triangles)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1TessellationSink), INTENT(IN) :: sink
    TYPE(D2D1_TRIANGLE), INTENT(IN) :: triangles(:)
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE AddTriangles_intf(this, triangles, trianglesCount)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_TRIANGLE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_TRIANGLE), INTENT(IN) :: triangles(*)
        INTEGER(C_INT), INTENT(IN) :: trianglesCount
        !DEC$ ATTRIBUTES STDCALL :: AddTriangles_intf
      END SUBROUTINE AddTriangles_intf
    END INTERFACE
    
    TYPE(ID2D1TessellationSinkVTbl), POINTER :: ptbl
    PROCEDURE(AddTriangles_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%AddTriangles, proc)
    CALL proc(sink%object, triangles, SIZE(triangles, KIND=C_INT))
    
  END SUBROUTINE AddTriangles
  
  
  !*****************************************************************************
  
  SUBROUTINE Close_ts(sink, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1TessellationSink), INTENT(IN) :: sink
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION Close_intf(this) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: Close_intf
      END FUNCTION Close_intf
    END INTERFACE
    
    TYPE(ID2D1TessellationSinkVTbl), POINTER :: ptbl
    PROCEDURE(Close_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(sink), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Close, proc)
    hr = proc(sink%object)
    
  END SUBROUTINE Close_ts
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1PathGeometry
  
  
  !*****************************************************************************
  
  FUNCTION d2d1pathgeometry_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1pathgeometry_IID
    
    !***************************************************************************
    
    d2d1pathgeometry_IID = GUID(  &
        INT(Z'2CD906A5', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1pathgeometry_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE Open_pg(geometry, geometrySink, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1PathGeometry), INTENT(IN) :: geometry
    TYPE(ID2D1GeometrySink), INTENT(OUT) :: geometrySink
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    
    ABSTRACT INTERFACE
      FUNCTION Open_intf(this, geometrySink)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: geometrySink
        !DEC$ ATTRIBUTES REFERENCE :: geometrySink
        INTEGER(HRESULT) :: Open_intf
        !DEC$ ATTRIBUTES STDCALL :: Open_intf
      END FUNCTION Open_intf
    END INTERFACE
    
    TYPE(ID2D1PathGeometryVTbl), POINTER :: ptbl
    PROCEDURE(Open_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Open, proc)
    hr = proc(geometry%object, geometrySink%object)
    
  END SUBROUTINE Open_pg
  
  
  !*****************************************************************************
  
  SUBROUTINE Stream(geometry, geometrySink, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1PathGeometry), INTENT(IN) :: geometry
    CLASS(ID2D1GeometrySink), INTENT(IN) :: geometrySink
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    
    ABSTRACT INTERFACE
      FUNCTION Stream_intf(this, geometrySink)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: geometrySink
        !DEC$ ATTRIBUTES REFERENCE :: geometrySink
        INTEGER(HRESULT) :: Stream_intf
        !DEC$ ATTRIBUTES STDCALL :: Stream_intf
      END FUNCTION Stream_intf
    END INTERFACE
    
    TYPE(ID2D1PathGeometryVTbl), POINTER :: ptbl
    PROCEDURE(Stream_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Stream, proc)
    hr = proc(geometry%object, geometrySink%object)
    
  END SUBROUTINE Stream
  
  
  !*****************************************************************************
  
  SUBROUTINE GetSegmentCount(geometry, count, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1PathGeometry), INTENT(IN) :: geometry
    INTEGER(UINT32), INTENT(OUT) :: count
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    
    ABSTRACT INTERFACE
      FUNCTION GetSegmentCount_intf(this, count)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: UINT32
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(UINT32), INTENT(OUT) :: count
        !DEC$ ATTRIBUTES REFERENCE :: count
        INTEGER(HRESULT) :: GetSegmentCount_intf
        !DEC$ ATTRIBUTES STDCALL :: GetSegmentCount_intf
      END FUNCTION GetSegmentCount_intf
    END INTERFACE
    
    TYPE(ID2D1PathGeometryVTbl), POINTER :: ptbl
    PROCEDURE(GetSegmentCount_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetSegmentCount, proc)
    hr = proc(geometry%object, count)
    
  END SUBROUTINE GetSegmentCount
  
  
  !*****************************************************************************
  
  SUBROUTINE GetFigureCount(geometry, count, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1PathGeometry), INTENT(IN) :: geometry
    INTEGER(UINT32), INTENT(OUT) :: count
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    
    ABSTRACT INTERFACE
      FUNCTION GetFigureCount_intf(this, count)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: UINT32
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(UINT32), INTENT(OUT) :: count
        !DEC$ ATTRIBUTES REFERENCE :: count
        INTEGER(HRESULT) :: GetFigureCount_intf
        !DEC$ ATTRIBUTES STDCALL :: GetFigureCount_intf
      END FUNCTION GetFigureCount_intf
    END INTERFACE
    
    TYPE(ID2D1PathGeometryVTbl), POINTER :: ptbl
    PROCEDURE(GetFigureCount_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(geometry), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetFigureCount, proc)
    hr = proc(geometry%object, count)
    
  END SUBROUTINE GetFigureCount
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1Mesh
  
  
  !*****************************************************************************
  
  FUNCTION d2d1mesh_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1mesh_IID
    
    !***************************************************************************
    
    d2d1mesh_IID = GUID(  &
        INT(Z'2CD906C2', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1mesh_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE Open_mesh(mesh, tessellationSink, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Mesh), INTENT(IN) :: mesh
    TYPE(ID2D1TessellationSink), INTENT(OUT) :: tessellationSink
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    
    ABSTRACT INTERFACE
      FUNCTION Open_intf(this, tessellationSink)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: tessellationSink
        !DEC$ ATTRIBUTES REFERENCE :: tessellationSink
        !DEC$ ATTRIBUTES STDCALL :: Open_intf
        INTEGER(HRESULT) :: Open_intf
      END FUNCTION Open_intf
    END INTERFACE
    
    TYPE(ID2D1MeshVTbl), POINTER :: ptbl
    PROCEDURE(Open_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(mesh), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Open, proc)
    hr = proc(mesh%object, tessellationSink%object)
    
  END SUBROUTINE Open_mesh
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1Layer.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1layer_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1layer_IID
    
    !***************************************************************************
    
    d2d1layer_IID = GUID(  &
        INT(Z'2CD9069B', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1layer_IID
  
  
  !*****************************************************************************
  
  FUNCTION GetSize_layer(layer) RESULT(size)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Layer), INTENT(IN) :: layer
    
    TYPE(D2D1_SIZE_F) :: size
    
    !---------------------------------------------------------------------------
    
    ABSTRACT INTERFACE
      SUBROUTINE GetSize_intf(this, size)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_SIZE_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_SIZE_F), INTENT(OUT) :: size
        !DEC$ ATTRIBUTES REFERENCE :: size
        !DEC$ ATTRIBUTES STDCALL :: GetSize_intf
      END SUBROUTINE GetSize_intf
    END INTERFACE
    
    TYPE(ID2D1LayerVTbl), POINTER :: ptbl
    PROCEDURE(GetSize_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(Layer), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetSize, proc)
    CALL proc(layer%object, size)
    
  END FUNCTION GetSize_layer
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1DrawingStateBlock.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1drawingstateblock_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1drawingstateblock_IID
    
    !***************************************************************************
    
    d2d1drawingstateblock_IID = GUID(  &
        INT(Z'28506E39', C_INT32_T),  &
        INT(Z'EBF6', C_INT16_T),  &
        INT(Z'46A1', C_INT16_T),  &
        [ INT(Z'BB', C_SIGNED_CHAR), INT(Z'47', C_SIGNED_CHAR),  &
          INT(Z'FD', C_SIGNED_CHAR), INT(Z'85', C_SIGNED_CHAR),  &
          INT(Z'56', C_SIGNED_CHAR), INT(Z'5A', C_SIGNED_CHAR),  &
          INT(Z'B9', C_SIGNED_CHAR), INT(Z'57', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1drawingstateblock_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE GetDescription(drawing_state, stateDescription)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1DrawingStateBlock), INTENT(IN) :: drawing_state
    TYPE(D2D1_DRAWING_STATE_DESCRIPTION), INTENT(OUT) :: stateDescription
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetDescription_intf(this, stateDescription)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_DRAWING_STATE_DESCRIPTION
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_DRAWING_STATE_DESCRIPTION), INTENT(OUT) :: stateDescription
        !DEC$ ATTRIBUTES REFERENCE :: stateDescription
        !DEC$ ATTRIBUTES STDCALL :: GetDescription_intf
      END SUBROUTINE GetDescription_intf
    END INTERFACE
    
    TYPE(ID2D1DrawingStateBlockVTbl), POINTER :: ptbl
    PROCEDURE(GetDescription_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(drawing_state), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetDescription, proc)
    CALL proc(drawing_state%object, stateDescription)
    
  END SUBROUTINE GetDescription
  
  
  !*****************************************************************************
  
  SUBROUTINE SetDescription(drawing_state, stateDescription)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1DrawingStateBlock), INTENT(IN) :: drawing_state
    TYPE(D2D1_DRAWING_STATE_DESCRIPTION), INTENT(IN) :: stateDescription
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetDescription_intf(this, stateDescription)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_DRAWING_STATE_DESCRIPTION
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_DRAWING_STATE_DESCRIPTION), INTENT(IN) :: stateDescription
        !DEC$ ATTRIBUTES REFERENCE :: stateDescription
        !DEC$ ATTRIBUTES STDCALL :: SetDescription_intf
      END SUBROUTINE SetDescription_intf
    END INTERFACE
    
    TYPE(ID2D1DrawingStateBlockVTbl), POINTER :: ptbl
    PROCEDURE(SetDescription_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(drawing_state), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetDescription, proc)
    CALL proc(drawing_state%object, stateDescription)
    
  END SUBROUTINE SetDescription
  
  
  !*****************************************************************************
  
  SUBROUTINE SetTextRenderingParams_state(drawing_state, textRenderingParams)
    
    USE DirectWrite, ONLY: IDWriteRenderingParams
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1DrawingStateBlock), INTENT(IN) :: drawing_state
    TYPE(IDWriteRenderingParams), INTENT(IN), OPTIONAL :: textRenderingParams
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetTextRenderingParams_intf(this, textRenderingParams)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_DRAWING_STATE_DESCRIPTION
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: textRenderingParams
        !DEC$ ATTRIBUTES STDCALL :: SetTextRenderingParams_intf
      END SUBROUTINE SetTextRenderingParams_intf
    END INTERFACE
    
    TYPE(ID2D1DrawingStateBlockVTbl), POINTER :: ptbl
    PROCEDURE(SetTextRenderingParams_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(drawing_state), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetTextRenderingParams, proc)
    IF (PRESENT(textRenderingParams)) THEN
      CALL proc(drawing_state%object, textRenderingParams%object)
    ELSE
      CALL proc(drawing_state%object, C_NULL_PTR)
    END IF
    
  END SUBROUTINE SetTextRenderingParams_state
  
  
  !*****************************************************************************
  
  SUBROUTINE GetTextRenderingParams_state(drawing_state, textRenderingParams)
    
    USE DirectWrite, ONLY: IDWriteRenderingParams
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1DrawingStateBlock), INTENT(IN) :: drawing_state
    TYPE(IDWriteRenderingParams), INTENT(OUT) :: textRenderingParams
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetTextRenderingParams_intf(this, textRenderingParams)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: textRenderingParams
        !DEC$ ATTRIBUTES REFERENCE :: textRenderingParams
        !DEC$ ATTRIBUTES STDCALL :: GetTextRenderingParams_intf
      END SUBROUTINE GetTextRenderingParams_intf
    END INTERFACE
    
    TYPE(ID2D1DrawingStateBlockVTbl), POINTER :: ptbl
    PROCEDURE(GetTextRenderingParams_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(drawing_state), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetTextRenderingParams, proc)
    CALL proc(drawing_state%object, textRenderingParams%object)
    
  END SUBROUTINE GetTextRenderingParams_state
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1RenderTarget.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1rendertarget_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1rendertarget_IID
    
    !***************************************************************************
    
    d2d1rendertarget_IID = GUID(  &
        INT(Z'2CD9069B4', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1rendertarget_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateBitmap( render_target, size, bitmapProperties,  &
      bitmap, hr, srcData, pitch )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_SIZE_U), INTENT(IN) :: size
    TYPE(D2D1_BITMAP_PROPERTIES), INTENT(IN) ::bitmapProperties
    TYPE(ID2D1Bitmap), INTENT(OUT) :: bitmap
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(C_PTR), INTENT(IN), OPTIONAL :: srcData
    INTEGER(UINT32), INTENT(IN), OPTIONAL :: pitch
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateBitmap_intf( this, size, srcData, pitch,  &
          bitmapProperties, bitmap )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_SIZE_U
        IMPORT :: UINT32
        IMPORT :: D2D1_BITMAP_PROPERTIES
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_SIZE_U), INTENT(IN) :: size
        ! Optional.
        TYPE(C_PTR), INTENT(IN) :: srcData
        INTEGER(UINT32), INTENT(IN) :: pitch
        TYPE(D2D1_BITMAP_PROPERTIES), INTENT(IN) :: bitmapProperties
        !DEC$ ATTRIBUTES REFERENCE :: bitmapProperties
        TYPE(C_PTR), INTENT(OUT) :: bitmap
        !DEC$ ATTRIBUTES REFERENCE :: bitmap
        INTEGER(HRESULT) :: CreateBitmap_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateBitmap_intf
      END FUNCTION CreateBitmap_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_data
    INTEGER(UINT32) :: local_pitch
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateBitmap_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(srcData)) THEN
      local_data = srcData
    ELSE
      local_data = C_NULL_PTR
    END IF
    
    IF (PRESENT(pitch)) THEN
      local_pitch = pitch
    ELSE
      local_pitch = 0
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateBitmap, proc)
    hr = proc( render_target%object, size, local_data, local_pitch,  &
        bitmapProperties, bitmap%object )
    
  END SUBROUTINE CreateBitmap
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateBitmapFromWicBitmap( render_target, wicBitmapSource,  &
      bitmap, hr, bitmapProperties )
    
    USE WinImagingComponent
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(IWICBitmapSource), INTENT(IN) :: wicBitmapSource
    TYPE(ID2D1Bitmap), INTENT(OUT) :: bitmap
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_BITMAP_PROPERTIES), INTENT(IN), OPTIONAL, TARGET  &
        :: bitmapProperties
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateBitmapFromWicBitmap_intf( this, wicBitmapSource,  &
          bitmapProperties, bitmap )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_BITMAP_PROPERTIES
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: wicBitmapSource
        ! Optional TYPE(D2D1_BITMAP_PROPERTIES)
        TYPE(C_PTR), INTENT(IN) :: bitmapProperties
        TYPE(C_PTR), INTENT(OUT) :: bitmap
        !DEC$ ATTRIBUTES REFERENCE :: bitmap
        INTEGER(HRESULT) :: CreateBitmapFromWicBitmap_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateBitmapFromWicBitmap_intf
      END FUNCTION CreateBitmapFromWicBitmap_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_properties
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateBitmapFromWicBitmap_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(bitmapProperties)) THEN
      local_properties = C_LOC(bitmapProperties)
    ELSE
      local_properties = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateBitmapFromWicBitmap, proc)
    hr = proc( render_target%object, wicBitmapSource%object,  &
        local_properties, bitmap%object )
    
  END SUBROUTINE CreateBitmapFromWicBitmap
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateSharedBitmap( render_target, riid, data,  &
      bitmap, hr, bitmapProperties )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(GUID), INTENT(IN) :: riid
    TYPE(C_PTR), INTENT(IN) :: data
    TYPE(ID2D1Bitmap), INTENT(OUT) :: bitmap
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_BITMAP_PROPERTIES), INTENT(IN), OPTIONAL, TARGET  &
        :: bitmapProperties
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateSharedBitmap_intf( this, riid, data, bitmapProperties,  &
          bitmap )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: GUID
        IMPORT :: D2D1_BITMAP_PROPERTIES
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(GUID), INTENT(IN) :: riid
        !DEC$ ATTRIBUTES REFERENCE :: riid
        TYPE(C_PTR), INTENT(IN) :: data
        ! Optional TYPE(D2D1_BITMAP_PROPERTIES)
        TYPE(C_PTR), INTENT(IN) :: bitmapProperties
        TYPE(C_PTR), INTENT(OUT) :: bitmap
        !DEC$ ATTRIBUTES REFERENCE :: bitmap
        INTEGER(HRESULT) :: CreateSharedBitmap_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateSharedBitmap_intf
      END FUNCTION CreateSharedBitmap_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_properties
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateSharedBitmap_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(bitmapProperties)) THEN
      local_properties = C_LOC(bitmapProperties)
    ELSE
      local_properties = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateSharedBitmap, proc)
    hr = proc(  &
        render_target%object,  &
        riid,  &
        data,  &
        local_properties,  &
        bitmap%object )
    
  END SUBROUTINE CreateSharedBitmap
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateBitmapBrush( render_target, bitmap, bitmapBrush, hr,  &
      bitmapBrushProperties, brushProperties )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    CLASS(ID2D1Bitmap), INTENT(IN) :: bitmap
    TYPE(ID2D1BitmapBrush), INTENT(OUT) :: bitmapBrush
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_BITMAP_BRUSH_PROPERTIES), INTENT(IN), OPTIONAL, TARGET  &
        :: bitmapBrushProperties
    TYPE(D2D1_BRUSH_PROPERTIES), INTENT(IN), OPTIONAL, TARGET  &
        :: brushProperties
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateBitmapBrush_intf( this, bitmap, bitmapBrushProperties,  &
          brushProperties, bitmapBrush )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: bitmap
        TYPE(C_PTR), INTENT(IN) :: bitmapBrushProperties
        TYPE(C_PTR), INTENT(IN) :: brushProperties
        TYPE(C_PTR), INTENT(OUT) :: bitmapBrush
        !DEC$ ATTRIBUTES REFERENCE :: bitmapBrush
        INTEGER(HRESULT) :: CreateBitmapBrush_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateBitmapBrush_intf
      END FUNCTION CreateBitmapBrush_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_brush_properties
    TYPE(C_PTR) :: local_bitmap_brush_properties
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateBitmapBrush_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(brushProperties)) THEN
      local_brush_properties = C_LOC(brushProperties)
    ELSE
      local_brush_properties = C_NULL_PTR
    END IF
    
    IF (PRESENT(bitmapBrushProperties)) THEN
      local_bitmap_brush_properties = C_LOC(bitmapBrushProperties)
    ELSE
      local_bitmap_brush_properties = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateBitmapBrush, proc)
    hr = proc(  &
        render_target%object,  &
        bitmap%object,  &
        local_bitmap_brush_properties,  &
        local_brush_properties,  &
        bitmapBrush%object )
    
  END SUBROUTINE CreateBitmapBrush
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateSolidColorBrush( render_target, color,  &
      solidColorBrush, hr, brushProperties )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_COLOR_F), INTENT(IN) :: color
    TYPE(ID2D1SolidColorBrush), INTENT(OUT), TARGET :: solidColorBrush
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_BRUSH_PROPERTIES), INTENT(IN), OPTIONAL, TARGET :: brushProperties
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateSolidColorBrush_intf( this, color, brushProperties,  &
          solidColorBrush )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_COLOR_F
        IMPORT :: D2D1_BRUSH_PROPERTIES
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_COLOR_F), INTENT(IN) :: color
        !DEC$ ATTRIBUTES REFERENCE :: color
        TYPE(C_PTR), INTENT(IN) :: brushProperties
        TYPE(C_PTR), INTENT(OUT) :: solidColorBrush
        !DEC$ ATTRIBUTES REFERENCE :: solidColorBrush
        INTEGER(HRESULT) :: CreateSolidColorBrush_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateSolidColorBrush_intf
      END FUNCTION CreateSolidColorBrush_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_brushProperties
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateSolidColorBrush_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(brushProperties)) THEN
      local_brushProperties = C_LOC(brushProperties)
    ELSE
      local_brushProperties = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateSolidColorBrush, proc)
    hr = proc(  &
        render_target%object,  &
        color,  &
        local_brushProperties,  &
        solidColorBrush%object )
    
  END SUBROUTINE CreateSolidColorBrush
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateGradientStopCollection( render_target, gradientStops,  &
      gradientStopCollection, hr, colorInterpolationGamma, extendMode )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_INT
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_GRADIENT_STOP), INTENT(IN) :: gradientStops(:)
    TYPE(ID2D1GradientStopCollection), INTENT(OUT) :: gradientStopCollection
    INTEGER(HRESULT), INTENT(OUT) :: hr
    INTEGER(D2D1_GAMMA), INTENT(IN), OPTIONAL :: colorInterpolationGamma
    INTEGER(D2D1_EXTEND_MODE), INTENT(IN), OPTIONAL :: extendMode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateGradientStopCollection_intf( this, gradientStops,  &
          gradientStopsCount, colorInterpolationGamma, extendMode,  &
          gradientStopCollection )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_GRADIENT_STOP
        IMPORT :: D2D1_GAMMA
        IMPORT :: D2D1_EXTEND_MODE
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_GRADIENT_STOP), INTENT(IN) :: gradientStops(*)
        INTEGER(C_INT), INTENT(IN) :: gradientStopsCount
        INTEGER(D2D1_GAMMA), INTENT(IN) :: colorInterpolationGamma
        INTEGER(D2D1_EXTEND_MODE), INTENT(IN) :: extendMode
        TYPE(C_PTR), INTENT(OUT) :: gradientStopCollection
        !DEC$ ATTRIBUTES REFERENCE :: gradientStopCollection
        INTEGER(HRESULT) :: CreateGradientStopCollection_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateGradientStopCollection_intf
      END FUNCTION CreateGradientStopCollection_intf
    END INTERFACE
    
    INTEGER(D2D1_GAMMA) :: local_gamma
    INTEGER(D2D1_EXTEND_MODE) :: local_extend
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateGradientStopCollection_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(colorInterpolationGamma)) THEN
      local_gamma = colorInterpolationGamma
    ELSE
      local_gamma = D2D1_GAMMA_2_2
    END IF
    
    IF (PRESENT(extendMode)) THEN
      local_extend = extendMode
    ELSE
      local_extend = D2D1_EXTEND_MODE_CLAMP
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateGradientStopCollection, proc)
    hr = proc(  &
        render_target%object,  &
        gradientStops,  &
        SIZE(gradientStops, KIND=C_INT),  &
        local_gamma,  &
        local_extend,  &
        gradientStopCollection%object )
    
  END SUBROUTINE CreateGradientStopCollection
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateLinearGradientBrush( render_target,  &
      linearGradientBrushProperties, gradientStopCollection,  &
      linearGradientBrush, hr, brushProperties )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_LINEAR_GRADIENT_BRUSH_PROPERTIES), INTENT(IN)  &
        :: linearGradientBrushProperties
    CLASS(ID2D1GradientStopCollection), INTENT(IN) :: gradientStopCollection
    TYPE(ID2D1LinearGradientBrush), INTENT(OUT) :: linearGradientBrush
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_BRUSH_PROPERTIES), INTENT(IN), OPTIONAL, TARGET  &
        :: brushProperties
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateLinearGradientBrush_intf( this,  &
          linearGradientBrushProperties, brushProperties,  &
          gradientStopCollection, linearGradientBrush )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_LINEAR_GRADIENT_BRUSH_PROPERTIES
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_LINEAR_GRADIENT_BRUSH_PROPERTIES), INTENT(IN)  &
            :: linearGradientBrushProperties
        !DEC$ ATTRIBUTES REFERENCE :: linearGradientBrushProperties
        TYPE(C_PTR), INTENT(IN) :: brushProperties
        TYPE(C_PTR), INTENT(IN) :: gradientStopCollection
        TYPE(C_PTR), INTENT(OUT) :: linearGradientBrush
        !DEC$ ATTRIBUTES REFERENCE :: linearGradientBrush
        INTEGER(HRESULT) :: CreateLinearGradientBrush_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateLinearGradientBrush_intf
      END FUNCTION CreateLinearGradientBrush_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_brush_properties
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateLinearGradientBrush_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(brushProperties)) THEN
      local_brush_properties = C_LOC(brushProperties)
    ELSE
      local_brush_properties = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateLinearGradientBrush, proc)
    hr = proc(  &
        render_target%object,  &
        linearGradientBrushProperties,  &
        local_brush_properties,  &
        gradientStopCollection%object,  &
        linearGradientBrush%object )
    
  END SUBROUTINE CreateLinearGradientBrush
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateRadialGradientBrush( render_target,  &
      radialGradientBrushProperties, gradientStopCollection,  &
      radialGradientBrush, hr, brushProperties )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_RADIAL_GRADIENT_BRUSH_PROPERTIES), INTENT(IN)  &
        :: radialGradientBrushProperties
    CLASS(ID2D1GradientStopCollection), INTENT(IN) :: gradientStopCollection
    TYPE(ID2D1RadialGradientBrush), INTENT(OUT) :: radialGradientBrush
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_BRUSH_PROPERTIES), INTENT(IN), OPTIONAL, TARGET  &
        :: brushProperties
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateRadialGradientBrush_intf( this,  &
          radialGradientBrushProperties, brushProperties,  &
          gradientStopCollection, radialGradientBrush )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RADIAL_GRADIENT_BRUSH_PROPERTIES
        IMPORT :: HRESULT
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_RADIAL_GRADIENT_BRUSH_PROPERTIES), INTENT(IN)  &
            :: radialGradientBrushProperties
        !DEC$ ATTRIBUTES REFERENCE :: radialGradientBrushProperties
        TYPE(C_PTR), INTENT(IN) :: brushProperties
        TYPE(C_PTR), INTENT(IN) :: gradientStopCollection
        TYPE(C_PTR), INTENT(OUT) :: radialGradientBrush
        !DEC$ ATTRIBUTES REFERENCE :: radialGradientBrush
        INTEGER(HRESULT) :: CreateRadialGradientBrush_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateRadialGradientBrush_intf
      END FUNCTION CreateRadialGradientBrush_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_brush_properties
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateRadialGradientBrush_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(brushProperties)) THEN
      local_brush_properties = C_LOC(brushProperties)
    ELSE
      local_brush_properties = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateRadialGradientBrush, proc)
    hr = proc(  &
        render_target%object,  &
        radialGradientBrushProperties,  &
        local_brush_properties,  &
        gradientStopCollection%object,  &
        radialGradientBrush%object )
    
  END SUBROUTINE CreateRadialGradientBrush
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateCompatibleRenderTarget( render_target,  &
      bitmapRenderTarget, hr, desiredSize, desiredPixelSize,  &
      desiredFormat, options )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(ID2D1BitmapRenderTarget), INTENT(OUT) :: bitmapRenderTarget
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_SIZE_F), INTENT(IN), OPTIONAL :: desiredSize
    TYPE(D2D1_SIZE_U), INTENT(IN), OPTIONAL :: desiredPixelSize
    TYPE(D2D1_PIXEL_FORMAT), INTENT(IN), OPTIONAL :: desiredFormat
    INTEGER(D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS), INTENT(IN), OPTIONAL  &
        :: options
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateCompatibleRenderTarget_intf( this, desiredSize,  &
          desiredPixelSize, desiredFormat, options, bitmapRenderTarget )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS
        IMPORT :: HRESULT
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: desiredSize
        TYPE(C_PTR), INTENT(IN) :: desiredPixelSize
        TYPE(C_PTR), INTENT(IN) :: desiredFormat
        INTEGER(D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS), INTENT(IN) :: options
        TYPE(C_PTR), INTENT(OUT) :: bitmapRenderTarget
        !DEC$ ATTRIBUTES REFERENCE :: bitmapRenderTarget
        INTEGER(HRESULT) :: CreateCompatibleRenderTarget_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateCompatibleRenderTarget_intf
      END FUNCTION CreateCompatibleRenderTarget_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_size
    TYPE(C_PTR) :: local_pixel_size
    TYPE(C_PTR) :: local_format
    INTEGER(D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS) :: local_options
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateCompatibleRenderTarget_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(desiredSize)) THEN
      local_size = C_LOC(desiredSize)
    ELSE
      local_size = C_NULL_PTR
    END IF
    
    IF (PRESENT(desiredPixelSize)) THEN
      local_pixel_size = C_LOC(desiredPixelSize)
    ELSE
      local_pixel_size = C_NULL_PTR
    END IF
    
    IF (PRESENT(desiredFormat)) THEN
      local_format = C_LOC(desiredFormat)
    ELSE
      local_format = C_NULL_PTR
    END IF
    
    IF (PRESENT(options)) THEN
      local_options = options
    ELSE
      local_options = D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS_NONE
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateCompatibleRenderTarget, proc)
    hr = proc(  &
        render_target%object,  &
        local_size,  &
        local_pixel_size,  &
        local_format,  &
        local_options,  &
        bitmapRenderTarget%object )
    
  END SUBROUTINE CreateCompatibleRenderTarget
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateLayer(render_target, layer, hr, size)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(ID2D1Layer), INTENT(OUT) :: layer
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_SIZE_F), INTENT(IN), OPTIONAL :: size
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateLayer_intf(this, size, layer)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: size
        TYPE(C_PTR), INTENT(OUT) :: layer
        !DEC$ ATTRIBUTES REFERENCE :: layer
        INTEGER(HRESULT) :: CreateLayer_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateLayer_intf
      END FUNCTION CreateLayer_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_size
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateLayer_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(size)) THEN
      local_size = C_LOC(size)
    ELSE
      local_size = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateLayer, proc)
    hr = proc(render_target%object, local_size, layer%object)
    
  END SUBROUTINE CreateLayer
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateMesh(render_target, mesh, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(ID2D1Mesh), INTENT(OUT) :: mesh
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateMesh_intf(this, mesh)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: mesh
        !DEC$ ATTRIBUTES REFERENCE :: mesh
        INTEGER(HRESULT) :: CreateMesh_intf
        !DEC$ ATTRIBUTES STDCALL :: CreateMesh_intf
      END FUNCTION CreateMesh_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CreateMesh_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateMesh, proc)
    hr = proc(render_target%object, mesh%object)
    
  END SUBROUTINE CreateMesh
  
  
  !*****************************************************************************
  
  SUBROUTINE DrawLine( render_target, point0, point1, brush, strokeWidth,  &
      strokeStyle )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_POINT_2F), INTENT(IN) :: point0
    TYPE(D2D1_POINT_2F), INTENT(IN) :: point1
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    REAL(FLOAT), INTENT(IN), OPTIONAL :: strokeWidth
    CLASS(ID2D1StrokeStyle), INTENT(IN), OPTIONAL :: strokeStyle
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE DrawLine_intf( this, point0, point1, brush, strokeWidth,  &
          strokeStyle )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: point0
        TYPE(D2D1_POINT_2F), INTENT(IN) :: point1
        TYPE(C_PTR), INTENT(IN) :: brush
        REAL(FLOAT), INTENT(IN) :: strokeWidth
        TYPE(C_PTR), INTENT(IN) :: strokeStyle
        !DEC$ ATTRIBUTES STDCALL :: DrawLine_intf
      END SUBROUTINE DrawLine_intf
    END INTERFACE
    
    REAL(FLOAT) :: local_strokeWidth
    TYPE(C_PTR) :: local_strokeStyle
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(DrawLine_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(strokeWidth)) THEN
      local_strokeWidth = strokeWidth
    ELSE
      local_strokeWidth = 1.0_FLOAT
    END IF
    
    IF (PRESENT(strokeStyle)) THEN
      local_strokeStyle = strokeStyle%object
    ELSE
      local_strokeStyle = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%DrawLine, proc)
    CALL proc(  &
        render_target%object,  &
        point0,  &
        point1,  &
        brush%object,  &
        local_strokeWidth,  &
        local_strokeStyle )
    
  END SUBROUTINE DrawLine
  
  
  !*****************************************************************************
  
  SUBROUTINE DrawRectangle( render_target, rect, brush, strokeWidth,  &
      strokeStyle )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_RECT_F), INTENT(IN) :: rect
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    REAL(FLOAT), INTENT(IN), OPTIONAL :: strokeWidth
    CLASS(ID2D1StrokeStyle), INTENT(IN), OPTIONAL :: strokeStyle
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE DrawRectangle_intf(this, rect, brush, strokeWidth, strokeStyle)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RECT_F
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_RECT_F), INTENT(IN) :: rect
        !DEC$ ATTRIBUTES REFERENCE :: rect
        TYPE(C_PTR), INTENT(IN) :: brush
        REAL(FLOAT), INTENT(IN) :: strokeWidth
        TYPE(C_PTR), INTENT(IN) :: strokeStyle
        !DEC$ ATTRIBUTES STDCALL :: DrawRectangle_intf
      END SUBROUTINE DrawRectangle_intf
    END INTERFACE
    
    REAL(FLOAT) :: local_strokeWidth
    TYPE(C_PTR) :: local_strokeStyle
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(DrawRectangle_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(strokeWidth)) THEN
      local_strokeWidth = strokeWidth
    ELSE
      local_strokeWidth = 1.0_FLOAT
    END IF
    
    IF (PRESENT(strokeStyle)) THEN
      local_strokeStyle = strokeStyle%object
    ELSE
      local_strokeStyle = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%DrawRectangle, proc)
    CALL proc(  &
        render_target%object,  &
        rect,  &
        brush%object,  &
        local_strokeWidth,  &
        local_strokeStyle )
    
  END SUBROUTINE DrawRectangle
  
  
  !*****************************************************************************
  
  SUBROUTINE FillRectangle(render_target, rect, brush)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_RECT_F), INTENT(IN) :: rect
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE FillRectangle_intf(this, rect, brush)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RECT_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_RECT_F), INTENT(IN) :: rect
        !DEC$ ATTRIBUTES REFERENCE :: rect
        TYPE(C_PTR), INTENT(IN) :: brush
        !DEC$ ATTRIBUTES STDCALL :: FillRectangle_intf
      END SUBROUTINE FillRectangle_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(FillRectangle_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%FillRectangle, proc)
    CALL proc(render_target%object, rect, brush%object)
    
  END SUBROUTINE FillRectangle
  
  
  !*****************************************************************************
  
  SUBROUTINE DrawRoundedRectangle( render_target, rect, brush, strokeWidth,  &
      strokeStyle )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_ROUNDED_RECT), INTENT(IN) :: rect
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    REAL(FLOAT), INTENT(IN), OPTIONAL :: strokeWidth
    CLASS(ID2D1StrokeStyle), INTENT(IN), OPTIONAL :: strokeStyle
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE DrawRoundedRectangle_intf(this, roundedRect, brush,  &
          strokeWidth, strokeStyle )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ROUNDED_RECT
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_ROUNDED_RECT), INTENT(IN) :: roundedRect
        !DEC$ ATTRIBUTES REFERENCE :: roundedRect
        TYPE(C_PTR), INTENT(IN) :: brush
        REAL(FLOAT), INTENT(IN) :: strokeWidth
        TYPE(C_PTR), INTENT(IN) :: strokeStyle
        !DEC$ ATTRIBUTES STDCALL :: DrawRoundedRectangle_intf
      END SUBROUTINE DrawRoundedRectangle_intf
    END INTERFACE
    
    REAL(FLOAT) :: local_strokeWidth
    TYPE(C_PTR) :: local_strokeStyle
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(DrawRoundedRectangle_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(strokeWidth)) THEN
      local_strokeWidth = strokeWidth
    ELSE
      local_strokeWidth = 1.0_FLOAT
    END IF
    
    IF (PRESENT(strokeStyle)) THEN
      local_strokeStyle = strokeStyle%object
    ELSE
      local_strokeStyle = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%DrawRoundedRectangle, proc)
    CALL proc( render_target%object, rect, brush%object,  &
        local_strokeWidth, local_strokeStyle )
    
  END SUBROUTINE DrawRoundedRectangle
  
  
  !*****************************************************************************
  
  SUBROUTINE FillRoundedRectangle(render_target, roundedRect, brush)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_ROUNDED_RECT), INTENT(IN) :: roundedRect
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE FillRoundedRectangle_intf(this, roundedRect, brush)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ROUNDED_RECT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_ROUNDED_RECT), INTENT(IN) :: roundedRect
        !DEC$ ATTRIBUTES REFERENCE :: roundedRect
        TYPE(C_PTR), INTENT(IN) :: brush
        !DEC$ ATTRIBUTES STDCALL :: FillRoundedRectangle_intf
      END SUBROUTINE FillRoundedRectangle_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(FillRoundedRectangle_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%FillRoundedRectangle, proc)
    CALL proc(render_target%object, roundedRect, brush%object)
    
  END SUBROUTINE FillRoundedRectangle
  
  
  !*****************************************************************************
  
  SUBROUTINE DrawEllipse( render_target, ellipse, brush, strokeWidth,  &
      strokeStyle )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_ELLIPSE), INTENT(IN) :: ellipse
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    REAL(FLOAT), INTENT(IN), OPTIONAL :: strokeWidth
    CLASS(ID2D1StrokeStyle), INTENT(IN), OPTIONAL :: strokeStyle
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE DrawEllipse_intf( this, ellipse, brush, strokeWidth,  &
          strokeStyle )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ELLIPSE
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_ELLIPSE), INTENT(IN) :: ellipse
        !DEC$ ATTRIBUTES REFERENCE :: ellipse
        TYPE(C_PTR), INTENT(IN) :: brush
        REAL(FLOAT), INTENT(IN) :: strokeWidth
        TYPE(C_PTR), INTENT(IN) :: strokeStyle
        !DEC$ ATTRIBUTES STDCALL :: DrawEllipse_intf
      END SUBROUTINE DrawEllipse_intf
    END INTERFACE
    
    REAL(FLOAT) :: local_strokeWidth
    TYPE(C_PTR) :: local_strokeStyle
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(DrawEllipse_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(strokeWidth)) THEN
      local_strokeWidth = strokeWidth
    ELSE
      local_strokeWidth = 1.0_FLOAT
    END IF
    
    IF (PRESENT(strokeStyle)) THEN
      local_strokeStyle = strokeStyle%object
    ELSE
      local_strokeStyle = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%DrawEllipse, proc)
    CALL proc( render_target%object, ellipse, brush%object,  &
        local_strokeWidth, local_strokeStyle )
    
  END SUBROUTINE DrawEllipse
  
  
  !*****************************************************************************
  
  SUBROUTINE FillEllipse(render_target, ellipse, brush)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_ELLIPSE), INTENT(IN) :: ellipse
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE FillEllipse_intf(this, ellipse, brush)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ELLIPSE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_ELLIPSE), INTENT(IN) :: ellipse
        !DEC$ ATTRIBUTES REFERENCE :: ellipse
        TYPE(C_PTR), INTENT(IN) :: brush
        !DEC$ ATTRIBUTES STDCALL :: FillEllipse_intf
      END SUBROUTINE FillEllipse_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(FillEllipse_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%FillEllipse, proc)
    CALL proc(render_target%object, ellipse, brush%object)
    
  END SUBROUTINE FillEllipse
  
  
  !*****************************************************************************
  
  SUBROUTINE DrawGeometry( render_target, geometry, brush, strokeWidth,  &
      strokeStyle )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    REAL(FLOAT), INTENT(IN), OPTIONAL :: strokeWidth
    CLASS(ID2D1StrokeStyle), INTENT(IN), OPTIONAL :: strokeStyle
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE DrawGeometry_intf( this, geometry, brush, strokeWidth,  &
          strokeStyle )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: geometry
        TYPE(C_PTR), INTENT(IN) :: brush
        REAL(FLOAT), INTENT(IN) :: strokeWidth
        TYPE(C_PTR), INTENT(IN) :: strokeStyle
        !DEC$ ATTRIBUTES STDCALL :: DrawGeometry_intf
      END SUBROUTINE DrawGeometry_intf
    END INTERFACE
    
    REAL(FLOAT) :: local_strokeWidth
    TYPE(C_PTR) :: local_strokeStyle
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(DrawGeometry_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(strokeWidth)) THEN
      local_strokeWidth = strokeWidth
    ELSE
      local_strokeWidth = 1.0_FLOAT
    END IF
    
    IF (PRESENT(strokeStyle)) THEN
      local_strokeStyle = strokeStyle%object
    ELSE
      local_strokeStyle = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%DrawGeometry, proc)
    CALL proc( render_target%object, geometry%object, brush%object,  &
        local_strokeWidth, local_strokeStyle )
    
  END SUBROUTINE DrawGeometry
  
  
  !*****************************************************************************
  
  SUBROUTINE FillGeometry(render_target, geometry, brush, opacityBrush)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    CLASS(ID2D1Geometry), INTENT(IN) :: geometry
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    CLASS(ID2D1Brush), INTENT(IN), OPTIONAL :: opacityBrush
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE FillGeometry_intf(this, geometry, brush, opacityBrush)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: geometry
        TYPE(C_PTR), INTENT(IN) :: brush
        TYPE(C_PTR), INTENT(IN) :: opacityBrush
        !DEC$ ATTRIBUTES STDCALL :: FillGeometry_intf
      END SUBROUTINE FillGeometry_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_opacityBrush
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(FillGeometry_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(opacityBrush)) THEN
      local_opacityBrush = opacityBrush%object
    ELSE
      local_opacityBrush = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%FillGeometry, proc)
    CALL proc(  &
        render_target%object,  &
        geometry%object,  &
        brush%object,  &
        local_opacityBrush )
    
  END SUBROUTINE FillGeometry
  
  
  !*****************************************************************************
  
  SUBROUTINE FillMesh(render_target, mesh, brush)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    CLASS(ID2D1Mesh), INTENT(IN) ::mesh
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE FillMesh_intf(this, mesh, brush)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: mesh
        TYPE(C_PTR), INTENT(IN) :: brush
        !DEC$ ATTRIBUTES STDCALL :: FillMesh_intf
      END SUBROUTINE FillMesh_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(FillMesh_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%FillMesh, proc)
    CALL proc(render_target%object, mesh%object, brush%object)
    
  END SUBROUTINE FillMesh
  
  
  !*****************************************************************************
  
  SUBROUTINE FillOpacityMask( render_target, opacityMask, brush, content,  &
      destinationRectangle, sourceRectangle )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    CLASS(ID2D1Bitmap), INTENT(IN) :: opacityMask
    CLASS(ID2D1Brush), INTENT(IN) :: brush
    INTEGER(D2D1_OPACITY_MASK_CONTENT), INTENT(IN) :: content
    TYPE(D2D1_RECT_F), INTENT(IN), OPTIONAL, TARGET :: destinationRectangle
    TYPE(D2D1_RECT_F), INTENT(IN), OPTIONAL, TARGET :: sourceRectangle
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE FillOpacityMask_intf( this, opacityMask, brush, content,  &
          destinationRectangle, sourceRectangle )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_OPACITY_MASK_CONTENT
        IMPORT :: D2D1_RECT_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: opacityMask
        TYPE(C_PTR), INTENT(IN) :: brush
        INTEGER(D2D1_OPACITY_MASK_CONTENT), INTENT(IN) :: content
        TYPE(C_PTR), INTENT(IN) :: destinationRectangle
        TYPE(C_PTR), INTENT(IN) :: sourceRectangle
        !DEC$ ATTRIBUTES STDCALL :: FillOpacityMask_intf
      END SUBROUTINE FillOpacityMask_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_destination
    TYPE(C_PTR) :: local_source
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(FillOpacityMask_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(destinationRectangle)) THEN
      local_destination = C_LOC(destinationRectangle)
    ELSE
      local_destination = C_NULL_PTR
    END IF
    
    IF (PRESENT(sourceRectangle)) THEN
      local_source = C_LOC(sourceRectangle)
    ELSE
      local_source = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%FillOpacityMask, proc)
    CALL proc(  &
        render_target%object,  &
        opacityMask%object,  &
        brush%object,  &
        content,  &
        local_destination,  &
        local_source )
    
  END SUBROUTINE FillOpacityMask
  
  
  !*****************************************************************************
  
  SUBROUTINE DrawBitmap( render_target, bitmap, destinationRectangle,  &
      opacity, interpolationMode, sourceRectangle )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    CLASS(ID2D1Bitmap), INTENT(IN) :: bitmap
    TYPE(D2D1_RECT_F), INTENT(IN), OPTIONAL, TARGET :: destinationRectangle
    REAL(FLOAT), INTENT(IN), OPTIONAL :: opacity
    INTEGER(D2D1_BITMAP_INTERPOLATION_MODE), INTENT(IN), OPTIONAL  &
        :: interpolationMode
    TYPE(D2D1_RECT_F), INTENT(IN), OPTIONAL, TARGET :: sourceRectangle
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE DrawBitmap_intf( this, bitmap, destinationRectangle,  &
          opacity, interpolationMode, sourceRectangle )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPORT :: D2D1_BITMAP_INTERPOLATION_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: bitmap
        TYPE(C_PTR), INTENT(IN) :: destinationRectangle
        REAL(FLOAT), INTENT(IN) :: opacity
        INTEGER(D2D1_BITMAP_INTERPOLATION_MODE), INTENT(IN)  &
            :: interpolationMode
        TYPE(C_PTR), INTENT(IN) :: sourceRectangle
        !DEC$ ATTRIBUTES STDCALL :: DrawBitmap_intf
      END SUBROUTINE DrawBitmap_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_destination
    TYPE(C_PTR) :: local_source
    REAL(FLOAT) :: local_opacity
    INTEGER(D2D1_BITMAP_INTERPOLATION_MODE) :: local_mode
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(DrawBitmap_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(destinationRectangle)) THEN
      local_destination = C_LOC(destinationRectangle)
    ELSE
      local_destination = C_NULL_PTR
    END IF
    
    IF (PRESENT(opacity)) THEN
      local_opacity = opacity
    ELSE
      local_opacity = 1.0_FLOAT
    END IF
    
    IF (PRESENT(interpolationMode)) THEN
      local_mode = interpolationMode
    ELSE
      local_mode = D2D1_BITMAP_INTERPOLATION_MODE_LINEAR
    END IF
    
    IF (PRESENT(sourceRectangle)) THEN
      local_source = C_LOC(sourceRectangle)
    ELSE
      local_source = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%DrawBitmap, proc)
    CALL proc(  &
        render_target%object,  &
        bitmap%object,  &
        local_destination,  &
        local_opacity,  &
        local_mode,  &
        local_source )
    
  END SUBROUTINE DrawBitmap
  
  
  !*****************************************************************************
  
  SUBROUTINE DrawText_1( render_target, string, textFormat, layoutRect,  &
      defaultForegroundBrush, options, measuringMode )
    
    USE DirectWrite, ONLY: IDWriteTextFormat, DWRITE_MEASURING_MODE
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    CHARACTER(*), INTENT(IN), TARGET :: string
    CLASS(IDWriteTextFormat), INTENT(IN) :: textFormat
    TYPE(D2D1_RECT_F), INTENT(IN) :: layoutRect
    CLASS(ID2D1Brush), INTENT(IN) :: defaultForegroundBrush
    INTEGER(D2D1_DRAW_TEXT_OPTIONS), INTENT(IN), OPTIONAL :: options
    INTEGER(DWRITE_MEASURING_MODE), INTENT(IN), OPTIONAL :: measuringMode
    
    !***************************************************************************
    
    CALL DrawText(  &
        render_target,  &
        UTF8ToUTF16(string),  &
        textFormat,  &
        layoutRect,  &
        defaultForegroundBrush,  &
        options,  &
        measuringMode )
    
  END SUBROUTINE DrawText_1
  
  
  !*****************************************************************************
  
  SUBROUTINE DrawText_2( render_target, string, textFormat, layoutRect,  &
      defaultForegroundBrush, options, measuringMode )
    
    USE DirectWrite, ONLY:  &
        IDWriteTextFormat,  &
        DWRITE_MEASURING_MODE,  &
        DWRITE_MEASURING_MODE_NATURAL
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    INTEGER(WCHAR), INTENT(IN) :: string(:)
    CLASS(IDWriteTextFormat), INTENT(IN) :: textFormat
    TYPE(D2D1_RECT_F), INTENT(IN) :: layoutRect
    CLASS(ID2D1Brush), INTENT(IN) :: defaultForegroundBrush
    INTEGER(D2D1_DRAW_TEXT_OPTIONS), INTENT(IN), OPTIONAL :: options
    INTEGER(DWRITE_MEASURING_MODE), INTENT(IN), OPTIONAL :: measuringMode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE DrawText_intf(this, string, stringLength, textFormat,  &
          layoutRect, defaultForegroundBrush, options, measuringMode )
        USE DirectWrite, ONLY: DWRITE_MEASURING_MODE
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: WCHAR
        IMPORT :: D2D1_RECT_F
        IMPORT :: D2D1_DRAW_TEXT_OPTIONS
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(WCHAR), INTENT(IN) :: string(*)
        INTEGER(C_INT), INTENT(IN) :: stringLength
        TYPE(C_PTR), INTENT(IN) :: textFormat
        TYPE(D2D1_RECT_F), INTENT(IN) :: layoutRect
        !DEC$ ATTRIBUTES REFERENCE :: layoutRect
        TYPE(C_PTR), INTENT(IN) :: defaultForegroundBrush
        INTEGER(D2D1_DRAW_TEXT_OPTIONS), INTENT(IN) :: options
        INTEGER(DWRITE_MEASURING_MODE), INTENT(IN) :: measuringMode
        !DEC$ ATTRIBUTES STDCALL :: DrawText_intf
      END SUBROUTINE DrawText_intf
    END INTERFACE
    
    INTEGER(D2D1_DRAW_TEXT_OPTIONS) :: local_options
    INTEGER(DWRITE_MEASURING_MODE) :: local_measuringMode
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(DrawText_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(options)) THEN
      local_options = options
    ELSE
      local_options = D2D1_DRAW_TEXT_OPTIONS_NONE
    END IF
    
    IF (PRESENT(measuringMode)) THEN
      local_measuringMode = measuringMode
    ELSE
      local_measuringMode = DWRITE_MEASURING_MODE_NATURAL
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%DrawText, proc)
    CALL proc(  &
        render_target%object,  &
        string,  &
        SIZE(string),  &
        textFormat%object,  &
        layoutRect,  &
        defaultForegroundBrush%object,  &
        local_options,  &
        local_measuringMode )
    
  END SUBROUTINE DrawText_2
  
  
  !*****************************************************************************
  
  SUBROUTINE DrawTextLayout( render_target, origin, textLayout,  &
      defaultForegroundBrush, options )
    
    USE DirectWrite, ONLY: IDWriteTextLayout
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_POINT_2F), INTENT(IN) :: origin
    TYPE(IDWriteTextLayout), INTENT(IN) :: textLayout
    CLASS(ID2D1Brush), INTENT(IN) :: defaultForegroundBrush
    INTEGER(D2D1_DRAW_TEXT_OPTIONS), INTENT(IN), OPTIONAL :: options
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE DrawTextLayout_intf( this, origin, textLayout,  &
          defaultForegroundBrush, options )
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPORT :: D2D1_DRAW_TEXT_OPTIONS
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: origin
        TYPE(C_PTR), INTENT(IN) :: textLayout
        TYPE(C_PTR), INTENT(IN) :: defaultForegroundBrush
        INTEGER(D2D1_DRAW_TEXT_OPTIONS), INTENT(IN) :: options
        !DEC$ ATTRIBUTES STDCALL :: DrawTextLayout_intf
      END SUBROUTINE DrawTextLayout_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(DrawTextLayout_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%DrawTextLayout, proc)
    IF (PRESENT(options)) THEN
      CALL proc(  &
          render_target%object,  &
          origin,  &
          textLayout%object,  &
          defaultForegroundBrush%object,  &
          options )
    ELSE
      CALL proc(  &
          render_target%object,  &
          origin,  &
          textLayout%object,  &
          defaultForegroundBrush%object,  &
          D2D1_DRAW_TEXT_OPTIONS_NONE )
    END IF
  END SUBROUTINE DrawTextLayout
  
  
  !*****************************************************************************
  
  SUBROUTINE DrawGlyphRun( render_target, baselineOrigin, glyphRun,  &
      foregroundBrush, measuringMode )
    
    USE DirectWrite, ONLY: DWRITE_MEASURING_MODE, DWRITE_GLYPH_RUN,  &
        DWRITE_MEASURING_MODE_NATURAL
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_POINT_2F), INTENT(IN) :: baselineOrigin
    TYPE(DWRITE_GLYPH_RUN), INTENT(IN) :: glyphRun
    CLASS(ID2D1Brush), INTENT(IN) :: foregroundBrush
    INTEGER(DWRITE_MEASURING_MODE), INTENT(IN), OPTIONAL :: measuringMode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE DrawGlyphRun_intf( this, baselineOrigin, glyphRun,  &
          foregroundBrush, measuringMode )
        USE DirectWrite, ONLY: DWRITE_MEASURING_MODE, DWRITE_GLYPH_RUN
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_POINT_2F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_POINT_2F), INTENT(IN) :: baselineOrigin
        TYPE(DWRITE_GLYPH_RUN), INTENT(IN) :: glyphRun
        !DEC$ ATTRIBUTES REFERENCE :: glyphRun
        TYPE(C_PTR), INTENT(IN) :: foregroundBrush
        INTEGER(DWRITE_MEASURING_MODE) :: measuringMode
        !DEC$ ATTRIBUTES STDCALL :: DrawGlyphRun_intf
      END SUBROUTINE DrawGlyphRun_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(DrawGlyphRun_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%DrawGlyphRun, proc)
    IF (PRESENT(measuringMode)) THEN
      CALL proc( render_target%object, baselineOrigin, glyphRun,  &
          foregroundBrush%object, measuringMode )
    ELSE
      CALL proc( render_target%object, baselineOrigin, glyphRun,  &
          foregroundBrush%object, DWRITE_MEASURING_MODE_NATURAL )
    END IF
    
  END SUBROUTINE DrawGlyphRun
  
  
  !*****************************************************************************
  
  SUBROUTINE SetTransform_render(render_target, transform)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_MATRIX_3x2_F), INTENT(IN) :: transform
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetTransform_intf(this, transform)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_MATRIX_3x2_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_MATRIX_3x2_F), INTENT(IN) :: transform
        !DEC$ ATTRIBUTES REFERENCE :: transform
        !DEC$ ATTRIBUTES STDCALL :: SetTransform_intf
      END SUBROUTINE SetTransform_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(SetTransform_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetTransform, proc)
    CALL proc(render_target%object, transform)
    
  END SUBROUTINE SetTransform_render
  
  
  !*****************************************************************************
  
  FUNCTION GetTransform_render(render_target) RESULT(transform)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    
    TYPE(D2D1_MATRIX_3X2_F) :: transform
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetTransform_intf(this, transform)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_MATRIX_3x2_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_MATRIX_3x2_F), INTENT(OUT) :: transform
        !DEC$ ATTRIBUTES REFERENCE :: transform
        !DEC$ ATTRIBUTES STDCALL :: GetTransform_intf
      END SUBROUTINE GetTransform_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetTransform_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetTransform, proc)
    CALL proc(render_target%object, transform)
    
  END FUNCTION GetTransform_render
  
  
  !*****************************************************************************
  
  SUBROUTINE SetAntialiasMode(render_target, mode)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    INTEGER(D2D1_ANTIALIAS_MODE), INTENT(IN) :: mode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetAntialiasMode_intf(this, mode)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ANTIALIAS_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_ANTIALIAS_MODE), INTENT(IN) :: mode
        !DEC$ ATTRIBUTES STDCALL :: SetAntialiasMode_intf
      END SUBROUTINE SetAntialiasMode_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(SetAntialiasMode_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetAntialiasMode, proc)
    CALL proc(render_target%object, mode)
    
  END SUBROUTINE SetAntialiasMode
  
  
  !*****************************************************************************
  
  FUNCTION GetAntialiasMode(render_target) RESULT(mode)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    
    INTEGER(D2D1_ANTIALIAS_MODE) :: mode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetAntialiasMode_intf(this) RESULT(mode)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ANTIALIAS_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_ANTIALIAS_MODE) :: mode
        !DEC$ ATTRIBUTES STDCALL :: GetAntialiasMode_intf
      END FUNCTION GetAntialiasMode_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetAntialiasMode_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetAntialiasMode, proc)
    mode = proc(render_target%object)
    
  END FUNCTION GetAntialiasMode
  
  
  !*****************************************************************************
  
  SUBROUTINE SetTextAntialiasMode(render_target, mode)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    INTEGER(D2D1_TEXT_ANTIALIAS_MODE), INTENT(IN) :: mode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetTextAntialiasMode_intf(this, mode)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_TEXT_ANTIALIAS_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_TEXT_ANTIALIAS_MODE), INTENT(IN) :: mode
        !DEC$ ATTRIBUTES STDCALL :: SetTextAntialiasMode_intf
      END SUBROUTINE SetTextAntialiasMode_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(SetTextAntialiasMode_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetTextAntialiasMode, proc)
    CALL proc(render_target%object, mode)
    
  END SUBROUTINE SetTextAntialiasMode
  
  
  !*****************************************************************************
  
  FUNCTION GetTextAntialiasMode(render_target) RESULT(mode)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    
    INTEGER(D2D1_TEXT_ANTIALIAS_MODE) :: mode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetTextAntialiasMode_intf(this) RESULT(mode)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_TEXT_ANTIALIAS_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_TEXT_ANTIALIAS_MODE) :: mode
        !DEC$ ATTRIBUTES STDCALL :: GetTextAntialiasMode_intf
      END FUNCTION GetTextAntialiasMode_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetTextAntialiasMode_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetTextAntialiasMode, proc)
    mode = proc(render_target%object)
    
  END FUNCTION GetTextAntialiasMode
  
  
  !*****************************************************************************
  
  SUBROUTINE SetTextRenderingParams_render(render_target, textRenderingParams)
    
    USE DirectWrite, ONLY: IDWriteRenderingParams
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(IDWriteRenderingParams), INTENT(IN), OPTIONAL :: textRenderingParams
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetTextRenderingParams_intf(this, textRenderingParams)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: textRenderingParams
        !DEC$ ATTRIBUTES STDCALL :: SetTextRenderingParams_intf
      END SUBROUTINE SetTextRenderingParams_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(SetTextRenderingParams_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetTextRenderingParams, proc)
    IF (PRESENT(textRenderingParams)) THEN
      CALL proc(render_target%object, textRenderingParams%object)
    ELSE
      CALL proc(render_target%object, C_NULL_PTR)
    END IF
    
  END SUBROUTINE SetTextRenderingParams_render
  
  
  !*****************************************************************************
  
  SUBROUTINE GetTextRenderingParams_render(render_target, textRenderingParams)
    
    USE DirectWrite, ONLY: IDWriteRenderingParams
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(IDWriteRenderingParams), INTENT(OUT) :: textRenderingParams
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetTextRenderingParams_intf(this, textRenderingParams)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: textRenderingParams
        !DEC$ ATTRIBUTES REFERENCE :: textRenderingParams
        !DEC$ ATTRIBUTES STDCALL :: GetTextRenderingParams_intf
      END SUBROUTINE GetTextRenderingParams_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetTextRenderingParams_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetTextRenderingParams, proc)
    CALL proc(render_target%object, textRenderingParams%object)
    
  END SUBROUTINE GetTextRenderingParams_render
  
  
  !*****************************************************************************
  
  SUBROUTINE SetTags(render_target, tag1, tag2)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    INTEGER(D2D1_TAG), INTENT(IN) :: tag1
    INTEGER(D2D1_TAG), INTENT(IN) :: tag2
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetTags_intf(this, tag1, tag2)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_TAG
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_TAG), INTENT(IN) :: tag1
        INTEGER(D2D1_TAG), INTENT(IN) :: tag2
        !DEC$ ATTRIBUTES STDCALL :: SetTags_intf
      END SUBROUTINE SetTags_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(SetTags_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetTags, proc)
    CALL proc(render_target%object, tag1, tag2)
    
  END SUBROUTINE SetTags
  
  
  !*****************************************************************************
  
  SUBROUTINE GetTags(render_target, tag1, tag2)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    INTEGER(D2D1_TAG), INTENT(OUT), OPTIONAL, TARGET :: tag1
    INTEGER(D2D1_TAG), INTENT(OUT), OPTIONAL, TARGET :: tag2
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetTags_intf(this, tag1, tag2)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: tag1
        TYPE(C_PTR), INTENT(IN) :: tag2
        !DEC$ ATTRIBUTES STDCALL :: GetTags_intf
      END SUBROUTINE GetTags_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_tag1
    TYPE(C_PTR) :: local_tag2
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetTags_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(tag1)) THEN
      local_tag1 = C_LOC(tag1)
    ELSE
      local_tag1 = C_NULL_PTR
    END IF
    
    IF (PRESENT(tag2)) THEN
      local_tag2 = C_LOC(tag2)
    ELSE
      local_tag2 = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetTags, proc)
    CALL proc(render_target%object, local_tag1, local_tag2)
    
  END SUBROUTINE GetTags
  
  
  !*****************************************************************************
  
  SUBROUTINE PushLayer(render_target, layerParameters, layer)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_LAYER_PARAMETERS), INTENT(IN) :: layerParameters
    CLASS(ID2D1Layer), INTENT(IN) :: layer
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE PushLayer_intf(this, layerParameters, layer)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_LAYER_PARAMETERS
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_LAYER_PARAMETERS), INTENT(IN) :: layerParameters
        !DEC$ ATTRIBUTES REFERENCE :: layerParameters
        TYPE(C_PTR), INTENT(IN) :: layer
        !DEC$ ATTRIBUTES STDCALL :: PushLayer_intf
      END SUBROUTINE PushLayer_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(PushLayer_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%PushLayer, proc)
    CALL proc(render_target%object, layerParameters, layer%object)
    
  END SUBROUTINE PushLayer
  
  
  !*****************************************************************************
  
  SUBROUTINE PopLayer(render_target)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE PopLayer_intf(this)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        !DEC$ ATTRIBUTES STDCALL :: PopLayer_intf
      END SUBROUTINE PopLayer_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(PopLayer_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%PopLayer, proc)
    CALL proc(render_target%object)
    
  END SUBROUTINE PopLayer
  
  
  !*****************************************************************************
  
  SUBROUTINE Flush(render_target, hr, tag1, tag2)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    INTEGER(HRESULT), INTENT(OUT) :: hr
    INTEGER(D2D1_TAG), INTENT(OUT), OPTIONAL, TARGET :: tag1
    INTEGER(D2D1_TAG), INTENT(OUT), OPTIONAL, TARGET :: tag2
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION Flush_intf(this, tag1, tag2) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: tag1
        TYPE(C_PTR), INTENT(IN) :: tag2
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: Flush_intf
      END FUNCTION Flush_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_tag1
    TYPE(C_PTR) :: local_tag2
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(Flush_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(tag1)) THEN
      local_tag1 = C_LOC(tag1)
    ELSE
      local_tag1 = C_NULL_PTR
    END IF
    
    IF (PRESENT(tag2)) THEN
      local_tag2 = C_LOC(tag2)
    ELSE
      local_tag2 = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Flush, proc)
    hr = proc(render_target%object, local_tag1, local_tag2)
    
  END SUBROUTINE Flush
  
  
  !*****************************************************************************
  
  SUBROUTINE SaveDrawingState(render_target, drawingStateBlock)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    CLASS(ID2D1DrawingStateBlock), INTENT(INOUT) :: drawingStateBlock
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SaveDrawingState_intf(this, drawingStateBlock)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RECT_F
        IMPORT :: D2D1_ANTIALIAS_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: drawingStateBlock
        !DEC$ ATTRIBUTES STDCALL :: SaveDrawingState_intf
      END SUBROUTINE SaveDrawingState_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(SaveDrawingState_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SaveDrawingState, proc)
    CALL proc(render_target%object, drawingStateBlock%object)
    
  END SUBROUTINE SaveDrawingState
  
  
  !*****************************************************************************
  
  SUBROUTINE RestoreDrawingState(render_target, drawingStateBlock)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    CLASS(ID2D1DrawingStateBlock), INTENT(IN) :: drawingStateBlock
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE RestoreDrawingState_intf(this, drawingStateBlock)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RECT_F
        IMPORT :: D2D1_ANTIALIAS_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: drawingStateBlock
        !DEC$ ATTRIBUTES STDCALL :: RestoreDrawingState_intf
      END SUBROUTINE RestoreDrawingState_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(RestoreDrawingState_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%RestoreDrawingState, proc)
    CALL proc(render_target%object, drawingStateBlock%object)
    
  END SUBROUTINE RestoreDrawingState
  
  
  !*****************************************************************************
  
  SUBROUTINE PushAxisAlignedClip(render_target, clipRect, antialiasMode)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_RECT_F), INTENT(IN) :: clipRect
    INTEGER(D2D1_ANTIALIAS_MODE), INTENT(IN) :: antialiasMode
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE PushAxisAlignedClip_intf(this, clipRect, antialiasMode)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RECT_F
        IMPORT :: D2D1_ANTIALIAS_MODE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_RECT_F), INTENT(IN) :: clipRect
        !DEC$ ATTRIBUTES REFERENCE :: clipRect
        INTEGER(D2D1_ANTIALIAS_MODE), INTENT(IN) :: antialiasMode
        !DEC$ ATTRIBUTES STDCALL :: PushAxisAlignedClip_intf
      END SUBROUTINE PushAxisAlignedClip_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(PushAxisAlignedClip_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%PushAxisAlignedClip, proc)
    CALL proc(render_target%object, clipRect, antialiasMode)
    
  END SUBROUTINE PushAxisAlignedClip
  
  
  !*****************************************************************************
  
  SUBROUTINE PopAxisAlignedClip(render_target)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE PopAxisAlignedClip_intf(this)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_COLOR_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        !DEC$ ATTRIBUTES STDCALL :: PopAxisAlignedClip_intf
      END SUBROUTINE PopAxisAlignedClip_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(PopAxisAlignedClip_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%PopAxisAlignedClip, proc)
    CALL proc(render_target%object)
    
  END SUBROUTINE PopAxisAlignedClip
  
  
  !*****************************************************************************
  
  SUBROUTINE Clear(render_target, color)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_COLOR_F), INTENT(IN) :: color
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE Clear_intf(this, clearColor)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_COLOR_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_COLOR_F), INTENT(IN) :: clearColor
        !DEC$ ATTRIBUTES REFERENCE :: clearColor
        !DEC$ ATTRIBUTES STDCALL :: Clear_intf
      END SUBROUTINE Clear_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(Clear_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Clear, proc)
    CALL proc(render_target%object, color)
    
  END SUBROUTINE Clear
  
  
  !*****************************************************************************
  
  SUBROUTINE BeginDraw(render_target)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE BeginDraw_intf(this)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        !DEC$ ATTRIBUTES STDCALL :: BeginDraw_intf
      END SUBROUTINE BeginDraw_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(BeginDraw_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%BeginDraw, proc)
    CALL proc(render_target%object)
    
  END SUBROUTINE BeginDraw
  
  
  !*****************************************************************************
  
  SUBROUTINE EndDraw(render_target, hr, tag1, tag2)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER,  &
        C_INT64_T
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    INTEGER(HRESULT), INTENT(OUT) :: hr
    INTEGER(C_INT64_T), INTENT(OUT), OPTIONAL :: tag1
    INTEGER(C_INT64_T), INTENT(OUT), OPTIONAL :: tag2
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION EndDraw_intf(this, tag1, tag2) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT64_T
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(C_INT64_T), INTENT(OUT) :: tag1
        !DEC$ ATTRIBUTES REFERENCE :: tag1
        INTEGER(C_INT64_T), INTENT(OUT) :: tag2
        !DEC$ ATTRIBUTES REFERENCE :: tag2
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: EndDraw_intf
      END FUNCTION EndDraw_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(EndDraw_intf), POINTER :: proc
    
    INTEGER(KIND(tag1)) :: local_tag1
    INTEGER(KIND(tag2)) :: local_tag2
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%EndDraw, proc)
    hr = proc(render_target%object, local_tag1, local_tag2)
    
    IF (PRESENT(tag1)) tag1 = local_tag1
    IF (PRESENT(tag2)) tag2 = local_tag2
    
  END SUBROUTINE EndDraw
  
  
  !*****************************************************************************
  
  FUNCTION GetPixelFormat_render(render_target) RESULT(pf)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    
    ! Function result.
    TYPE(D2D1_PIXEL_FORMAT) :: pf
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetPixelFormat_intf(this, pf)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_PIXEL_FORMAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_PIXEL_FORMAT), INTENT(OUT) :: pf
        !DEC$ ATTRIBUTES REFERENCE :: pf
        !DEC$ ATTRIBUTES STDCALL :: GetPixelFormat_intf
      END SUBROUTINE GetPixelFormat_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetPixelFormat_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetPixelFormat, proc)
    CALL proc(render_target%object, pf)
    
  END FUNCTION GetPixelFormat_render
  
  
  !*****************************************************************************
  
  SUBROUTINE SetDpi_render(render_target, dpiX, dpiY)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    REAL(FLOAT), INTENT(IN) :: dpiX
    REAL(FLOAT), INTENT(IN) :: dpiY
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE SetDpi_intf(this, dpiX, dpiY)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(IN) :: dpiX
        REAL(FLOAT), INTENT(IN) :: dpiY
        !DEC$ ATTRIBUTES STDCALL :: SetDpi_intf
      END SUBROUTINE SetDpi_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(SetDpi_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%SetDpi, proc)
    CALL proc(render_target%object, dpiX, dpiY)
    
  END SUBROUTINE SetDpi_render
  
  
  !*****************************************************************************
  
  SUBROUTINE GetDpi_render(render_target, dpiX, dpiY)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    REAL(FLOAT), INTENT(OUT) :: dpiX
    REAL(FLOAT), INTENT(OUT) :: dpiY
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetDpi_intf(this, dpiX, dpiY)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(OUT) :: dpiX
        !DEC$ ATTRIBUTES REFERENCE :: dpiX
        REAL(FLOAT), INTENT(OUT) :: dpiY
        !DEC$ ATTRIBUTES REFERENCE :: dpiY
        !DEC$ ATTRIBUTES STDCALL :: GetDpi_intf
      END SUBROUTINE GetDpi_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetDpi_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetDpi, proc)
    CALL proc(render_target%object, dpiX, dpiY)
    
  END SUBROUTINE GetDpi_render
  
  
  !*****************************************************************************
  
  FUNCTION GetSize_render(render_target) RESULT(size)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    
    ! Function result.
    TYPE(D2D1_SIZE_F) :: size
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetSize_intf(this, sz)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_SIZE_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_SIZE_F), INTENT(OUT) :: sz
        !DEC$ ATTRIBUTES REFERENCE :: sz
        !DEC$ ATTRIBUTES STDCALL :: GetSize_intf
      END SUBROUTINE GetSize_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetSize_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetSize, proc)
    CALL proc(render_target%object, size)
    
  END FUNCTION GetSize_render
  
  
  !*****************************************************************************
  
  FUNCTION GetPixelSize_render(render_target) RESULT(size)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    
    TYPE(D2D1_SIZE_F) :: size
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetPixelSize_intf(this, size)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_SIZE_F
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_SIZE_F), INTENT(OUT) :: size
        !DEC$ ATTRIBUTES REFERENCE :: size
        !DEC$ ATTRIBUTES STDCALL :: GetPixelSize_intf
      END SUBROUTINE GetPixelSize_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetPixelSize_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetPixelSize, proc)
    CALL proc(render_target%object, size)
    
  END FUNCTION GetPixelSize_render
  
  
  !*****************************************************************************
  
  FUNCTION GetMaximumBitmapSize(render_target) RESULT(size)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    
    INTEGER(UINT32) :: size
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetMaximumBitmapSize_intf(this) RESULT(size)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: UINT32
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(UINT32) :: size
        !DEC$ ATTRIBUTES STDCALL :: GetMaximumBitmapSize_intf
      END FUNCTION GetMaximumBitmapSize_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetMaximumBitmapSize_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetMaximumBitmapSize, proc)
    size = proc(render_target%object)
    
  END FUNCTION GetMaximumBitmapSize
  
  
  !*****************************************************************************
  
  FUNCTION IsSupported(render_target, renderTargetProperties)  &
      RESULT(is_supported)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1RenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_RENDER_TARGET_PROPERTIES), INTENT(IN)  &
        :: renderTargetProperties
    
    LOGICAL :: is_supported
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION IsSupported_intf(this, renderTargetProperties)  &
          RESULT(is_supported)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_RENDER_TARGET_PROPERTIES
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_RENDER_TARGET_PROPERTIES), INTENT(IN)  &
            :: renderTargetProperties
        !DEC$ ATTRIBUTES REFERENCE :: renderTargetProperties
        INTEGER(C_INT) :: is_supported
        !DEC$ ATTRIBUTES STDCALL :: IsSupported_intf
      END FUNCTION IsSupported_intf
    END INTERFACE
    
    TYPE(ID2D1RenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(IsSupported_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%IsSupported, proc)
    is_supported = proc(render_target%object, renderTargetProperties)
    
  END FUNCTION IsSupported
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1BitmapRenderTarget.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1bitmaprendertarget_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1bitmaprendertarget_IID
    
    !***************************************************************************
    
    d2d1bitmaprendertarget_IID = GUID(  &
        INT(Z'2CD90695', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1bitmaprendertarget_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE GetBitmap_render(render_target, bitmap, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1BitmapRenderTarget), INTENT(IN) :: render_target
    CLASS(ID2D1Bitmap), INTENT(OUT) :: bitmap
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetBitmap_intf(this, bitmap) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: bitmap
        !DEC$ ATTRIBUTES REFERENCE :: bitmap
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: GetBitmap_intf
      END FUNCTION GetBitmap_intf
    END INTERFACE
    
    TYPE(ID2D1BitmapRenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetBitmap_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetBitmap, proc)
    hr = proc(render_target%object, bitmap%object)
    
  END SUBROUTINE GetBitmap_render
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1HwndRenderTarget.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1hwndrendertarget_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1hwndrendertarget_IID
    
    !***************************************************************************
    
    d2d1hwndrendertarget_IID = GUID(  &
        INT(Z'2CD90698', C_INT32_T),  &
        INT(Z'12E2', C_INT16_T),  &
        INT(Z'11DC', C_INT16_T),  &
        [ INT(Z'9F', C_SIGNED_CHAR), INT(Z'ED', C_SIGNED_CHAR),  &
          INT(Z'00', C_SIGNED_CHAR), INT(Z'11', C_SIGNED_CHAR),  &
          INT(Z'43', C_SIGNED_CHAR), INT(Z'A0', C_SIGNED_CHAR),  &
          INT(Z'55', C_SIGNED_CHAR), INT(Z'F9', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1hwndrendertarget_IID
  
  
  !*****************************************************************************
  
  FUNCTION CheckWindowState(render_target) RESULT(window_state)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1HwndRenderTarget), INTENT(IN) :: render_target
    
    INTEGER(D2D1_WINDOW_STATE) :: window_state
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CheckWindowState_intf(this) RESULT(window_state)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_WINDOW_STATE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_WINDOW_STATE) :: window_state
        !DEC$ ATTRIBUTES STDCALL :: CheckWindowState_intf
      END FUNCTION CheckWindowState_intf
    END INTERFACE
    
    TYPE(ID2D1HwndRenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(CheckWindowState_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CheckWindowState, proc)
    window_state = proc(render_target%object)
    
  END FUNCTION CheckWindowState
  
  
  !*****************************************************************************
  
  SUBROUTINE Resize(render_target, pixelSize, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1HwndRenderTarget), INTENT(IN) :: render_target
    TYPE(D2D1_SIZE_U), INTENT(IN) :: pixelSize
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION Resize_intf(this, pixelSize) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_SIZE_U
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_SIZE_U), INTENT(IN) :: pixelSize
        !DEC$ ATTRIBUTES REFERENCE :: pixelSize
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: Resize_intf
      END FUNCTION Resize_intf
    END INTERFACE
    
    TYPE(ID2D1HwndRenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(Resize_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%Resize, proc)
    hr = proc(render_target%object, pixelSize)
    
  END SUBROUTINE Resize
  
  
  !*****************************************************************************
  
  FUNCTION GetHwnd(render_target) RESULT(hwnd)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1HwndRenderTarget), INTENT(IN) :: render_target
    
    INTEGER(HANDLE) :: hwnd
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetHwnd_intf(this) RESULT(hwnd)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HANDLE
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(HANDLE) :: hwnd
        !DEC$ ATTRIBUTES STDCALL :: GetHwnd_intf
      END FUNCTION GetHwnd_intf
    END INTERFACE
    
    TYPE(ID2D1HwndRenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetHwnd_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetHwnd, proc)
    hwnd = proc(render_target%object)
    
  END FUNCTION GetHwnd
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1GdiInteropRenderTarget.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1gdiinteroprendertarget_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1gdiinteroprendertarget_IID
    
    !***************************************************************************
    
    d2d1gdiinteroprendertarget_IID = GUID(  &
        INT(Z'E0DB51C3', C_INT32_T),  &
        INT(Z'6F77', C_INT16_T),  &
        INT(Z'4BAE', C_INT16_T),  &
        [ INT(Z'B3', C_SIGNED_CHAR), INT(Z'D5', C_SIGNED_CHAR),  &
          INT(Z'E4', C_SIGNED_CHAR), INT(Z'75', C_SIGNED_CHAR),  &
          INT(Z'09', C_SIGNED_CHAR), INT(Z'B3', C_SIGNED_CHAR),  &
          INT(Z'58', C_SIGNED_CHAR), INT(Z'38', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1gdiinteroprendertarget_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE GetDC(render_target, mode, hdc, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1GdiInteropRenderTarget), INTENT(IN) :: render_target
    INTEGER(D2D1_DC_INITIALIZE_MODE), INTENT(IN) :: mode
    INTEGER(HANDLE), INTENT(OUT) :: hdc
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION GetDC_intf(this, mode, hdc) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_DC_INITIALIZE_MODE
        IMPORT :: HANDLE
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_DC_INITIALIZE_MODE), INTENT(IN) :: mode
        INTEGER(HANDLE), INTENT(OUT) :: hdc
        !DEC$ ATTRIBUTES REFERENCE :: hdc
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: GetDC_intf
      END FUNCTION GetDC_intf
    END INTERFACE
    
    TYPE(ID2D1GdiInteropRenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(GetDC_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetDC, proc)
    hr = proc(render_target%object, mode, hdc)
    
  END SUBROUTINE GetDC
  
  
  !*****************************************************************************
  
  SUBROUTINE ReleaseDC(render_target, hr, update)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1GdiInteropRenderTarget), INTENT(IN) :: render_target
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    ! Temporary definition - should be in some other module, but we are lazy.
    TYPE, BIND(C) :: RECT
      INTEGER(C_LONG) :: left
      INTEGER(C_LONG) :: top
      INTEGER(C_LONG) :: right
      INTEGER(C_LONG) :: bottom
    END TYPE RECT
    
    TYPE(RECT), INTENT(IN), OPTIONAL, TARGET :: update
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION ReleaseDC_intf(this, update) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: update
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: ReleaseDC_intf
      END FUNCTION ReleaseDC_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_update
    
    TYPE(ID2D1GdiInteropRenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(ReleaseDC_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(update)) THEN
      local_update = C_LOC(update)
    ELSE
      local_update = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%ReleaseDC, proc)
    hr = proc(render_target%object, local_update)
    
  END SUBROUTINE ReleaseDC
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for D2D1DCRenderTarget.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1dcrendertarget_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1dcrendertarget_IID
    
    !***************************************************************************
    
    d2d1dcrendertarget_IID = GUID(  &
        INT(Z'1C51BC64', C_INT32_T),  &
        INT(Z'DE61', C_INT16_T),  &
        INT(Z'46FD', C_INT16_T),  &
        [ INT(Z'98', C_SIGNED_CHAR), INT(Z'99', C_SIGNED_CHAR),  &
          INT(Z'63', C_SIGNED_CHAR), INT(Z'A5', C_SIGNED_CHAR),  &
          INT(Z'D8', C_SIGNED_CHAR), INT(Z'F0', C_SIGNED_CHAR),  &
          INT(Z'39', C_SIGNED_CHAR), INT(Z'50', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1dcrendertarget_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE BindDC(render_target, hDC, sub_rect, hr)
    
    USE :: IFWINTY, ONLY: HANDLE, T_RECT
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1DCRenderTarget), INTENT(IN) :: render_target
    INTEGER(HANDLE), INTENT(IN) :: hDC
    TYPE(T_RECT), INTENT(IN) :: sub_rect
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION BindDC_intf(this, hDC, sub_rect) RESULT(hr)
        USE :: IFWINTY, ONLY: HANDLE, T_RECT
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(HANDLE), INTENT(IN) :: hDC
        TYPE(T_RECT), INTENT(IN) :: sub_rect
        !DEC$ ATTRIBUTES REFERENCE :: sub_rect
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: BindDC_intf
      END FUNCTION BindDC_intf
    END INTERFACE
    
    TYPE(ID2D1DCRenderTargetVTbl), POINTER :: ptbl
    PROCEDURE(BindDC_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(render_target), ptbl)
    CALL C_F_PROCPOINTER(ptbl%BindDC, proc)
    hr = proc(render_target%object, hDC, sub_rect)
    
  END SUBROUTINE BindDC
  
  
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  !
  ! Procedures for ID2D1Factory.
  
  
  !*****************************************************************************
  
  FUNCTION d2d1factory_IID()
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_INT32_T, C_INT16_T, C_SIGNED_CHAR
    
    !---------------------------------------------------------------------------
    ! Characteristics
    
    TYPE(GUID) :: d2d1factory_IID
    
    !***************************************************************************
    
    d2d1factory_IID = GUID(  &
        INT(Z'06152247', C_INT32_T),  &
        INT(Z'6F50', C_INT16_T),  &
        INT(Z'465A', C_INT16_T),  &
        [ INT(Z'92', C_SIGNED_CHAR), INT(Z'45', C_SIGNED_CHAR),  &
          INT(Z'11', C_SIGNED_CHAR), INT(Z'8B', C_SIGNED_CHAR),  &
          INT(Z'FD', C_SIGNED_CHAR), INT(Z'3B', C_SIGNED_CHAR),  &
          INT(Z'60', C_SIGNED_CHAR), INT(Z'07', C_SIGNED_CHAR) ] )
    
  END FUNCTION d2d1factory_IID
  
  
  !*****************************************************************************
  
  SUBROUTINE D2D1CreateFactory(factory_type, factory, hr, options)
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    INTEGER(D2D1_FACTORY_TYPE), INTENT(IN) :: factory_type
    TYPE(ID2D1Factory), INTENT(OUT) :: factory
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_FACTORY_OPTIONS), INTENT(IN), OPTIONAL :: options
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    INTERFACE
      FUNCTION D2D1CreateFactory_api( factoryType, riid, pFactoryOptions,  &
          ppIFactory ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_FACTORY_TYPE
        IMPORT :: GUID
        IMPORT :: D2D1_FACTORY_OPTIONS
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        INTEGER(D2D1_FACTORY_TYPE), INTENT(IN) :: factoryType
        TYPE(GUID), INTENT(IN) :: riid
        !DEC$ ATTRIBUTES REFERENCE :: riid
        TYPE(D2D1_FACTORY_OPTIONS), INTENT(IN) :: pFactoryOptions
        !DEC$ ATTRIBUTES REFERENCE :: pFactoryOptions
        TYPE(C_PTR), INTENT(OUT) :: ppIFactory
        !DEC$ ATTRIBUTES REFERENCE :: ppIFactory
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: D2D1CreateFactory_api
        !DEC$ ATTRIBUTES ALIAS:'D2D1CreateFactory' :: D2D1CreateFactory_api
        !DEC$ ATTRIBUTES DECORATE :: D2D1CreateFactory_api
      END FUNCTION D2D1CreateFactory_api
    END INTERFACE
    
    TYPE(D2D1_FACTORY_OPTIONS), PARAMETER :: default_options  &
        = D2D1_FACTORY_OPTIONS(D2D1_DEBUG_LEVEL_NONE)
    
    TYPE(D2D1_FACTORY_OPTIONS) :: local_options
    
    !***************************************************************************
    
    IF (PRESENT(options)) THEN
      local_options = options
    ELSE
      local_options = default_options
    END IF
    
    hr = D2D1CreateFactory_api(  &
        factory_type,  &
        d2d1factory_IID(),  &
        local_options,  &
        factory%object )
    
  END SUBROUTINE D2D1CreateFactory
  
  
  !*****************************************************************************
  
  SUBROUTINE ReloadSystemMetrics(factory, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION ReloadSystemMetrics_intf(this) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: ReloadSystemMetrics_intf
      END FUNCTION ReloadSystemMetrics_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(ReloadSystemMetrics_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%ReloadSystemMetrics, proc)
    hr = proc(factory%object)
    
  END SUBROUTINE ReloadSystemMetrics
  
  
  !*****************************************************************************
  
  SUBROUTINE GetDesktopDpi(factory, dpiX, dpiY)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    REAL(FLOAT), INTENT(OUT) :: dpiX
    REAL(FLOAT), INTENT(OUT) :: dpiY
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      SUBROUTINE GetDesktopDpi_intf(this, dpiX, dpiY)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: FLOAT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        REAL(FLOAT), INTENT(OUT) :: dpiX
        !DEC$ ATTRIBUTES REFERENCE :: dpiX
        REAL(FLOAT), INTENT(OUT) :: dpiY
        !DEC$ ATTRIBUTES REFERENCE :: dpiY
        !DEC$ ATTRIBUTES STDCALL :: GetDesktopDpi_intf
      END SUBROUTINE GetDesktopDpi_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(GetDesktopDpi_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%GetDesktopDpi, proc)
    CALL proc(factory%object, dpiX, dpiY)
    
  END SUBROUTINE GetDesktopDpi
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateRectangleGeometry(factory, rectangle, rectangleGeometry, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    TYPE(D2D1_RECT_F), INTENT(IN) :: rectangle
    TYPE(ID2D1RectangleGeometry), INTENT(OUT) :: rectangleGeometry
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateRectangleGeometry_intf( this, rectangle,  &
          rectangleGeometry) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RECT_F
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_RECT_F), INTENT(IN) :: rectangle
        !DEC$ ATTRIBUTES REFERENCE :: rectangle
        TYPE(C_PTR), INTENT(OUT) :: rectangleGeometry
        !DEC$ ATTRIBUTES REFERENCE :: rectangleGeometry
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateRectangleGeometry_intf
      END FUNCTION CreateRectangleGeometry_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateRectangleGeometry_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateRectangleGeometry, proc)
    hr = proc(factory%object, rectangle, rectangleGeometry%object)
    
  END SUBROUTINE CreateRectangleGeometry
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateRoundedRectangleGeometry( factory, roundedRectangle,  &
      roundedRectangleGeometry, hr )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    TYPE(D2D1_ROUNDED_RECT), INTENT(IN) :: roundedRectangle
    TYPE(ID2D1RoundedRectangleGeometry), INTENT(OUT) :: roundedRectangleGeometry
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateRoundedRectangleGeometry_intf( this, roundedRectangle,  &
          roundedRectangleGeometry) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ROUNDED_RECT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_ROUNDED_RECT), INTENT(IN) :: roundedRectangle
        !DEC$ ATTRIBUTES REFERENCE :: roundedRectangle
        TYPE(C_PTR), INTENT(OUT) :: roundedRectangleGeometry
        !DEC$ ATTRIBUTES REFERENCE :: roundedRectangleGeometry
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateRoundedRectangleGeometry_intf
      END FUNCTION CreateRoundedRectangleGeometry_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateRoundedRectangleGeometry_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateRoundedRectangleGeometry, proc)
    hr = proc(factory%object, roundedRectangle, roundedRectangleGeometry%object)
    
  END SUBROUTINE CreateRoundedRectangleGeometry
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateEllipseGeometry(factory, ellipse, ellipseGeometry, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    TYPE(D2D1_ELLIPSE), INTENT(IN) :: ellipse
    TYPE(ID2D1EllipseGeometry), INTENT(OUT) :: ellipseGeometry
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateEllipseGeometry_intf(this, ellipse, ellipseGeometry)  &
          RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_ELLIPSE
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_ELLIPSE), INTENT(IN) :: ellipse
        !DEC$ ATTRIBUTES REFERENCE :: ellipse
        TYPE(C_PTR), INTENT(OUT) :: ellipseGeometry
        !DEC$ ATTRIBUTES REFERENCE :: ellipseGeometry
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateEllipseGeometry_intf
      END FUNCTION CreateEllipseGeometry_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateEllipseGeometry_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateEllipseGeometry, proc)
    hr = proc(factory%object, ellipse, ellipseGeometry%object)
    
  END SUBROUTINE CreateEllipseGeometry
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateGeometryGroup( factory, fillMode, geometries,  &
      geometryGroup, hr )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER,  &
        C_INT
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    INTEGER(D2D1_FILL_MODE), INTENT(IN) :: fillMode
    CLASS(ID2D1Geometry), INTENT(IN) :: geometries(:)
    TYPE(ID2D1GeometryGroup), INTENT(OUT) :: geometryGroup
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateGeometryGroup_intf( this, fillMode, geometries,  &
          geometriesCount, geometryGroup ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_FILL_MODE
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        INTEGER(D2D1_FILL_MODE), INTENT(IN) :: fillMode
        TYPE(C_PTR), INTENT(IN) :: geometries(*)
        INTEGER(C_INT), INTENT(IN) :: geometriesCount
        TYPE(C_PTR), INTENT(OUT) :: geometryGroup
        !DEC$ ATTRIBUTES REFERENCE :: geometryGroup
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateGeometryGroup_intf
      END FUNCTION CreateGeometryGroup_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateGeometryGroup_intf), POINTER :: proc
    
    ! Copy of the geometry pointers.
    TYPE(C_PTR) :: objects(SIZE(geometries))
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateGeometryGroup, proc)
    
    ! This gives us a contiguous array, which the compiler should do 
    ! for us, but there appear to be issues.
    objects = geometries%object
    
    hr = proc(  &
        factory%object,  &
        fillMode,  &
        objects,  &
        SIZE(objects, KIND=C_INT),  &
        geometryGroup%object )
    
  END SUBROUTINE CreateGeometryGroup
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateTransformedGeometry( factory, sourceGeometry, transform,  &
      transformedGeometry, hr )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    CLASS(ID2D1Geometry), INTENT(IN) :: sourceGeometry
    TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: transform
    TYPE(ID2D1TransformedGeometry), INTENT(OUT) :: transformedGeometry
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateTransformedGeometry_intf( this, sourceGeometry,  &
          transform, transformedGeometry ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_MATRIX_3X2_F
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: sourceGeometry
        TYPE(D2D1_MATRIX_3X2_F), INTENT(IN) :: transform
        !DEC$ ATTRIBUTES REFERENCE :: transform
        TYPE(C_PTR), INTENT(OUT) :: transformedGeometry
        !DEC$ ATTRIBUTES REFERENCE :: transformedGeometry
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateTransformedGeometry_intf
      END FUNCTION CreateTransformedGeometry_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateTransformedGeometry_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateTransformedGeometry, proc)
    hr = proc( factory%object, sourceGeometry%object, transform,  &
        transformedGeometry%object )
    
  END SUBROUTINE CreateTransformedGeometry
  
  
  !*****************************************************************************
  
  SUBROUTINE CreatePathGeometry(factory, pathGeometry, hr)
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    TYPE(ID2D1PathGeometry), INTENT(OUT) :: pathGeometry
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreatePathGeometry_intf(this, pathGeometry) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(OUT) :: pathGeometry
        !DEC$ ATTRIBUTES REFERENCE :: pathGeometry
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreatePathGeometry_intf
      END FUNCTION CreatePathGeometry_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreatePathGeometry_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreatePathGeometry, proc)
    hr = proc(factory%object, pathGeometry%object)
    
  END SUBROUTINE CreatePathGeometry
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateStrokeStyle( factory, strokeStyleProperties, strokeStyle,  &
      hr, dashes )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER,  &
        C_LOC, C_INT
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    TYPE(D2D1_STROKE_STYLE_PROPERTIES), INTENT(IN) :: strokeStyleProperties
    TYPE(ID2D1StrokeStyle), INTENT(OUT) :: strokeStyle
    INTEGER(HRESULT), INTENT(OUT) :: hr
    REAL(FLOAT), INTENT(IN), OPTIONAL, TARGET :: dashes(:)
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateStrokeStyle_intf( this, strokeStyleProperties,  &
          dashes, dashesCount, strokeStyle ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR, C_INT
        IMPORT :: D2D1_STROKE_STYLE_PROPERTIES
        IMPORT :: FLOAT
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_STROKE_STYLE_PROPERTIES) :: strokeStyleProperties
        !DEC$ ATTRIBUTES REFERENCE :: strokeStyleProperties
        TYPE(C_PTR), INTENT(IN) :: dashes
        INTEGER(C_INT), INTENT(IN) :: dashesCount
        TYPE(C_PTR), INTENT(OUT) :: strokeStyle
        !DEC$ ATTRIBUTES REFERENCE :: strokeStyle
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateStrokeStyle_intf
      END FUNCTION CreateStrokeStyle_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateStrokeStyle_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateStrokeStyle, proc)
    IF (PRESENT(dashes)) THEN
      hr = proc(  &
          factory%object,  &
          strokeStyleProperties,  &
          C_LOC(dashes),  &
          SIZE(dashes, KIND=C_INT),  &
          strokeStyle%object )
    ELSE
      hr = proc(  &
          factory%object,  &
          strokeStyleProperties,  &
          C_NULL_PTR,  &
          0_C_INT,  &
          strokeStyle%object )
    END IF
    
  END SUBROUTINE CreateStrokeStyle
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateDrawingStateBlock( factory, drawingStateBlock, hr,  &
      drawingStateDescription, textRenderingParams )
    
    USE :: DirectWrite, ONLY: IDWriteRenderingParams
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER, C_LOC
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    TYPE(ID2D1DrawingStateBlock), INTENT(OUT) :: drawingStateBlock
    INTEGER(HRESULT), INTENT(OUT) :: hr
    TYPE(D2D1_DRAWING_STATE_DESCRIPTION), INTENT(IN), OPTIONAL, TARGET  &
        :: drawingStateDescription
    TYPE(IDWriteRenderingParams), INTENT(IN), OPTIONAL, TARGET  &
        :: textRenderingParams
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateDrawingStateBlock_intf( this, drawingStateDescription,  &
          textRenderingParams, drawingStateBLock ) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RENDER_TARGET_PROPERTIES
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: drawingStateDescription
        TYPE(C_PTR), INTENT(IN) :: textRenderingParams
        TYPE(C_PTR), INTENT(OUT) :: drawingStateBlock
        !DEC$ ATTRIBUTES REFERENCE :: drawingStateBlock
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateDrawingStateBlock_intf
      END FUNCTION CreateDrawingStateBlock_intf
    END INTERFACE
    
    TYPE(C_PTR) :: local_description
    TYPE(C_PTR) :: local_params
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateDrawingStateBlock_intf), POINTER :: proc
    
    !***************************************************************************
    
    IF (PRESENT(drawingStateDescription)) THEN
      local_description = C_LOC(drawingStateDescription)
    ELSE
      local_description = C_NULL_PTR
    END IF
    
    IF (PRESENT(textRenderingParams)) THEN
      local_params = C_LOC(textRenderingParams)
    ELSE
      local_params = C_NULL_PTR
    END IF
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateDrawingStateBlock, proc)
    hr = proc(  &
        factory%object,  &
        local_description,  &
        local_params,  &
        drawingStateBlock%object )
    
  END SUBROUTINE CreateDrawingStateBlock
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateWicBitmapRenderTarget( factory, renderTargetProperties,  &
      renderTarget, hr )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    TYPE(D2D1_RENDER_TARGET_PROPERTIES), INTENT(IN) :: renderTargetProperties
    TYPE(ID2D1RenderTarget), INTENT(OUT), TARGET :: renderTarget
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateWicBitmapRenderTarget_intf( this,  &
          renderTargetProperties, renderTarget) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RENDER_TARGET_PROPERTIES
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_RENDER_TARGET_PROPERTIES) :: renderTargetProperties
        !DEC$ ATTRIBUTES REFERENCE :: renderTargetProperties
        TYPE(C_PTR), INTENT(OUT) :: renderTarget
        !DEC$ ATTRIBUTES REFERENCE :: renderTarget
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateWicBitmapRenderTarget_intf
      END FUNCTION CreateWicBitmapRenderTarget_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateWicBitmapRenderTarget_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateWicBitmapRenderTarget, proc)
    hr = proc(  &
        factory%object,  &
        renderTargetProperties,  &
        renderTarget%object )
    
  END SUBROUTINE CreateWicBitmapRenderTarget
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateHwndRenderTarget( factory, renderTargetProperties,  &
        hwndRenderTargetProperties, hwndRenderTarget, hr )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    TYPE(D2D1_RENDER_TARGET_PROPERTIES), INTENT(IN) :: renderTargetProperties
    TYPE(D2D1_HWND_RENDER_TARGET_PROPERTIES), INTENT(IN)  &
        :: hwndRenderTargetProperties
    TYPE(ID2D1HwndRenderTarget), INTENT(OUT), TARGET :: hwndRenderTarget
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateHwndRenderTarget_intf( this, renderTargetProperties,  &
          hwndRenderTargetProperties, hwndRenderTarget) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RENDER_TARGET_PROPERTIES
        IMPORT :: D2D1_HWND_RENDER_TARGET_PROPERTIES
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_RENDER_TARGET_PROPERTIES) :: renderTargetProperties
        !DEC$ ATTRIBUTES REFERENCE :: renderTargetProperties
        TYPE(D2D1_HWND_RENDER_TARGET_PROPERTIES) :: hwndRenderTargetProperties
        !DEC$ ATTRIBUTES REFERENCE :: hwndRenderTargetProperties
        TYPE(C_PTR), INTENT(OUT) :: hwndRenderTarget
        !DEC$ ATTRIBUTES REFERENCE :: hwndRenderTarget
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateHwndRenderTarget_intf
      END FUNCTION CreateHwndRenderTarget_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateHwndRenderTarget_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateHwndRenderTarget, proc)
    hr = proc(  &
        factory%object,  &
        renderTargetProperties,  &
        hwndRenderTargetProperties,  &
        hwndRenderTarget%object )
    
  END SUBROUTINE CreateHwndRenderTarget
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateDxgiSurfaceRenderTarget( factory, dxgiSurf,  &
      renderTargetProperties, renderTarget, hr )
    
    USE DirectX, ONLY: IDXGISurface
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    TYPE(IDXGISurface), INTENT(IN) :: dxgiSurf
    TYPE(D2D1_RENDER_TARGET_PROPERTIES), INTENT(IN) :: renderTargetProperties
    TYPE(ID2D1RenderTarget), INTENT(OUT), TARGET :: renderTarget
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateDxgiSurfaceRenderTarget_intf( this, dxgiSurface,  &
          renderTargetProperties, renderTarget) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RENDER_TARGET_PROPERTIES
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(C_PTR), INTENT(IN) :: dxgiSurface
        TYPE(D2D1_RENDER_TARGET_PROPERTIES) :: renderTargetProperties
        !DEC$ ATTRIBUTES REFERENCE :: renderTargetProperties
        TYPE(C_PTR), INTENT(OUT) :: renderTarget
        !DEC$ ATTRIBUTES REFERENCE :: RenderTarget
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateDxgiSurfaceRenderTarget_intf
      END FUNCTION CreateDxgiSurfaceRenderTarget_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateDxgiSurfaceRenderTarget_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateDxgiSurfaceRenderTarget, proc)
    hr = proc(  &
        factory%object,  &
        dxgiSurf%object,  &
        renderTargetProperties,  &
        renderTarget%object )
    
  END SUBROUTINE CreateDxgiSurfaceRenderTarget
  
  
  !*****************************************************************************
  
  SUBROUTINE CreateDCRenderTarget( factory, renderTargetProperties,  &
      dcRenderTarget, hr )
    
    USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_F_POINTER, C_F_PROCPOINTER
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    CLASS(ID2D1Factory), INTENT(IN) :: factory
    TYPE(D2D1_RENDER_TARGET_PROPERTIES), INTENT(IN) :: renderTargetProperties
    TYPE(ID2D1DCRenderTarget), INTENT(OUT), TARGET :: dcRenderTarget
    INTEGER(HRESULT), INTENT(OUT) :: hr
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    ABSTRACT INTERFACE
      FUNCTION CreateDCRenderTarget_intf( this, renderTargetProperties,  &
          dcRenderTarget) RESULT(hr)
        USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_PTR
        IMPORT :: D2D1_RENDER_TARGET_PROPERTIES
        IMPORT :: HRESULT
        IMPLICIT NONE
        !-----------------------------------------------------------------------
        TYPE(C_PTR), INTENT(IN) :: this
        TYPE(D2D1_RENDER_TARGET_PROPERTIES) :: renderTargetProperties
        !DEC$ ATTRIBUTES REFERENCE :: renderTargetProperties
        TYPE(C_PTR), INTENT(OUT) :: dcRenderTarget
        !DEC$ ATTRIBUTES REFERENCE :: dcRenderTarget
        INTEGER(HRESULT) :: hr
        !DEC$ ATTRIBUTES STDCALL :: CreateDCRenderTarget_intf
      END FUNCTION CreateDCRenderTarget_intf
    END INTERFACE
    
    TYPE(ID2D1FactoryVTbl), POINTER :: ptbl
    PROCEDURE(CreateDCRenderTarget_intf), POINTER :: proc
    
    !***************************************************************************
    
    CALL C_F_POINTER(VTable(factory), ptbl)
    CALL C_F_PROCPOINTER(ptbl%CreateDCRenderTarget, proc)
    hr = proc(  &
        factory%object,  &
        renderTargetProperties,  &
        dcRenderTarget%object )
    
  END SUBROUTINE CreateDCRenderTarget
  
END MODULE Direct2D

